@@ -26,6 +26,8 @@
  */
 package javajs.util;
 
+import org.jmol.viewer.JC;
+
 import javajs.api.EigenInterface;
 import javajs.api.Interface;
 
@@ -89,78 +91,6 @@ final public class Measure {
     return (asDegrees ? torsion / radiansPerDegree : torsion);
   }
 
-  /**
-   * This method calculates measures relating to two points in space 
-   * with related quaternion frame difference. It is used in Jmol for
-   * calculating straightness and many other helical quantities.
-   * 
-   * @param a
-   * @param b
-   * @param dq
-   * @return  new T3[] { pt_a_prime, n, r, P3.new3(theta, pitch, residuesPerTurn), pt_b_prime };
-   */
-  public static T3[] computeHelicalAxis(P3 a, P3 b, Quat dq) {
-    
-    //                b
-    //           |   /|
-    //           |  / |
-    //           | /  |
-    //           |/   c
-    //         b'+   / \
-    //           |  /   \      Vcb = Vab . n
-    //         n | /     \d    Vda = (Vcb - Vab) / 2
-    //           |/theta  \
-    //         a'+---------a
-    //                r 
-
-    V3 vab = new V3();
-    vab.sub2(b, a);
-    /*
-     * testing here to see if directing the normal makes any difference -- oddly
-     * enough, it does not. When n = -n and theta = -theta vab.n is reversed,
-     * and that magnitude is multiplied by n in generating the A'-B' vector.
-     * 
-     * a negative angle implies a left-handed axis (sheets)
-     */
-    float theta = dq.getTheta();
-    V3 n = dq.getNormal();
-    float v_dot_n = vab.dot(n);
-    if (Math.abs(v_dot_n) < 0.0001f)
-      v_dot_n = 0;
-    V3 va_prime_d = new V3();
-    va_prime_d.cross(vab, n);
-    if (va_prime_d.dot(va_prime_d) != 0)
-      va_prime_d.normalize();
-    V3 vda = new V3();
-    V3 vcb = V3.newV(n);
-    if (v_dot_n == 0)
-      v_dot_n = PT.FLOAT_MIN_SAFE; // allow for perpendicular axis to vab
-    vcb.scale(v_dot_n);
-    vda.sub2(vcb, vab);
-    vda.scale(0.5f);
-    va_prime_d.scale(theta == 0 ? 0 : (float) (vda.length() / Math.tan(theta
-        / 2 / 180 * Math.PI)));
-    V3 r = V3.newV(va_prime_d);
-    if (theta != 0)
-      r.add(vda);
-    P3 pt_a_prime = P3.newP(a);
-    pt_a_prime.sub(r);
-    // already done this. ??
-    if (v_dot_n != PT.FLOAT_MIN_SAFE)
-      n.scale(v_dot_n);
-    // must calculate directed angle:
-    P3 pt_b_prime = P3.newP(pt_a_prime);
-    pt_b_prime.add(n);
-    theta = computeTorsion(a, pt_a_prime, pt_b_prime, b, true);
-    if (Float.isNaN(theta) || r.length() < 0.0001f)
-      theta = dq.getThetaDirectedV(n); // allow for r = 0
-    // anything else is an array
-    float residuesPerTurn = Math.abs(theta == 0 ? 0 : 360f / theta);
-    float pitch = Math.abs(v_dot_n == PT.FLOAT_MIN_SAFE ? 0 : n.length()
-        * (theta == 0 ? 1 : 360f / theta));
-    return new T3[] { pt_a_prime, n, r, P3.new3(theta, pitch, residuesPerTurn), pt_b_prime };
-  }
-
   public static P4 getPlaneThroughPoints(T3 pointA,
                                               T3 pointB,
                                               T3 pointC, V3 vNorm,
@@ -265,7 +195,7 @@ final public class Measure {
     retNorm.normalize();
     if (dist > 0)
       retNorm.scale(-1);
-    retPtProj.scaleAdd2(-dist, retNorm, pt);
+    retPtProj.scaleAdd2(Math.abs(dist), retNorm, pt);
     return dist;
   }
 
@@ -1052,4 +982,76 @@ final public class Measure {
     return cpts;
   }
 
+  /**
+   * This method calculates measures relating to two points in space 
+   * with related quaternion frame difference. It is used in Jmol for
+   * calculating straightness and many other helical quantities.
+   * 
+   * @param a
+   * @param b
+   * @param dq
+   * @return  new T3[] { pt_a_prime, n, r, P3.new3(theta, pitch, residuesPerTurn), pt_b_prime };
+   */
+  public static T3[] computeHelicalAxis(P3 a, P3 b, Quat dq) {
+    
+    //                b
+    //           |   /|
+    //           |  / |
+    //           | /  |
+    //           |/   c
+    //         b'+   / \
+    //           |  /   \      Vcb = Vab . n
+    //         n | /     \d    Vda = (Vcb - Vab) / 2
+    //           |/theta  \
+    //         a'+---------a
+    //                r 
+  
+    V3 vab = new V3();
+    vab.sub2(b, a);
+    /*
+     * testing here to see if directing the normal makes any difference -- oddly
+     * enough, it does not. When n = -n and theta = -theta vab.n is reversed,
+     * and that magnitude is multiplied by n in generating the A'-B' vector.
+     * 
+     * a negative angle implies a left-handed axis (sheets)
+     */
+    float theta = dq.getTheta();
+    V3 n = dq.getNormal();
+    float v_dot_n = vab.dot(n);
+    if (Math.abs(v_dot_n) < 0.0001f)
+      v_dot_n = 0;
+    V3 va_prime_d = new V3();
+    va_prime_d.cross(vab, n);
+    if (va_prime_d.dot(va_prime_d) != 0)
+      va_prime_d.normalize();
+    V3 vda = new V3();
+    V3 vcb = V3.newV(n);
+    if (v_dot_n == 0)
+      v_dot_n = JC.FLOAT_MIN_SAFE; // allow for perpendicular axis to vab
+    vcb.scale(v_dot_n);
+    vda.sub2(vcb, vab);
+    vda.scale(0.5f);
+    va_prime_d.scale(theta == 0 ? 0 : (float) (vda.length() / Math.tan(theta
+        / 2 / 180 * Math.PI)));
+    V3 r = V3.newV(va_prime_d);
+    if (theta != 0)
+      r.add(vda);
+    P3 pt_a_prime = P3.newP(a);
+    pt_a_prime.sub(r);
+    // already done this. ??
+    if (v_dot_n != JC.FLOAT_MIN_SAFE)
+      n.scale(v_dot_n);
+    // must calculate directed angle:
+    P3 pt_b_prime = P3.newP(pt_a_prime);
+    pt_b_prime.add(n);
+    theta = computeTorsion(a, pt_a_prime, pt_b_prime, b, true);
+    if (Float.isNaN(theta) || r.length() < 0.0001f)
+      theta = dq.getThetaDirectedV(n); // allow for r = 0
+    // anything else is an array
+    float residuesPerTurn = Math.abs(theta == 0 ? 0 : 360f / theta);
+    float pitch = Math.abs(v_dot_n == JC.FLOAT_MIN_SAFE ? 0 : n.length()
+        * (theta == 0 ? 1 : 360f / theta));
+    return new T3[] { pt_a_prime, n, r, P3.new3(theta, pitch, residuesPerTurn), pt_b_prime };
+  }
+
 }
