@@ -528,8 +528,7 @@ public class PT {
     return lines;
   }
 
-  public final static float FLOAT_MIN_SAFE = 2E-45f; 
-  // Float.MIN_VALUE (1.45E-45) is not reliable with JavaScript because of the float/double difference there
+  
   
   /// general static string-parsing class ///
 
@@ -681,11 +680,10 @@ public class PT {
    * @param precision
    * @param alignLeft
    * @param zeroPad
-   * @param allowOverflow IGNORED
    * @return formatted string
    */
   public static String formatD(double value, int width, int precision,
-                              boolean alignLeft, boolean zeroPad, boolean allowOverflow) {
+                              boolean alignLeft, boolean zeroPad) {
     return formatS(DF.formatDecimal((float)value, -1 - precision), width, 0, alignLeft, zeroPad);
   }
 
@@ -1207,7 +1205,7 @@ public class PT {
               zeroPad);
         else if (!Double.isNaN(doubleT)) // 'e'
           strLabel += formatD(doubleT, width, precision - 1, alignLeft,
-              zeroPad, true);
+              zeroPad);
         if (doOne)
           break;
       } catch (IndexOutOfBoundsException ioobe) {
@@ -1264,7 +1262,7 @@ public class PT {
                 Float.NaN, Double.NaN, true);
             break;
           case 'f':
-            strFormat = formatString(strFormat, "f", null, ((Float) values[o])
+            strFormat = formatString(strFormat, "f", null, ((Number) values[o])
                 .floatValue(), Double.NaN, true);
             break;
           case 'i':
@@ -1275,7 +1273,7 @@ public class PT {
             break;
           case 'd':
             strFormat = formatString(strFormat, "e", null, Float.NaN,
-                ((Double) values[o]).doubleValue(), true);
+                ((Number) values[o]).doubleValue(), true);
             break;
           case 'p':
             T3 pVal = (T3) values[o];
@@ -1328,7 +1326,6 @@ public class PT {
         }
         return rep(strFormat, "%%", "%");
       } catch (Exception e) {
-        //
       }
     System.out.println("TextFormat.sprintf error " + list + " " + strFormat);
     return rep(strFormat, "%", "?");
@@ -1551,6 +1548,10 @@ public class PT {
     pt.z = Math.round(pt.z * f) / f;
   }
   
+  public static float fixFloat(float d, double f) {
+    return (float) (Math.round(d * f) / f);
+  }
+  
   public static double fixDouble(double d, double f) {
     return Math.round(d * f) / f;
   }
