@@ -32,7 +32,7 @@ import org.jmol.adapter.smarter.AtomSetCollection;
 import org.jmol.adapter.smarter.AtomSetCollectionReader;
 import org.jmol.adapter.smarter.Structure;
 import org.jmol.api.JmolAdapter;
-import org.jmol.api.SymmetryInterface;
+import org.jmol.adapter.smarter.XtalSymmetry.FileSymmetry;
 import org.jmol.c.STR;
 import org.jmol.util.Escape;
 import org.jmol.util.Logger;
@@ -156,6 +156,9 @@ public class PdbReader extends AtomSetCollectionReader {
   private boolean isCourseGrained;
   private boolean isbiomol;
 
+  private float cryst1;
+  private String fileSgName;
+
   final private static String lineOptions = 
    "ATOM    " + //0
    "HETATM  " + //1
@@ -421,7 +424,7 @@ public class PdbReader extends AtomSetCollectionReader {
     checkUnitCellParams();
     if (!isCourseGrained)
       connectAll(maxSerial, isConnectStateBug);
-    SymmetryInterface symmetry;
+    FileSymmetry symmetry;
     if (vBiomolecules != null && vBiomolecules.size() > 0 && asc.ac > 0) {
       asc.setCurrentModelInfo("biomolecules", vBiomolecules);
       setBiomoleculeAtomCounts();
@@ -433,7 +436,7 @@ public class PdbReader extends AtomSetCollectionReader {
       }
     }
     if (vTlsModels != null) {
-      symmetry = (SymmetryInterface) getInterface("org.jmol.symmetry.Symmetry");
+      symmetry = asc.newFileSymmetry();
       int n = asc.atomSetCount;
       if (n == vTlsModels.size()) {
         for (int i = n; --i >= 0;)
@@ -460,7 +463,7 @@ public class PdbReader extends AtomSetCollectionReader {
       sgName = fileSgName;
       fractionalizeCoordinates(true);
       asc.setModelInfoForSet("biosymmetry", null, asc.iSet);
-      asc.checkSpecial = false;
+      checkNearAtoms = false;
     }
     if (latticeCells != null && latticeCells[0] != 0)
       addJmolScript("unitcell;axes on;axes unitcell;");
@@ -482,14 +485,18 @@ public class PdbReader extends AtomSetCollectionReader {
   }
 
   private void checkUnitCellParams() {
+    if (isbiomol && (unitCellParams == null || Double.isNaN(unitCellParams[0]))) {
+        setUnitCell(1, 1, 1, 90, 90, 90);
+        addSpaceGroupName("P1");
+    }
     if (iHaveUnitCell) {
-      asc.setCurrentModelInfo("unitCellParams", unitCellParams);
+      asc.setCurrentModelInfo(JC.INFO_UNIT_CELL_PARAMS, unitCellParams);
       if (sgName != null)
         asc.setCurrentModelInfo("spaceGroup", sgName);
     }
   }
 
-  private void checkForResidualBFactors(SymmetryInterface symmetry) {
+  private void checkForResidualBFactors(FileSymmetry symmetry) {
     Atom[] atoms = asc.atoms;
     boolean isResidual = false;
      for (int i = asc.ac; --i >= 0;) {
@@ -1305,7 +1312,7 @@ public class PdbReader extends AtomSetCollectionReader {
       return;
 
     String structureID = line.substring(11, 15).trim();
-    int serialID = parseIntRange(line, 7, 10);
+    String serialID = line.substring(7, 10).trim();
     int startChainID = vwr.getChainID(line.substring(startChainIDIndex, startChainIDIndex + 1), true);
     int startSequenceNumber = parseIntRange(line, startIndex, startIndex + 4);
     char startInsertionCode = line.charAt(startIndex + 4);
@@ -1386,24 +1393,26 @@ public class PdbReader extends AtomSetCollectionReader {
     // No special-position atoms in mmCIF files, because there will
     // be no center of symmetry, no rotation-inversions, 
     // no atom-centered rotation axes, and no mirror or glide planes. 
-    asc.checkSpecial = !isPDB;
+    checkNearAtoms = !isPDB;
     setModelPDB(isPDB);
     nUNK = nRes = 0;
     currentGroup3 = null;
   }
 
-  private float cryst1;
-  private String fileSgName;
   private void cryst1() throws Exception {
     float a = cryst1 = getFloat(6, 9);
     if (a == 1)
       a = Float.NaN; // 1 for a means no unit cell
     setUnitCell(a, getFloat(15, 9), getFloat(24, 9), getFloat(33,
         7), getFloat(40, 7), getFloat(47, 7));
+    addSpaceGroupName(PT.parseTrimmedRange(line, 55, 66));
+  }
+
+  private void addSpaceGroupName(String name) {
     if (isbiomol)
       doConvertToFractional = false;
     if (sgName == null || sgName.equals("unspecified!"))
-      setSpaceGroupName(PT.parseTrimmedRange(line, 55, 66));
+      setSpaceGroupName(name);
     fileSgName = sgName;
   }
 
@@ -1849,7 +1858,7 @@ public class PdbReader extends AtomSetCollectionReader {
    * @param symmetry 
    */
   @SuppressWarnings("unchecked")
-  private void setTlsGroups(int iGroup, int iModel, SymmetryInterface symmetry) {
+  private void setTlsGroups(int iGroup, int iModel, FileSymmetry symmetry) {
 
     // TLS.groupCount   Integer
     // TLS.groups       List of Map
@@ -1942,11 +1951,11 @@ public class PdbReader extends AtomSetCollectionReader {
 
   private final float[] dataT = new float[8];
 
-  private static final float RAD_PER_DEG = (float) (Math.PI / 180);
-  private static final float _8PI2_ = (float) (8 * Math.PI * Math.PI);
+  private static final double RAD_PER_DEG = (Math.PI / 180);
+  private static final double _8PI2_ = (8 * Math.PI * Math.PI);
   private Map<Atom, float[]>tlsU;
   
-  private void setTlsTensor(Atom atom, Map<String, Object> group, SymmetryInterface symmetry) {
+  private void setTlsTensor(Atom atom, Map<String, Object> group, FileSymmetry symmetry) {
     P3 origin = (P3) group.get("origin");
     if (Float.isNaN(origin.x))
       return;
@@ -1961,9 +1970,9 @@ public class PdbReader extends AtomSetCollectionReader {
     // just factor degrees-to-radians into x, y, and z rather
     // than create all new matrices
 
-    float x = (atom.x - origin.x) * RAD_PER_DEG;
-    float y = (atom.y - origin.y) * RAD_PER_DEG;
-    float z = (atom.z - origin.z) * RAD_PER_DEG;
+    float x = (float) ((atom.x - origin.x) * RAD_PER_DEG);
+    float y = (float) ((atom.y - origin.y) * RAD_PER_DEG);
+    float z = (float) ((atom.z - origin.z) * RAD_PER_DEG);
 
     float xx = x * x;
     float yy = y * y;
@@ -1988,7 +1997,7 @@ public class PdbReader extends AtomSetCollectionReader {
 
     float[] anisou = new float[8];
 
-    float bresidual = (Float.isNaN(atom.bfactor) ? 0 : atom.bfactor / _8PI2_);
+    float bresidual = (Float.isNaN(atom.bfactor) ? 0 : (float) (atom.bfactor / _8PI2_));
 
     anisou[0] /*u11*/= dataT[0] + L[1][1] * zz + L[2][2] * yy - 2 * L[1][2]
         * yz + 2 * S[1][0] * z - 2 * S[2][0] * y;
