@@ -87,7 +87,7 @@ import javajs.util.V3;
 
 public class CastepReader extends AtomSetCollectionReader {
 
-  private static final float RAD_TO_DEG = (float) (180.0 / Math.PI);
+  private static final double RAD_TO_DEG = 180.0 / Math.PI;
 
   private String[] tokens;
 
@@ -470,9 +470,9 @@ public class CastepReader extends AtomSetCollectionReader {
     a = abc[0].length();
     b = abc[1].length();
     c = abc[2].length();
-    alpha = (abc[1].angle(abc[2]) * RAD_TO_DEG);
-    beta = (abc[2].angle(abc[0]) * RAD_TO_DEG);
-    gamma = (abc[0].angle(abc[1]) * RAD_TO_DEG);
+    alpha = (float) (abc[1].angle(abc[2]) * RAD_TO_DEG);
+    beta = (float) (abc[2].angle(abc[0]) * RAD_TO_DEG);
+    gamma = (float) (abc[0].angle(abc[1]) * RAD_TO_DEG);
   }
 
   private void readPositionsFrac() throws Exception {
