@@ -102,9 +102,8 @@ class AtomIterator implements JmolAdapterAtomIterator {
 	
 	@Override
   public int getFormalCharge() {
-		return atom.formalCharge;
-	}
-
+    return (atom.formalCharge == Integer.MIN_VALUE ? 0 : atom.formalCharge);
+  }
 	
 	@Override
   public float getPartialCharge() {
@@ -204,4 +203,15 @@ class AtomIterator implements JmolAdapterAtomIterator {
 		return atom;
 	}
 
+	 @Override
+	  public int getElement() {  
+	    return getElementNumber() & 0x7F;
+	  }
+
+	  @Override
+	  public int getIsotope() {
+	    return getElementNumber() >> 7;
+	  }
+
+
 }
