@@ -1,7 +1,7 @@
 /* $RCSfile$
  * $Author: hansonr $
- * $Date: 2022-12-11 13:26:30 -0600 (Sun, 11 Dec 2022) $
- * $Revision: 22490 $
+ * $Date: 2024-03-15 17:42:21 -0500 (Fri, 15 Mar 2024) $
+ * $Revision: 22610 $
  *
  * Copyright (C) 2003-2005  Miguel, Jmol Development, www.jmol.org
  *
@@ -35,7 +35,8 @@ import javajs.util.P3;
 import javajs.util.V3;
 
 import org.jmol.api.Interface;
-import org.jmol.api.SymmetryInterface;
+import org.jmol.api.JmolAdapter;
+import org.jmol.adapter.smarter.XtalSymmetry.FileSymmetry;
 import javajs.util.BS;
 
 import org.jmol.util.BSUtil;
@@ -220,8 +221,7 @@ public class AtomSetCollection {
 
     // append to bsAtoms if necessary (CIF reader molecular mode)
     if (collection.bsAtoms != null) {
-      if (bsAtoms == null)
-        bsAtoms = new BS();
+      getBSAtoms(0);
       for (int i = collection.bsAtoms
           .nextSetBit(0); i >= 0; i = collection.bsAtoms.nextSetBit(i + 1))
         bsAtoms.set(existingAtomsCount + i);
@@ -705,7 +705,7 @@ public class AtomSetCollection {
   }
 
   public int baseSymmetryAtomCount;
-  public boolean checkLatticeOnly;
+  public boolean crystalReaderLatticeOpsOnly;
 
   public XtalSymmetry xtalSymmetry;
 
@@ -716,23 +716,26 @@ public class AtomSetCollection {
     return xtalSymmetry;
   }
 
-  public SymmetryInterface getSymmetry() {
+  public FileSymmetry getSymmetry() {
     return getXSymmetry().getSymmetry();
   }
 
-  public SymmetryInterface setSymmetry(SymmetryInterface symmetry) {
+  public FileSymmetry setSymmetry(FileSymmetry symmetry) {
     return (symmetry == null ? null : getXSymmetry().setSymmetry(symmetry));
   }
 
-  public void setTensors() {
+  public FileSymmetry newFileSymmetry() {
+    return getXSymmetry().newFileSymmetry();
+  }
+
+
+ public void setTensors() {
     if (haveAnisou)
       getXSymmetry().setTensors();
   }
 
   int bondIndex0;
 
-  public boolean checkSpecial = true;
-
   public Map<String, Atom> atomSymbolicMap = new Hashtable<String, Atom>();
 
   public boolean haveUnitCell;
@@ -1210,5 +1213,58 @@ public class AtomSetCollection {
       atomSetCount--;
   }
 
+  /**
+   * Create bsAtoms if it is null, and set bits if desired.
+   * 
+   * @param n  if created only, -1 to set all atoms, 0 for no setting, otherwise, set [0,n)
+   * @return bsAtoms
+   */
+  public BS getBSAtoms(int n) {
+    if (bsAtoms == null) {
+      bsAtoms = new BS();
+      if (n != 0) 
+        bsAtoms.setBits(0, (n < 0 ? ac : n));
+    }
+    return bsAtoms;
+  }
+
+  /**
+   * Add a full set of atoms to BSAtoms if they are not already indicated there.
+   * @param iSet
+   */
+  public void setBSAtomsForSet(int iSet) {
+    if (bsAtoms != null) {
+      if (iSet < 0)
+        iSet = this.iSet;
+      int pt = atomSetAtomIndexes[iSet];
+      if (bsAtoms.nextSetBit(pt) < 0) {
+        int n = atomSetAtomCounts[iSet];
+        bsAtoms.setBits(pt, pt + n);
+      }
+    }
+  }
+
+  public void fix2Stereo() {
+    getBSAtoms(-1);
+    for (int i = bondCount; --i >= 0;) {
+      Bond b = bonds[i];
+      if (atoms[b.atomIndex2].elementSymbol.equals("H")
+          && b.order != JmolAdapter.ORDER_STEREO_NEAR
+          && b.order != JmolAdapter.ORDER_STEREO_FAR
+          && atoms[b.atomIndex1].elementSymbol.equals("C")
+          ) {
+        bsAtoms.clear(b.atomIndex2);
+      } else if (atoms[b.atomIndex1].elementSymbol.equals("H")
+          && atoms[b.atomIndex2].elementSymbol.equals("C")
+          ) {
+        // includes backward C-H wedge/hash here, which makes no sense.
+        bsAtoms.clear(b.atomIndex1);
+      }
+    }
+
+    // TODO
+    
+  }
+
 
 }
