@@ -1,7 +1,7 @@
 /* $RCSfile$
  * $Author: hansonr $
- * $Date: 2023-01-05 08:42:06 -0600 (Thu, 05 Jan 2023) $
- * $Revision: 22500 $
+ * $Date: 2024-04-16 21:43:34 -0500 (Tue, 16 Apr 2024) $
+ * $Revision: 22613 $
  *
  * Copyright (C) 2003-2005  Miguel, Jmol Development, www.jmol.org
  *
@@ -33,9 +33,7 @@ import java.util.StringTokenizer;
 
 import org.jmol.api.Interface;
 import org.jmol.util.Logger;
-import org.jmol.viewer.FileManager;
 import org.jmol.viewer.JC;
-import org.jmol.viewer.Viewer;
 
 import javajs.api.GenericBinaryDocument;
 import javajs.util.LimitedLineReader;
@@ -46,19 +44,19 @@ public class Resolver {
 
   private final static String classBase = "org.jmol.adapter.readers.";
   private final static String[] readerSets = new String[] {
-    "cif.", ";Cif;Cif2;MMCif;MMTF;MagCif",
+    "cif.", ";Cif;Cif2;MMCif;MMTF;MagCif;BCIF;",
     "molxyz.", ";Mol3D;Mol;Xyz;",
     "more.", ";AFLOW;BinaryDcd;CDX;Gromacs;Jcampdx;MdCrd;MdTop;Mol2;TlsDataOnly;",
     "quantum.", ";Adf;Csf;Dgrid;GamessUK;GamessUS;Gaussian;GaussianFchk;GaussianWfn;Jaguar;" +
                  "Molden;MopacGraphf;GenNBO;NWChem;Psi;Qchem;QCJSON;" +
-                 "WebMO;Orca;MO;", // MO is for XmlMolpro 
+                 "WebMO;Orca;MO;Ams;", // MO is for XmlMolpro 
     "pdb.", ";Pdb;Pqr;P2n;JmolData;",
     "pymol.", ";PyMOL;",
     "simple.", ";Alchemy;Ampac;Cube;FoldingXyz;GhemicalMM;HyperChem;Jme;JSON;Mopac;MopacArchive;Tinker;Input;FAH;",
     "spartan.", ";Spartan;SpartanSmol;Odyssey;",
     "xtal.", ";Abinit;Aims;Bilbao;Castep;Cgd;Crystal;Dmol;Espresso;Gulp;Jana;Magres;Shelx;Siesta;VaspOutcar;" +
-             "VaspPoscar;Wien2k;Xcrysden;PWmat;Optimade;",
-    "xml.",  ";XmlChemDraw;XmlArgus;XmlCml;XmlChem3d;XmlMolpro;XmlOdyssey;XmlXsd;XmlVasp;XmlQE;",
+             "VaspPoscar;Wien2k;Xcrysden;PWmat;Optimade;Cmdf;",
+    "xml.",  ";XmlCdx;XmlArgus;XmlCml;XmlChem3d;XmlMolpro;XmlOdyssey;XmlXsd;XmlVasp;XmlQE;",
   };
   
 
@@ -107,23 +105,23 @@ public class Resolver {
                                         Map<String, Object> htParams, int ptFile)
       throws Exception {
     String readerName;
-    fullName = FileManager.fixDOSName(fullName);
+    fullName = fixDOSName(fullName);
     String errMsg = null;
-    if (type == null) {
-      type = SmarterJmolAdapter.getFileTypefromFilter((String) htParams.get("filter"));
-    }      
+    if (type == null && htParams != null) {
+      type = Resolver.getFileTypefromFilter((String) htParams.get("filter"));
+    }
     if (type != null) {
       readerName = getReaderFromType(type);
       if (readerName == null)
         readerName = getReaderFromType("Xml" + type);
       if (readerName == null)
-        errMsg = "unrecognized file format type " + type;
+        errMsg = "unrecognized file format type " + type + " file=" + fullName;
       else
-        Logger.info("The Resolver assumes " + readerName);
+        Logger.info("The Resolver assumes " + readerName + " file=" + fullName);
     } else {
       readerName = determineAtomSetCollectionReader(readerOrDocument, htParams);
       if (readerName.charAt(0) == '\n') {
-        type = (String) htParams.get("defaultType");
+        type = (htParams == null ? null : (String) htParams.get("defaultType"));
         if (type != null) {
           // allow for MDTOP to specify default MDCRD
           type = getReaderFromType(type);
@@ -163,7 +161,7 @@ public class Resolver {
     String className = null;
     String err = null;
     className = getReaderClassBase(readerName);
-    if ((rdr = (AtomSetCollectionReader) Interface.getInterface(className, (Viewer) htParams.get("vwr"), "reader")) == null) {
+    if ((rdr = (AtomSetCollectionReader) Interface.getInterface(className, null, "reader")) == null) {
       err = JC.READER_NOT_FOUND  + className;
       Logger.error(err);
       return err;
@@ -176,7 +174,7 @@ public class Resolver {
       type = "Xml" + type.substring(0, type.length() - 5);
     }
     type = ";" + type.toLowerCase() + ";";
-    if (";zmatrix;cfi;c;vfi;v;mnd;jag;adf;gms;g;gau;mp;nw;orc;pqs;qc;".indexOf(type) >= 0)
+    if (";zmatrix;cfi;c;vfi;v;mnd;jag;gms;g;gau;mp;nw;orc;pqs;qc;".indexOf(type) >= 0)
       return "Input";
 
     String set;
@@ -233,7 +231,7 @@ public class Resolver {
    * @return readerName or a few lines, if requested, or null
    * @throws Exception
    */
-  private static String determineAtomSetCollectionReader(Object readerOrDocument,
+  public static String determineAtomSetCollectionReader(Object readerOrDocument,
                                                          Map<String, Object> htParams)
       throws Exception {
 
@@ -343,9 +341,15 @@ public class Resolver {
 
   private static byte[] cdxMagic = new byte[] { 'V', 'j', 'C', 'D' };
   
+  private static byte[] cmdfMagic = new byte[] { 'C', 'M', 'D', 'F' };
+
   public static String getBinaryType(InputStream inputStream) {
-    return (Rdr.isPickleS(inputStream) ? "PyMOL" : (Rdr.getMagic(inputStream, 1)[0] & 0xDE) == 0xDE ? "MMTF" : 
-      bytesMatch(Rdr.getMagic(inputStream, 4), cdxMagic) ? "CDX" : null);
+    byte[] magic4 = null;
+    return (Rdr.isPickleS(inputStream) ? "PyMOL" 
+        : (Rdr.getMagic(inputStream, 1)[0] & 0xFF) == 0xDE ? "MMTF" 
+        : (Rdr.getMagic(inputStream, 10)[9] & 0xFF) == 0xB6 ? "BCIF" 
+        : bytesMatch((magic4 = Rdr.getMagic(inputStream, 4)), cdxMagic) ? "CDX" 
+        : bytesMatch(magic4, cmdfMagic) ? "Cmdf" : null);
   }
 
   private static boolean bytesMatch(byte[] a, byte[] b) {
@@ -698,7 +702,7 @@ public class Resolver {
   { "Cgd", "EDGE ", "edge " };
 
   private final static String[] shelxLineStartRecords =
-  { "Shelx", "TITL ", "ZERR ", "LATT ", "SYMM ", "CELL " };
+  { "Shelx", "TITL ", "ZERR ", "LATT ", "SYMM ", "CELL ", "TITL\t" };
 
   private final static String[] ghemicalMMLineStartRecords =
   { "GhemicalMM", "!Header mm1gp", "!Header gpr" };
@@ -792,7 +796,7 @@ public class Resolver {
       return "XmlChem3d";
     }
     if (header.indexOf("CDXML") >= 0) {
-      return "XmlChemDraw";
+      return "XmlCdx";
     }
     if (header.indexOf("arguslab") >= 0) {
       return "XmlArgus";
@@ -850,8 +854,11 @@ public class Resolver {
   { "Spartan", "Spartan", "converted archive file" };  // very old Spartan files; sparchive files
 
   private final static String[] adfContainsRecords =
-  { "Adf", "Amsterdam Density Functional" };
+  { "Adf", "Amsterdam Density Functional"};
   
+  private final static String[] amsContainsRecords =
+  { "Ams", "Amsterdam Modeling Suite"};
+
   private final static String[] psiContainsRecords =
   { "Psi", "    PSI  3", "PSI3:"};
  
@@ -931,7 +938,8 @@ public class Resolver {
     espressoContainsRecords, siestaContainsRecords, xcrysDenContainsRecords,
     mopacArchiveContainsRecords,abinitContainsRecords,gaussianFchkContainsRecords,
     inputContainsRecords, aflowContainsRecords, magCifContainsRecords, 
-    qcJsonContainsRecords, optimadeContainsRecords, orcaContainsRecords, jsonArrayContainsRecords
+    qcJsonContainsRecords, optimadeContainsRecords, orcaContainsRecords, jsonArrayContainsRecords,
+    amsContainsRecords
   };
 
   private final static String[][] jsonContainsRecords =
@@ -1010,5 +1018,14 @@ public class Resolver {
     return null;
   }
 
+  public static String getFileTypefromFilter(String filter) {
+    int pt = (filter == null ? -1 : filter.toLowerCase().indexOf("filetype"));
+    return (pt < 0 ? null : filter.substring(pt + 8, (filter+ ";").indexOf(";", pt)).replace('=', ' ').trim());
+  }
+
+  public static String fixDOSName(String fileName) {
+    return (fileName.indexOf(":\\") >= 0 ? fileName.replace('\\', '/') : fileName);
+  }
+
 }
 
