@@ -251,8 +251,10 @@ public final class ScriptEditor extends JDialog implements JmolScriptEditorInter
   
   @Override
   public void setVisible(boolean b) {
+    boolean isChange = (b != isVisible());
     super.setVisible(b);
-    vwr.getProperty("DATA_API", "scriptEditorState", b ? Boolean.TRUE : Boolean.FALSE);
+    if (isChange)
+      vwr.getProperty("DATA_API", "scriptEditorState", b ? Boolean.TRUE : Boolean.FALSE);
     if (b)
       editor.grabFocus();
   }
