@@ -20,6 +20,7 @@ package org.jmol.inchi;
 
 import java.io.BufferedReader;
 import java.io.IOException;
+import java.io.InputStream;
 import java.io.StringReader;
 import java.util.Hashtable;
 import java.util.Map;
@@ -53,6 +54,7 @@ import net.sf.jniinchi.JniInchiAtom;
 import net.sf.jniinchi.JniInchiBond;
 import net.sf.jniinchi.JniInchiInput;
 import net.sf.jniinchi.JniInchiInputInchi;
+import net.sf.jniinchi.JniInchiOutput;
 import net.sf.jniinchi.JniInchiOutputStructure;
 import net.sf.jniinchi.JniInchiStereo0D;
 import net.sf.jniinchi.JniInchiStructure;
@@ -65,9 +67,9 @@ public class InChIJNI implements JmolInChI {
   }
 
   @Override
-  public String getInchi(Viewer vwr, BS atoms, String molData, String options) {
+  public String getInchi(Viewer vwr, BS atoms, Object molData, String options) {
     try {
-      if (atoms == null ? molData == null : atoms.cardinality() == 0)
+      if (atoms == null ? molData == null : atoms.isEmpty())
         return "";
       boolean getKey = false;
       if (options == null)
@@ -81,8 +83,8 @@ public class InChIJNI implements JmolInChI {
         inchi = options.substring(10);
         options = lc = "";
       }
-      if (molData != null && molData.startsWith("InChI=")) {
-        inchi = molData;
+      if (molData instanceof String && ((String) molData).startsWith("InChI=")) {
+        inchi = (String) molData;
         if (!getSmiles) {
           options = lc.replace("structure", "");
           getKey = true;
@@ -106,12 +108,16 @@ public class InChIJNI implements JmolInChI {
           if (atoms == null) {
             in.setStructure(struc = newJniInchiStructure(vwr, molData));
           } else {
-            in.setStructure(struc = newJniInchiStructure(vwr, atoms));
+            in.setStructure(struc = newJniInchiStructureBS(vwr, atoms));
           }
           if (getStructure) {
             return toString(struc);
           }
-          inchi = JniInchiWrapper.getInchi(in).getInchi();
+          JniInchiOutput out = JniInchiWrapper.getInchi(in);
+          String msg = out.getMessage();
+          if (msg != null)
+            System.err.println(msg);
+          inchi = out.getInchi();
         }
       }
       if (getSmiles || getStructure) {
@@ -122,7 +128,7 @@ public class InChIJNI implements JmolInChI {
             : getStructure(struc));
       }
       return (getKey ? JniInchiWrapper.getInchiKey(inchi).getKey() : inchi);
-    } catch (Exception e) {
+    } catch (Throwable e) {
 
       System.out.println(e);
 
@@ -147,7 +153,7 @@ public class InChIJNI implements JmolInChI {
    * @param bsAtoms
    * @return a structure for JniInput
    */
-  private static JniInchiStructure newJniInchiStructure(Viewer vwr,
+  private static JniInchiStructure newJniInchiStructureBS(Viewer vwr,
                                                         BS bsAtoms) {
     JniInchiStructure mol = new JniInchiStructure();
     JniInchiAtom[] atoms = new JniInchiAtom[bsAtoms.cardinality()];
@@ -156,7 +162,7 @@ public class InChIJNI implements JmolInChI {
     for (int pt = 0, i = bsAtoms.nextSetBit(0); i >= 0; i = bsAtoms
         .nextSetBit(i + 1)) {
       Atom a = vwr.ms.at[i];
-      String sym = a.getElementSymbol();
+      String sym = a.getElementSymbolIso(false);
       int iso = a.getIsotopeNumber();
       if (a.getElementNumber() == 1) {
         sym = "H"; // in case this is D
@@ -174,25 +180,8 @@ public class InChIJNI implements JmolInChI {
       // but oddly still do not get generated, probably because the carbons are then too strange?
       INCHI_BOND_TYPE order = getOrder(Math.max(bond.isPartial() ? 1 : 0, bond.getCovalentOrder()));
       if (order != null) {
-        INCHI_BOND_STEREO stereo;
-        switch (bond.getBondType()) {
-        case Edge.BOND_STEREO_FAR:
-          stereo = INCHI_BOND_STEREO.SINGLE_1DOWN;
-          break;
-        case Edge.BOND_STEREO_NEAR:
-          stereo = INCHI_BOND_STEREO.SINGLE_1UP;
-          break;
-        case Edge.BOND_STEREO_EITHER:
-          // this will generate a stereo ?
-          stereo = INCHI_BOND_STEREO.SINGLE_1EITHER;
-          break;
-        default:
-            stereo = INCHI_BOND_STEREO.NONE;
-            break;
-        }
-        JniInchiBond b = new JniInchiBond(atoms[map[bond.getAtomIndex1()]],
-            atoms[map[bond.getAtomIndex2()]], order, stereo);
-        mol.addBond(b);
+        mol.addBond(new JniInchiBond(atoms[map[bond.getAtomIndex1()]],
+            atoms[map[bond.getAtomIndex2()]], order, getStereo(bond.getBondType())));
       }
     }
     return mol;
@@ -204,13 +193,19 @@ public class InChIJNI implements JmolInChI {
    * 
    * @param vwr
    * @param molData
+   *        String or InputStream
    * @return a structure for JniInput
    */
   private static JniInchiStructure newJniInchiStructure(Viewer vwr,
-                                                        String molData) {
+                                                        Object molData) {
     JniInchiStructure mol = new JniInchiStructure();
-    BufferedReader r = new BufferedReader(new StringReader(molData));
+    Object r = null;
     try {
+      if (molData instanceof String) {
+        r = new BufferedReader(new StringReader((String) molData));
+      } else if (molData instanceof InputStream) {
+        r = molData;
+      }
       Map<String, Object> htParams = new Hashtable<String, Object>();
       JmolAdapter adapter = vwr.getModelAdapter();
       Object atomSetReader = adapter.getAtomSetCollectionReader("String", null,
@@ -225,32 +220,82 @@ public class InChIJNI implements JmolInChI {
       JmolAdapterBondIterator bi = adapter.getBondIterator(asc);
       JniInchiAtom[] atoms = new JniInchiAtom[asc.getAtomSetAtomCount(0)];
       int n = 0;
+      Map<Object, Integer> atomMap = new Hashtable<Object, Integer>();
       while (ai.hasNext() && n < atoms.length) {
         P3 p = ai.getXYZ();
-        JniInchiAtom a = new JniInchiAtom(p.x, p.y, p.z,
-            Elements.elementSymbolFromNumber(ai.getElementNumber()));
+        int atno = ai.getElementNumber();
+        if (atno <= 0) {
+          System.err.println("InChIJNI atom " + p + " index " + ai.getUniqueID()
+              + " is not a valid element");
+          return null;
+        }
+        String sym = Elements.elementSymbolFromNumber(atno);
+        JniInchiAtom a = new JniInchiAtom(p.x, p.y, p.z, sym);
         a.setCharge(ai.getFormalCharge());
         mol.addAtom(a);
+        atomMap.put(ai.getUniqueID(), Integer.valueOf(n));
         atoms[n++] = a;
-      }
+          }
+      int nb = 0;
       while (bi.hasNext()) {
-        INCHI_BOND_TYPE order = getOrder(bi.getEncodedOrder());
-        if (order != null)
-          mol.addBond(new JniInchiBond(
-              atoms[((Integer) bi.getAtomUniqueID1()).intValue()],
-              atoms[((Integer) bi.getAtomUniqueID2()).intValue()], order));
+        int jmolOrder = bi.getEncodedOrder();
+        INCHI_BOND_TYPE order = getOrder(jmolOrder);
+        if (order != null) {
+          Integer id1 = atomMap.get(bi.getAtomUniqueID1());
+          Integer id2 = atomMap.get(bi.getAtomUniqueID2());
+          if (id1 == null || id2 == null) {
+            System.err.println("InChIJNI bond " + nb + "has null atom "
+                + (id1 == null ? bi.getAtomUniqueID1() : "") + " "
+                + (id2 == null ? bi.getAtomUniqueID2() : ""));
+            return null;
+          }
+          JniInchiAtom a = atoms[id1.intValue()];
+          JniInchiAtom b = atoms[id2.intValue()];
+          if (a == null || b == null) {
+            System.err
+                .println("InChIJNI bond " + nb + "has null atom: " + a + "/" + b
+                    + " for ids " + id1 + " " + id2 + " and " + n + " atoms");
+            return null;
+          }
+          mol.addBond(new JniInchiBond(a, b, order, getStereo(jmolOrder)));
+          nb++;
+        }
       }
+    } catch (Throwable t) {
+      t.printStackTrace();
+      System.err.println(t.toString());
     } finally {
       try {
-        r.close();
+        if (r instanceof BufferedReader) {
+          ((BufferedReader) r).close();
+        } else {
+          ((InputStream) r).close();
+        }
       } catch (IOException e) {
       }
     }
     return mol;
   }
 
-  private static INCHI_BOND_TYPE getOrder(int order) {
-    switch (order) {
+  private static INCHI_BOND_STEREO getStereo(int jmolOrder) {
+    switch (jmolOrder) {
+    case Edge.BOND_STEREO_FAR:
+      return INCHI_BOND_STEREO.SINGLE_1DOWN;
+    case Edge.BOND_STEREO_NEAR:
+      return INCHI_BOND_STEREO.SINGLE_1UP;
+    case Edge.BOND_STEREO_EITHER:
+      // this will generate a stereo ?
+      return INCHI_BOND_STEREO.SINGLE_1EITHER;
+    default:
+      return INCHI_BOND_STEREO.NONE;
+    }
+  }
+
+  private static INCHI_BOND_TYPE getOrder(int jmolOrder) {
+    switch (jmolOrder) {
+    case Edge.BOND_STEREO_EITHER:
+    case Edge.BOND_STEREO_FAR:
+    case Edge.BOND_STEREO_NEAR:
     case Edge.BOND_COVALENT_SINGLE:
       return INCHI_BOND_TYPE.SINGLE;
     case Edge.BOND_COVALENT_DOUBLE:
@@ -354,7 +399,7 @@ public class InChIJNI implements JmolInChI {
       int i1 = map.get(an[1]).getIndex();
       int i2 = map.get(an[2]).getIndex();
       int i3 = map.get(an[3]).getIndex();
-//      System.out.println(aatoms[i0] + "\n" +  aatoms[i1] + "\n" +  aatoms[i2] + "\n" +  aatoms[i3]);
+//System.out.println(aatoms[i0] + "\n" +  aatoms[i1] + "\n" +  aatoms[i2] + "\n" +  aatoms[i3]);
       boolean isEven = (sd.getParity() == INCHI_PARITY.EVEN);
       INCHI_STEREOTYPE type = sd.getStereoType();
       switch (type) {
@@ -493,7 +538,7 @@ public class InChIJNI implements JmolInChI {
   private static Integer getIntKey(int i, int iA, int j) {
     Integer v =  Integer.valueOf((Math.min(i, j) << 24) +
         (iA << 12) +  Math.max(i, j));
-//    System.out.println("getIntKey " + i + " " + iA + " "+ j + " " + 
+//System.out.println("getIntKey " + i + " " + iA + " "+ j + " " + 
 //        (v == null ? null : Integer.toHexString(v.intValue())));
     return v;
   }
