@@ -219,8 +219,8 @@ public class JvxlData {
 
   public void setSurfaceInfo(P4 thePlane, P3 mapLattice, int nSurfaceInts, String surfaceData) {
     jvxlSurfaceData = surfaceData;
-    if (jvxlSurfaceData.indexOf("--") == 0)
-      jvxlSurfaceData = jvxlSurfaceData.substring(2);
+//    if (jvxlSurfaceData.indexOf("--") == 0)
+//      jvxlSurfaceData = jvxlSurfaceData.substring(2);
     jvxlPlane = thePlane;
     this.mapLattice = mapLattice;
     this.nSurfaceInts = nSurfaceInts;
