@@ -48,29 +48,33 @@ import org.jmol.util.Edge;
 import org.jmol.util.Logger;
 
 import org.jmol.script.T;
+import org.jmol.viewer.JC;
 import org.jmol.viewer.JmolAsyncException;
 import org.jmol.viewer.Viewer;
 
 public class Minimizer {
 
+  public static int staticID = 0;
+  
+  public int id;
   public Viewer vwr;
   public Atom[] atoms;
   public Bond[] bonds;
   public int rawBondCount;
-  
+  public BS bsAtoms;
+
+  public Lst<MMConstraint> constraints;
+
   public MinAtom[] minAtoms;
   public MinBond[] minBonds;
   public MinAngle[] minAngles;
   public MinTorsion[] minTorsions;
-  public MinPosition[] minPositions;
   
   public BS bsMinFixed;
   private int ac;
   private int bondCount;
   private int[] atomMap; 
  
-  public double[] partialCharges;
-  
   private int steps = 50;
   private double crit = 1e-3;
 
@@ -79,22 +83,29 @@ public class Minimizer {
   private ForceField pFF;
   private String ff = "UFF";
   private BS bsTaint, bsSelected;
-  public BS bsAtoms;
   private BS bsFixedDefault;
   private BS bsFixed;
-  
-  public Lst<MMConstraint> constraints;
-  public BS bsBasis;
+
+  private boolean modelkitMinimizing;
+  private BS bsBasis;
   
   private boolean isSilent;
- 
+
   public Minimizer() {
+    id = (++staticID) * 100;
   }
 
   
   public Minimizer setProperty(String propertyName, Object value) {
-    switch (("ff        " + "cancel    " + "clear     " + "constraint"
-        +    "fixed     " + "stop      " + "vwr    ").indexOf(propertyName)) {
+    switch ((
+        "ff        " + // 0
+        "cancel    " + // 10
+        "clear     " + // 20
+        "constraint" + // 30
+        "fixed     " + // 40
+        "stop      " + // 50
+        "vwr    "
+        ).indexOf(propertyName)) {
     case 0:
       // UFF or MMFF
       if (!ff.equals(value)) {
@@ -116,6 +127,8 @@ public class Minimizer {
       break;
     case 40:
       bsFixedDefault = (BS) value;
+      if (bsFixedDefault != null && bsFixedDefault.cardinality() == 0)
+        bsFixedDefault = null;
       break;
     case 50:
       stopMinimization(true);
@@ -130,11 +143,27 @@ public class Minimizer {
   public boolean minimize(int steps, double crit, BS bsSelected, BS bsFixed,
                           BS bsBasis, int flags, String ff)
       throws JmolAsyncException {
-    this.bsBasis = bsBasis;
-    trustRadius = (bsBasis == null ? 0.3 : 0.03);
+    id++;
     isSilent = ((flags & Viewer.MIN_SILENT) == Viewer.MIN_SILENT);
     isQuick = (ff.indexOf("2D") >= 0
         || (flags & Viewer.MIN_QUICK) == Viewer.MIN_QUICK);
+    modelkitMinimizing = (bsBasis != null && vwr.getModelkitPropertySafely(JC.MODELKIT_MINIMIZING) == Boolean.TRUE);
+    if (bsBasis != null) {
+      if (bsFixed == null)
+        bsFixed = new BS();
+      vwr.getMotionFixedAtoms(null, bsFixed);
+      bsBasis.andNot(bsFixed);
+      bsFixed.or(bsSelected);
+      bsFixed.andNot(bsBasis);
+      if (bsBasis.isEmpty()) {
+        report(" symmetry-based minimization failed -- all atoms are fully constrained", false);
+        return false;
+      }
+      int n = bsBasis.cardinality();
+      report(" symmetry-based minimization for " + n + " atom" + (n == 1 ? "" : "s"), false);
+    }
+    this.bsBasis = bsBasis;
+    trustRadius = (bsBasis == null ? 0.3 : 0.01);
     boolean haveFixed = ((flags
         & Viewer.MIN_HAVE_FIXED) == Viewer.MIN_HAVE_FIXED);
     BS bsXx = ((flags & Viewer.MIN_XX) == Viewer.MIN_XX ? new BS() : null);
@@ -154,12 +183,12 @@ public class Minimizer {
     try {
       setEnergyUnits();
 
-      // if the user indicated minimize ... FIX ... or we don't have any defualt,
+      // if the user indicated minimize ... FIX ... or we don't have any default,
       // use the bsFixed coming in here, which is set to "nearby and in frame" in that case. 
       // and if something is fixed, then AND it with "nearby and in frame" as well.
       if (!haveFixed && bsFixedDefault != null)
         bsFixed.and(bsFixedDefault);
-      if (minimizationOn)
+      if (minimizing)
         return false;
       ForceField pFF0 = pFF;
       getForceField(ff);
@@ -167,11 +196,14 @@ public class Minimizer {
         Logger.error(GT.o(GT.$("Could not get class for force field {0}"), ff));
         return false;
       }
-      Logger.info("minimize: initializing " + pFF.name + " (steps = " + steps
+      Logger.info("minimize: " + id + " initializing " + pFF.name + " (steps = " + steps
           + " criterion = " + crit + ")"
                 + " silent=" + isSilent 
                 + " quick=" + isQuick 
                 + " fixed=" + haveFixed 
+                + " bsSelected=" + bsSelected
+                + " bsFixed=" + bsFixed
+                + " bsFixedDefault=" + bsFixedDefault
                 + " Xx=" + (bsXx != null)
               + " ...");
       if (bsSelected.nextSetBit(0) < 0) {
@@ -185,10 +217,10 @@ public class Minimizer {
         if (atoms[i].getElementNumber() == 0) {
           if (bsXx == null) {
             bsAtoms.clear(i);
-            Logger.info("minimize: Ignoring Xx for atomIndex=" + i);
+            Logger.info("minimize: " + id + " Ignoring Xx for atomIndex=" + i);
           } else {
             bsXx.set(i);
-            Logger.info("minimize: Setting Xx to fluorine for atomIndex=" + i);
+            Logger.info("minimize: " + id + " Setting Xx to fluorine for atomIndex=" + i);
             atoms[i].setAtomicAndIsotopeNumber(9); // Xx -> fluorine
           }
         }
@@ -203,8 +235,8 @@ public class Minimizer {
         sameAtoms = false;
       if (!sameAtoms)
         pFF.clear();
-      if ((!sameAtoms || !BSUtil.areEqual(bsFixed, this.bsFixed))
-          && !setupMinimization()) {
+      boolean isSame = (sameAtoms && BSUtil.areEqual(bsFixed, this.bsFixed));
+      if (!setupMinimization(bsFixed, isSame)) {
         clear();
         return false;
       }
@@ -213,9 +245,6 @@ public class Minimizer {
         BSUtil.andNot(bsTaint, bsFixed);
         vwr.ms.setTaintedAtoms(bsTaint, AtomCollection.TAINT_COORD);
       }
-      if (bsFixed != null)
-        this.bsFixed = bsFixed;
-      setAtomPositions();
 
       if (constraints != null)
         for (int i = constraints.size(); --i >= 0;)
@@ -250,6 +279,9 @@ public class Minimizer {
     if (propertyName.equals("log")) {
       return (pFF == null ? "" : pFF.getLogData());
     }
+    if (propertyName.equals("fixed")) {
+      return bsFixedDefault;
+    }
     return null;
   }
   
@@ -303,7 +335,6 @@ public class Minimizer {
     minBonds = null;
     minAngles = null;
     minTorsions = null;
-    partialCharges = null;
     coordSaved = null;
     atomMap = null;
     bsTaint = null;
@@ -322,8 +353,12 @@ public class Minimizer {
     units = (s.equalsIgnoreCase("kcal") ? "kcal" : "kJ");
   }
 
-  private boolean setupMinimization() throws JmolAsyncException {
-
+  private boolean setupMinimization(BS bsFixed, boolean isSame)
+      throws JmolAsyncException {
+    if (isSame) {
+      setAtomPositions();
+      return true;
+    }
     coordSaved = null;
     atomMap = new int[atoms.length];
     minAtoms = new MinAtom[ac];
@@ -340,16 +375,17 @@ public class Minimizer {
           atom.z }, ac);
       minAtoms[pt].sType = atom.getAtomName();
     }
-
+    if (bsFixed != null)
+      this.bsFixed = bsFixed;
     Logger.info(GT.i(GT.$("{0} atoms will be minimized."), ac));
-    Logger.info("minimize: getting bonds...");
+    Logger.info("minimize:  " + id + " getting bonds...");
     bonds = vwr.ms.bo;
     rawBondCount = vwr.ms.bondCount;
     getBonds();
-    Logger.info("minimize: getting angles...");
+    Logger.info("minimize:  " + id + " getting angles...");
     getAngles();
-    Logger.info("minimize: getting torsions...");
-    getTorsions();
+    Logger.info("minimize:  " + id + " getting torsions...");
+    getTorsions(ff.startsWith("MMFF"));
     return setModel(bsElements);
   }
   
@@ -358,6 +394,7 @@ public class Minimizer {
       //pFF.log("could not setup force field " + ff);
       Logger.error(GT.o(GT.$("could not setup force field {0}"), ff));
       if (ff.startsWith("MMFF")) {
+        report(" MMFF not applicable", false);
         getForceField("UFF");
         //pFF.log("could not setup force field " + ff);
         return setModel(bsElements);        
@@ -370,13 +407,14 @@ public class Minimizer {
   private void setAtomPositions() {
     for (int i = 0; i < ac; i++)
       minAtoms[i].set();
-    bsMinFixed = null;
-    if (bsFixed != null) {
+    if (bsFixed == null || bsFixed.cardinality() == 0) {
+      bsMinFixed = null;
+    } else {
       bsMinFixed = new BS();
-      for (int i = bsAtoms.nextSetBit(0), pt = 0; i >= 0; i = bsAtoms
-          .nextSetBit(i + 1), pt++)
-        if (bsFixed.get(i))
-          bsMinFixed.set(pt);
+      for (int i = 0; i < ac; i++) {
+        if (bsFixed.get(minAtoms[i].atom.i))
+          bsMinFixed.set(i);
+      }
     }
   }
 
@@ -438,11 +476,6 @@ public class Minimizer {
             vAngles.addLast(new MinAngle(new int[] { ia, ib, ic, i,
                 minAtoms[ib].getBondIndex(j)}));
             minAtoms[ia].bsVdw.clear(ic);
-/*            System.out.println (" " 
-                + minAtoms[ia].getIdentity() + " -- " 
-                + minAtoms[ib].getIdentity() + " -- " 
-                + minAtoms[ic].getIdentity());
-*/
           }
       }
       if (minAtoms[ia].nBonds > 1) {
@@ -453,12 +486,6 @@ public class Minimizer {
                 .addLast(new MinAngle(new int[] { ic, ia, ib, minAtoms[ia].getBondIndex(j),
                     i}));
             minAtoms[ic].bsVdw.clear(ib);
-/*
-            System.out.println ("a " 
-                + minAtoms[ic].getIdentity() + " -- " 
-                + minAtoms[ia].getIdentity() + " -- " 
-                + minAtoms[ib].getIdentity());
-*/            
           }
       }
     }
@@ -466,7 +493,7 @@ public class Minimizer {
     Logger.info(minAngles.length + " angles");
   }
 
-  public void getTorsions() {
+  public void getTorsions(boolean isMMFF) {
     Lst<MinTorsion> vTorsions = new  Lst<MinTorsion>();
     int id;
     // extend all angles a-b-c by one, but only
@@ -485,12 +512,8 @@ public class Minimizer {
             vTorsions.addLast(new MinTorsion(new int[] { ia, ib, ic, id, 
                 angle[ForceField.ABI_IJ], angle[ForceField.ABI_JK],
                 minAtoms[ic].getBondIndex(j) }));
+            if (isMMFF)
               minAtoms[Math.min(ia, id)].bs14.set(Math.max(ia, id));
-/*            System.out.println("t " + minAtoms[ia].getIdentity() + " -- "
-                + minAtoms[ib].getIdentity() + " -- "
-                + minAtoms[ic].getIdentity() + " -- "
-                + minAtoms[id].getIdentity());
-*/
           }
         }
       }
@@ -502,12 +525,8 @@ public class Minimizer {
             vTorsions.addLast(new MinTorsion(new int[] { ic, ib, ia, id, 
                 angle[ForceField.ABI_JK], angle[ForceField.ABI_IJ],
                 minAtoms[ia].getBondIndex(j) }));
+            if (isMMFF)
             minAtoms[Math.min(ic, id)].bs14.set(Math.max(ic, id));
-/*            System.out.println("t " + minAtoms[ic].getIdentity() + " -- "
-                + minAtoms[ib].getIdentity() + " -- "
-                + minAtoms[ia].getIdentity() + " -- "
-                + minAtoms[id].getIdentity());
-*/
           }
         }
       }
@@ -533,7 +552,8 @@ public class Minimizer {
       if (!isQuick)
         vwr.setStringProperty("_minimizationForceField", ff);
     }
-    //Logger.info("minimize: forcefield = " + pFF);
+    report(" forcefield is " + ff, false);
+    pFF.setNth(vwr.getInt(T.minimizationreportsteps));
     return pFF;
   }
   
@@ -541,10 +561,10 @@ public class Minimizer {
    * Minimization thead support
    ****************************************************************/
 
-  private boolean minimizationOn;
+  private boolean minimizing;
   
   public boolean minimizationOn() {
-    return minimizationOn;
+    return minimizing;
   }
 
   private MinimizationThread minimizationThread;
@@ -556,7 +576,7 @@ public class Minimizer {
   }
 
   private void setMinimizationOn(boolean minimizationOn) {
-    this.minimizationOn = minimizationOn;
+    this.minimizing = minimizationOn;
     if (!minimizationOn) {
       if (minimizationThread != null) {
         minimizationThread = null;
@@ -578,6 +598,12 @@ public class Minimizer {
     reportEnergy();
     vwr.setStringProperty("_minimizationStatus", "calculate");
     vwr.notifyMinimizationStatus();
+    if (bsBasis != null) {
+      //report(pFF.getLogData(), true);
+      // to delete this model
+      vwr.getModelkit(false).minimizeEnd(null, true);
+    }
+  
   }
   
   private void reportEnergy() {
@@ -587,7 +613,7 @@ public class Minimizer {
   
   public boolean startMinimization() {
    try {
-      Logger.info("minimize: startMinimization");
+      Logger.info("minimize:  " + id + " startMinimization");
       vwr.setIntProperty("_minimizationStep", 0);
       vwr.setStringProperty("_minimizationStatus", "starting");
       vwr.setFloatProperty("_minimizationEnergy", 0);
@@ -598,35 +624,42 @@ public class Minimizer {
       reportEnergy();
       saveCoordinates();
     } catch (Exception e) {
+      //e.printStackTrace();
       Logger.error("minimization error vwr=" + vwr + " pFF = " + pFF);
       return false;
     }
-    minimizationOn = true;
+    minimizing = true;
     return true;
   }
 
   
   public boolean stepMinimization() {
-    if (!minimizationOn)
+    if (!minimizing)
       return false;
     boolean doRefresh = (!isSilent && vwr.getBooleanProperty("minimizationRefresh"));
     vwr.setStringProperty("_minimizationStatus", "running");
-    boolean going = pFF.steepestDescentTakeNSteps(1);
+    boolean going = pFF.steepestDescentTakeNSteps(1, bsBasis != null);
     int currentStep = pFF.getCurrentStep();
     vwr.setIntProperty("_minimizationStep", currentStep);
+    if (doRefresh) {
+      vwr.refresh(Viewer.REFRESH_SYNC_MASK, "minimization step " + currentStep);
+    }
     reportEnergy();
     vwr.setFloatProperty("_minimizationEnergyDiff", pFF.toUserUnits(pFF.getEnergyDiff()));
     vwr.notifyMinimizationStatus();
     if (doRefresh) {
-      updateAtomXYZ();
-      vwr.refresh(Viewer.REFRESH_SYNC_MASK, "minimization step " + currentStep);
+      if (!modelkitMinimizing)
+        updateAtomXYZ(false);
+      vwr.refresh(3, "minimization step " + currentStep);
     }
     return going;
   }
 
   
-  public void endMinimization() {
-    updateAtomXYZ();
+  public void endMinimization(boolean normalFinish) {
+    System.out.println("minimization: " + id + " end minimizing=" + minimizing + " normal=" + normalFinish);
+    if (!minimizing)
+      return;
     setMinimizationOn(false);
     if (pFF == null) {
       System.out.println("pFF was null");
@@ -634,14 +667,16 @@ public class Minimizer {
       boolean failed = pFF.detectExplosion();
       if (failed)
         restoreCoordinates();
+      else
+        updateAtomXYZ(true);
       vwr.setIntProperty("_minimizationStep", pFF.getCurrentStep());
       reportEnergy();
-      vwr.setStringProperty("_minimizationStatus", (failed ? "failed" : "done"));
+      vwr.setStringProperty("_minimizationStatus", (failed ? "failed" : normalFinish ? "done" : "stopped"));
       vwr.notifyMinimizationStatus();
       vwr.refresh(Viewer.REFRESH_SYNC_MASK, "minimize:done"
           + (failed ? " EXPLODED" : "OK"));
     }
-    Logger.info("minimize: endMinimization");
+    Logger.info("minimize:  " + id + " endMinimization complete");
   }
 
   double[][] coordSaved;
@@ -660,53 +695,65 @@ public class Minimizer {
     for (int i = 0; i < ac; i++) 
       for (int j = 0; j < 3; j++)
         minAtoms[i].coord[j] = coordSaved[i][j];
-    updateAtomXYZ();
+    updateAtomXYZ(true);
   }
 
   public void stopMinimization(boolean coordAreOK) {
-    if (!minimizationOn)
+    if (!minimizing)
       return;
-    setMinimizationOn(false);
     if (coordAreOK)
-      endMinimization();
+      endMinimization(false);
     else
       restoreCoordinates();
+    setMinimizationOn(false);
   }
   
   private P3 p = new P3();
   
-  void updateAtomXYZ() {
-    if (steps <= 0)
+  public void updateAtomXYZ(boolean isEnd) {
+    if (steps <= 0 || pFF != null && pFF.getCurrentStep() == 0)
       return;
-    if (bsBasis == null) {
+    if (!modelkitMinimizing) {
       for (int i = 0; i < ac; i++) {
         MinAtom minAtom = minAtoms[i];
-        minAtom.atom.set((float) minAtom.coord[0], (float) minAtom.coord[1],
+        if (bsFixed == null || !bsFixed.get(minAtom.atom.i))
+          minAtom.atom.set((float) minAtom.coord[0], (float) minAtom.coord[1],
             (float) minAtom.coord[2]);
       }
+      isEnd = true;
     } else {
       Atom a;
-      // only accept the minimization for the basis atoms
+      // only accept the minimization for the basis atoms (which won't be fixed)
       // ModelKit will transfer the corresponding symmetry changes to the other atoms
+      boolean doUpdateMinAtoms = false;
+      MinAtom minAtom = minAtoms[0];
       for (int i = 0; i < ac; i++) {
-        if (bsFixed != null && bsFixed.get(i))
+        minAtom = minAtoms[i];
+        if (bsMinFixed != null && bsMinFixed.get(i))
           continue;
-        MinAtom minAtom = minAtoms[i];
-        if (bsBasis.get((a = minAtom.atom).i)) {
-          p.set((float) minAtom.coord[0], (float) minAtom.coord[1],
-              (float) minAtom.coord[2]);
-          vwr.getModelkit(false).moveConstrained(a.i, p, true, true);
+        a = minAtom.atom;
+        p.set((float) minAtom.coord[0], (float) minAtom.coord[1],
+            (float) minAtom.coord[2]);
+
+        if (vwr.getModelkit(false).moveMinConstrained(a.i, p, bsAtoms) > 0) {
+          doUpdateMinAtoms = true;
         }
       }
       // now transfer back all atom coordinates
-    for (int i = 0; i < ac; i++) {
-      MinAtom minAtom = minAtoms[i];
-        minAtom.coord[0] = (a = minAtom.atom).x;
-        minAtom.coord[1] = a.y;
-        minAtom.coord[2] = a.z;
+      if (doUpdateMinAtoms) {
+        for (int i = 0; i < ac; i++) {
+          minAtom = minAtoms[i];
+          minAtom.coord[0] = (a = minAtom.atom).x;
+          minAtom.coord[1] = a.y;
+          minAtom.coord[2] = a.z;
+        }
       }
+      vwr.getModelkit(false).minimizeEnd(bsBasis, isEnd);
+    }
+    if (isEnd) {
+      //report(pFF.getLogData(), true);
+      vwr.refreshMeasures(false);
     }
-    vwr.refreshMeasures(false);
   }
 
   private void minimizeWithoutThread() {
@@ -715,7 +762,7 @@ public class Minimizer {
       return;
     while (stepMinimization()) {
     }
-    endMinimization();
+    endMinimization(true);
   }
   
   public void report(String msg, boolean isEcho) {
@@ -738,4 +785,24 @@ public class Minimizer {
         ff.getPartialCharges(), null);
   }
 
+  public String getForceFieldUsed() {
+    return (pFF == null ? null : pFF.name);
+  }
+
+  public Boolean isLoggable(int[] iData, int n) {
+    if (bsBasis == null)
+      return Boolean.TRUE;
+    if (iData == null)
+      return bsBasis.get(minAtoms[n].atom.i) ? Boolean.TRUE : Boolean.FALSE;
+    for (int i = 0; i < n; i++) {
+      if (bsBasis.get(minAtoms[iData[i]].atom.i))
+        return Boolean.TRUE;
+    }
+    return Boolean.FALSE;
+  }
+
+  @Override
+  public String toString() {
+    return "[minimizer " + id + " step " + (pFF == null ? 0 : pFF.getCurrentStep()) + " atoms=" + ac + "]";
+  }
 }
