@@ -1,7 +1,7 @@
 /* $RCSfile$
  * $Author: hansonr $
- * $Date: 2023-01-07 08:24:29 -0600 (Sat, 07 Jan 2023) $
- * $Revision: 22501 $
+ * $Date: 2024-04-14 22:04:09 -0500 (Sun, 14 Apr 2024) $
+ * $Revision: 22612 $
 
  *
  * Copyright (C) 2003-2005  The Jmol Development Team
@@ -81,7 +81,13 @@ public class Atom extends Point3fi implements Node {
   public Group group;
   private float userDefinedVanDerWaalRadius;
   byte valence;
-  short atomicAndIsotopeNumber;
+  /**
+   * atomNumberFlags encodes atomic number (a), isotope number(i) as:
+   * 
+   * iiii iiii iaaa aaaa  
+   *
+   */
+  short atomNumberFlags;
   public BS atomSymmetry;
 
   private int formalChargeAndFlags; //  cccc CIP_ _CIP --hv
@@ -141,13 +147,14 @@ public class Atom extends Point3fi implements Node {
     this.atomSymmetry = atomSymmetry;
     this.atomSite = atomSite;
     this.i = atomIndex;
-    this.atomicAndIsotopeNumber = atomicAndIsotopeNumber;
+    this.atomNumberFlags = atomicAndIsotopeNumber;
     if (isHetero)
       formalChargeAndFlags = IS_HETERO_FLAG;
     if (formalCharge != 0 && formalCharge != Integer.MIN_VALUE)
       setFormalCharge(formalCharge);
     userDefinedVanDerWaalRadius = radius;
-    setT(xyz);
+    if (xyz != null)
+      setT(xyz);
     return this;
   }
 
@@ -325,27 +332,27 @@ public class Atom extends Point3fi implements Node {
 
   @Override
   public int getElementNumber() {
-    return Elements.getElementNumber(atomicAndIsotopeNumber);
+    return Elements.getElementNumber(atomNumberFlags);
   }
   
   @Override
   public int getIsotopeNumber() {
-    return Elements.getIsotopeNumber(atomicAndIsotopeNumber);
+    return Elements.getIsotopeNumber(atomNumberFlags);
   }
   
   @Override
   public int getAtomicAndIsotopeNumber() {
-    return atomicAndIsotopeNumber;
+    return atomNumberFlags;
   }
 
   public void setAtomicAndIsotopeNumber(int n) {
     if (n < 0 || (n & 127) >= Elements.elementNumberMax || n > Short.MAX_VALUE)
       n = 0;
-    atomicAndIsotopeNumber = (short) n;
+    atomNumberFlags = (short) n;
   }
 
   public String getElementSymbolIso(boolean withIsotope) {
-    return Elements.elementSymbolFromNumber(withIsotope ? atomicAndIsotopeNumber : atomicAndIsotopeNumber & 127);    
+    return Elements.elementSymbolFromNumber(withIsotope ? atomNumberFlags : atomNumberFlags & 127);    
   }
   
   public String getElementSymbol() {
@@ -432,6 +439,19 @@ public class Atom extends Point3fi implements Node {
    */
   @Override
   public int getValence() {
+    return (isDeleted() ? -1 : valence > 0 ? valence : getValenceAromatic(true));
+  }
+
+  /**
+   * Calculate the number of bonds for this atom, counting double as 2. 
+   * Partial bonds may increment the count.
+   * 
+   * Allow for aromatic-bonded C-O when calculating to return only 1 here.
+   *  
+   * @param checkAromatic
+   * @return the total bond order for this atom
+   */
+  int getValenceAromatic(boolean checkAromatic) {
     if (isDeleted())
       return -1;
     int n = valence;
@@ -439,7 +459,7 @@ public class Atom extends Point3fi implements Node {
       int npartial = 0;
       for (int i = bonds.length; --i >= 0;) {
         n += bonds[i].getValence();
-        if (bonds[i].is(Edge.BOND_AROMATIC))
+        if (checkAromatic && bonds[i].is(Edge.BOND_AROMATIC))
           npartial++;
       }
       if (n > 0 && n < 3 && npartial != 0)
@@ -551,7 +571,7 @@ public class Atom extends Point3fi implements Node {
     // AtomCollection.getAtomPropertyState with VDW_AUTO
     // AtomCollection.getVdwRadius with passed on type
     return (Float.isNaN(userDefinedVanDerWaalRadius) 
-        ? vwr.getVanderwaalsMarType(atomicAndIsotopeNumber, getVdwType(type)) / 1000f
+        ? vwr.getVanderwaalsMarType(atomNumberFlags, getVdwType(type)) / 1000f
         : userDefinedVanDerWaalRadius);
   }
 
@@ -579,7 +599,7 @@ public class Atom extends Point3fi implements Node {
   public float getBondingRadius() {
     float[] rr = group.chain.model.ms.bondingRadii;
     float r = (rr == null  || i >= rr.length ? 0 : rr[i]);
-    return (r == 0 ? Elements.getBondingRadius(atomicAndIsotopeNumber,
+    return (r == 0 ? Elements.getBondingRadius(atomNumberFlags,
         getFormalCharge()) : r);
   }
 
@@ -828,7 +848,7 @@ public class Atom extends Point3fi implements Node {
         c.toFractional(pt, false);
     }
     if (fixJavaFloat)
-      PT.fixPtFloats(pt, asCartesian ? PT.CARTESIAN_PRECISION : PT.FRACTIONAL_PRECISION);      
+      PT.fixPtFloats(pt, asCartesian ? PT.CARTESIAN_PRECISION : PT.FRACTIONAL_PRECISION);
     return pt;
   }
   
@@ -928,6 +948,7 @@ public class Atom extends Point3fi implements Node {
   public final static int ID_ALL  = 2;
   public final static int ID_XTAL = 3;
   public final static int ID_CHIME = 4;
+  public final static int ID_ATOMS = 5;
   
   public String getIdentityXYZ(P3 pt, int mode) {
     pt = (mode == ID_XTAL || group.chain.model.isJmolDataFrame ? getFractionalCoordPt(!group.chain.model.ms.vwr.g.legacyJavaFloat, false, pt) : this);
@@ -935,14 +956,15 @@ public class Atom extends Point3fi implements Node {
         + PT.formatF(pt.x, 0, 3, true, true) 
         + " " + PT.formatF(pt.y, 0, 3, true, true) 
         + " " + PT.formatF(pt.z, 0, 3, true, true) 
-        ;  
+        ; 
     return s;
   }
   
   String getIdentity(int mode) {
     SB info = new SB();
     String group3 = getGroup3(true);
-    if (group3 != null && group3.length() > 0 && (!group3.equals("UNK") || group.chain.model.isBioModel)) {
+    if (group3 != null && group3.length() > 0
+        && (!group3.equals("UNK") || group.chain.model.isBioModel)) {
       info.append("[");
       info.append(group3);
       info.append("]");
@@ -957,7 +979,7 @@ public class Atom extends Point3fi implements Node {
           s = PT.esc(s);
         info.append(s);
       }
-      if (mode != ID_ALL)
+      if (mode != ID_ALL && mode != ID_ATOMS)
         return info.toString();
       info.append(".");
     }
@@ -968,19 +990,21 @@ public class Atom extends Point3fi implements Node {
       info.append(" ");
       info.appendI(getAtomNumber());
     }
-    if (altloc != '\0') {
-      info.append("%");
-      info.appendC(altloc);
+    if (mode == ID_ALL) {
+      if (altloc != '\0') {
+        info.append("%");
+        info.appendC(altloc);
+      }
+      if (group.chain.model.ms.mc > 1 && !group.chain.model.isJmolDataFrame) {
+        info.append("/");
+        info.append(getModelNumberForLabel());
+      }
+      info.append(" #");
+      info.appendI(getAtomNumber());
     }
-    if (group.chain.model.ms.mc > 1 && !group.chain.model.isJmolDataFrame) {
-      info.append("/");
-      info.append(getModelNumberForLabel());
-    }
-    info.append(" #");
-    info.appendI(getAtomNumber());
     return info.toString();
   }
-
+  
   @Override
   public String getGroup3(boolean allowNull) {
     String group3 = group.getGroup3();
@@ -1161,7 +1185,7 @@ public class Atom extends Point3fi implements Node {
     case T.elemno:
       return getElementNumber();
     case T.elemisono:
-      return atomicAndIsotopeNumber;
+      return atomNumberFlags;
     case T.file:
       return group.chain.model.fileIndex + 1;
     case T.formalcharge:
@@ -1261,7 +1285,7 @@ public class Atom extends Point3fi implements Node {
     case T.chemicalshift:
       return vwr.getNMRCalculation().getChemicalShift(this);
     case T.covalentradius:
-      return Elements.getCovalentRadius(atomicAndIsotopeNumber);
+      return Elements.getCovalentRadius(atomNumberFlags);
     case T.eta:
     case T.theta:
     case T.straightness:
@@ -1383,6 +1407,8 @@ public class Atom extends Point3fi implements Node {
     char ch;
     String s;
     switch (tokWhat) {
+    case T.atoms:
+      return getIdentity(ID_ATOMS);
     case T.altloc:
       ch = altloc;
       return (ch == '\0' ? "" : "" + ch);
@@ -1396,6 +1422,11 @@ public class Atom extends Point3fi implements Node {
       return getCIPChirality(true);
     case T.ciprule:
       return getCIPChiralityRule();
+    case T.w:
+    case T.wyckoff:
+      return getWyckoffPosition(false);
+    case T.wyckoffm:
+      return getWyckoffPosition(true);
     case T.sequence:
       return getGroup1('?');
     case T.seqcode:
@@ -1418,6 +1449,8 @@ public class Atom extends Point3fi implements Node {
       if (s == null)
         s = "";
       return s;
+    case T.rxyz:
+      return vwr.getSymStatic().staticToRationalXYZ(getFractionalCoordPt(!vwr.g.legacyJavaFloat, false, null), " "); 
     case T.structure:
       return group.getProteinStructureType().getBioStructureTypeName(false);
     case T.substructure:
@@ -1434,6 +1467,27 @@ public class Atom extends Point3fi implements Node {
     return ""; 
   }
 
+  public String getWyckoffPosition(boolean withMultiplicity) {
+
+    ModelSet ms = group.chain.model.ms;
+    Atom a = ms.getBasisAtom(i, true);
+    int id = a.getSeqID();
+    if (id != 0) {
+      int m = id >> 16;
+      char c = (char) (id & 0xFF);
+      return (withMultiplicity ? "" + (id >> 16) : "") + (char) (id & 0xFF);
+    }
+    SymmetryInterface sym = getUnitCell();
+    String s;
+    if (sym == null
+        || (s = (String) sym.getWyckoffPosition(ms.vwr, this, "M")) == null) {
+      s = "0?";
+    }
+    ms.setAtomSeqID(i, (PT.parseInt(s) << 16) + s.charAt(s.length() - 1));
+    // remove multiplicity
+    return (withMultiplicity ? s : s.substring(s.length() - 1));
+  }
+
   /**
    * Determine R/S chirality at this position; non-H atoms only; cached in formalChargeAndFlags
    * @param doCalculate 
@@ -1443,7 +1497,7 @@ public class Atom extends Point3fi implements Node {
   @Override
   public String getCIPChirality(boolean doCalculate) {
     int flags = (formalChargeAndFlags & CIP_CHIRALITY_MASK) >> CIP_CHIRALITY_OFFSET;
-    if (flags == 0 && atomicAndIsotopeNumber > 1 && doCalculate) {
+    if (flags == 0 && atomNumberFlags > 1 && doCalculate) {
       flags = group.chain.model.ms.getAtomCIPChiralityCode(this);
       formalChargeAndFlags |= ((flags == 0 ? JC.CIP_CHIRALITY_NONE : flags) << CIP_CHIRALITY_OFFSET);
     }
