@@ -35,6 +35,9 @@ import org.jmol.util.Edge;
 import org.jmol.util.JmolMolecule;
 import org.jmol.viewer.JC;
 import javajs.util.BS;
+import javajs.util.Measure;
+import javajs.util.V3;
+
 import org.jmol.script.T;
 
 abstract public class BondCollection extends AtomCollection {
@@ -55,6 +58,8 @@ abstract public class BondCollection extends AtomCollection {
 
   public boolean haveHiddenBonds;
   
+  protected V3 v1 = new V3();
+  protected V3 v2 = new V3();
 
   protected final static int BOND_GROWTH_INCREMENT = 250;
   protected final static int MAX_BONDS_LENGTH_TO_CACHE = 5;
@@ -113,6 +118,27 @@ abstract public class BondCollection extends AtomCollection {
 
   public BS getBondsForSelectedAtoms(BS bsAtoms, boolean bondSelectionModeOr) {
     BS bs = new BS();
+    int n = bsAtoms.cardinality();
+    switch (n) {
+    case 0:
+      return bs;
+    case 1:
+    case 2:
+      Atom a = at[bsAtoms.nextSetBit(0)];
+      Atom b = (n == 2 ? at[bsAtoms.nextSetBit(a.i + 1)] : null);
+      if (n == 1 || bondSelectionModeOr) {
+        for (int i = a.getBondCount(); --i >= 0;)
+          bs.set(a.bonds[i].index);
+        if (b != null)
+          for (int i = b.getBondCount(); --i >= 0;)
+            bs.set(b.bonds[i].index);
+      } else {
+        Bond bond = a.getBond(b);
+        if (b != null)
+          bs.set(bond.index);
+      }
+      return bs;
+    }
     for (int iBond = 0; iBond < bondCount; ++iBond) {
       Bond bond = bo[iBond];
       boolean isSelected1 = bsAtoms.get(bond.atom1.i);
@@ -482,8 +508,8 @@ abstract public class BondCollection extends AtomCollection {
         if (bond.atom1.getElementNumber() == 8 || bond.atom2.getElementNumber() == 8) {
           if (!assignAromaticDouble(bond))
             assignAromaticSingle(bond);
-        } else {
-          bsTest.set(i);
+//?        } else {
+//          bsTest.set(i);
         }
       }
     // now test non-O atoms
@@ -498,7 +524,7 @@ abstract public class BondCollection extends AtomCollection {
           if (!bond.is(Edge.BOND_AROMATIC_DOUBLE)) {
             bsAromatic.set(i);
             bsModels.set(bond.atom1.mi);
-            bond.setOrder(Edge.BOND_AROMATIC_DOUBLE);
+            bond.setOrder(isLinear(bond, v1 , v2) ? Edge.BOND_COVALENT_TRIPLE : Edge.BOND_AROMATIC_DOUBLE);
           }
         } else if (bsAromaticSingle.get(i) || bond.isAromatic()) {
           if (!bond.is(Edge.BOND_AROMATIC_SINGLE)) {
@@ -516,20 +542,33 @@ abstract public class BondCollection extends AtomCollection {
 
     bsAromaticSingle = null;
     bsAromaticDouble = null;
-    
-    ///////
-    // This was in the former method in ModelSet, which was not accessible:
-    //
-    // send a message to STICKS indicating that these bonds
-    // should be part of the state of the model. They will 
-    // appear in the state as bondOrder commands.    
-    //if (isUserCalculation)
-      //shapeManager.setShapeSizeBs(JC.SHAPE_STICKS, Integer.MIN_VALUE, null, bsAromatic);
-    ///////
+  }    
 
-    
+  private boolean isLinear(Bond b, V3 v1, V3 v2) {
+    if (b.order == Edge.BOND_COVALENT_TRIPLE)
+      return true;
+    if (b.atom1.getCovalentBondCount() != 2 || b.atom2.getCovalentBondCount() != 2)
+      return false;
+    Edge[] edges = b.atom1.getEdges();
+    for (int i = edges.length; -i >= 0;) {
+      if (edges[i] != b && edges[i].isCovalent()) {
+        if (Measure.computeAngle((Atom) edges[i].getOtherNode(b.atom1), b.atom1, b.atom2, v1, v2, true) < 175)
+            return false;
+        break;
+      }
+    }
+    edges = b.atom2.getEdges();
+    for (int i = edges.length; -i >= 0;) {
+      if (edges[i] != b && edges[i].isCovalent()) {
+        if (Measure.computeAngle((Atom) edges[i].getOtherNode(b.atom2), b.atom2, b.atom1, v1, v2, true) < 175)
+            return false;
+        break;
+      }
+    }
+    return true;
   }
 
+
   /**
    * try to assign AROMATICDOUBLE to this bond. Each atom needs to be
    * have all single bonds except for this one.  
@@ -659,16 +698,16 @@ abstract public class BondCollection extends AtomCollection {
     default:
       return true;
     }
-    int nAtoms = atom.getValence();
+    int valence = atom.getValenceAromatic(false);
     switch (n) {
     case 6: // C
-      return (nAtoms == 4);
+      return (valence == 4);
     case 7: // N
-      return (atom.group.getNitrogenAtom() == atom || nAtoms == 3 && atom.getFormalCharge() < 1);
+      return (atom.group.getNitrogenAtom() == atom || valence == 3 && atom.getFormalCharge() < 1);
     case 8: // O
-      return (atom.group.getCarbonylOxygenAtom() != atom && nAtoms == 2 && atom.getFormalCharge() < 1);
+      return (atom.group.getCarbonylOxygenAtom() != atom && valence == 2 && atom.getFormalCharge() < 1);
     case 16: // S
-      return (atom.group.groupID == JC.GROUPID_CYSTEINE || nAtoms == 2 && atom.getFormalCharge() < 1);
+      return (atom.group.groupID == JC.GROUPID_CYSTEINE || valence == 2 && atom.getFormalCharge() < 1);
     }
     return false;
   }
@@ -726,8 +765,12 @@ abstract public class BondCollection extends AtomCollection {
     case T.bonds:
       BS bsBonds = (BS) specInfo;
       for (int i = bsBonds.nextSetBit(0); i >= 0; i = bsBonds.nextSetBit(i + 1)) {
-        bs.set(bo[i].atom1.i);
-        bs.set(bo[i].atom2.i);
+        if (i < bondCount) {
+          bs.set(bo[i].atom1.i);
+          bs.set(bo[i].atom2.i);
+        } else {
+          bsBonds.clear(i);
+        }
       }
       return bs;
     case T.isaromatic:
@@ -807,6 +850,15 @@ abstract public class BondCollection extends AtomCollection {
     return bsResult;
   }
 
+  public void addConnectedHAtoms(Atom atom, BS bsAtoms) {
+    if (atom.bonds != null)
+      for (int i = atom.bonds.length; --i >= 0;) {
+        Atom atom2 = atom.bonds[i].getOtherAtom(atom);
+        if (atom2.getElementNumber() == 1)
+          bsAtoms.set(atom2.i);
+      }
+  }
+
   protected boolean haveAtropicBonds;
   
 //  public void setAtropicBonds(BS bs) {
