@@ -97,10 +97,10 @@ public class LabelToken {
 
   // do not change array order without changing string order as well
   // new tokens can be added to the list at the end
-  // and then also added in appendTokenValue()
+  // and then also added in appendAtomTokenValue()
   // and also in Eval, to atomProperty()
 
-  final private static String labelTokenParams = "AaBbCcDEefGgIiLlMmNnOoPpQqRrSsTtUuVvWXxxYyyZzz%%%gqW";
+  final private static String labelTokenParams = "AaBbCcDEefGgIiLlMmNnOoPpQqRrSsTtUuVvWwXxxYyyZzz%%%gqW";
   final private static int[] labelTokenIds = {
       /* 'A' */T.altloc,
       /* 'a' */T.atomname,
@@ -139,6 +139,7 @@ public class LabelToken {
       /* 'V' */T.vanderwaals,
       /* 'v' */T.vibxyz,
       /* 'W' */T.w, // identifier and XYZ coord
+      /* 'w' */T.wyckoffm, // with multiplicity
       /* 'X' */T.fracx,
       /* 'x' */T.atomx,
       /* 'x' */T.x,
@@ -156,7 +157,7 @@ public class LabelToken {
       T.geosurface, T.halo, T.meshRibbon, T.ribbon,
       T.rocket, T.star, T.strands, T.trace,
 
-      T.adpmax, T.adpmin, T.atomid, T.bondcount, T.color,
+      T.adpmax, T.adpmin, T.atoms, T.atomid, T.bondcount, T.color,
       T.groupid, T.covalentradius, T.file, T.format, T.label,
       T.mass, T.modelindex, T.eta, T.omega, T.polymer, T.property,
       T.radius, T.selected, T.shape, T.sequence,
@@ -170,7 +171,9 @@ public class LabelToken {
       T.modx, T.mody, T.modz, T.modo, T.modxyz, T.symop, 
       T.nbo, // added in 14.8.2
       T.chirality, // added in 14.11.4
-      T.ciprule // added in 14.17.0
+      T.ciprule, // added in 14.17.0
+      T.wyckoff, // added in 16.1.44
+      T.rxyz // added in 16.2.5
   };
 
   public LabelToken() {
@@ -401,7 +404,7 @@ public class LabelToken {
     }
     if (ich < cch && htValues != null)
       for (String key: htValues.keySet())
-        if (strFormat.indexOf(key) == ich)
+        if (strFormat.indexOf(key, ich) == ich)
           return ich + (lt.key = key).length();
     if (ich < cch)
       switch (ch = strFormat.charAt(ich++)) {
@@ -496,6 +499,9 @@ public class LabelToken {
 
       // special cases only for labels 
 
+      case T.atoms:
+          strT = atom.atomPropertyString(vwr, T.atoms);
+          break;
       case T.atomindex:
         strT = "" + (indices == null ? atom.i : indices[atom.i]);
         break;
@@ -609,6 +615,12 @@ public class LabelToken {
       case T.w:
         strT = atom.getIdentityXYZ(ptTemp, Atom.ID_U);
         break;
+      case T.wyckoff:
+        strT = atom.getWyckoffPosition(false);
+        break;
+      case T.wyckoffm:
+        strT = atom.getWyckoffPosition(true);
+        break;
         
       // characters only
       // JavaScript switch cannot handle mixed cases of numbers and character codes
@@ -641,6 +653,8 @@ public class LabelToken {
             strT = "";
           break;
         default:
+          if (t.tok == T.rxyz)
+            strT = atom.atomPropertyString(vwr, T.rxyz);
           // any dual case would be here -- must handle specially
         }
       }
