@@ -25,44 +25,41 @@
 
 package org.jmol.modelset;
 
-import org.jmol.util.Elements;
-import javajs.util.P3;
-
-import org.jmol.util.BSUtil;
-import org.jmol.util.Edge;
-import org.jmol.util.JmolMolecule;
-import org.jmol.util.Logger;
-import javajs.util.V3;
-import org.jmol.viewer.JC;
-import org.jmol.script.T;
-import org.jmol.viewer.Viewer;
+import java.util.Arrays;
+import java.util.Hashtable;
+import java.util.Map;
+import java.util.Map.Entry;
+import java.util.Properties;
 
 import org.jmol.api.Interface;
 import org.jmol.api.JmolAdapter;
 import org.jmol.api.JmolAdapterAtomIterator;
 import org.jmol.api.JmolAdapterBondIterator;
 import org.jmol.api.JmolDataManager;
+import org.jmol.api.JmolModulationSet;
 import org.jmol.api.SymmetryInterface;
 import org.jmol.atomdata.RadiusData;
 import org.jmol.c.VDW;
-import javajs.util.BS;
-
 import org.jmol.modelsetbio.BioModel;
 import org.jmol.modelsetbio.BioResolver;
+import org.jmol.script.T;
+import org.jmol.symmetry.Symmetry;
+import org.jmol.util.BSUtil;
+import org.jmol.util.Edge;
+import org.jmol.util.Elements;
+import org.jmol.util.JmolMolecule;
+import org.jmol.util.Logger;
+import org.jmol.viewer.JC;
+import org.jmol.viewer.Viewer;
 
 import javajs.util.AU;
+import javajs.util.BS;
 import javajs.util.Lst;
+import javajs.util.P3;
 import javajs.util.PT;
 import javajs.util.Quat;
 import javajs.util.SB;
-
-import java.util.Arrays;
-
-import java.util.Hashtable;
-
-import java.util.Map;
-import java.util.Map.Entry;
-import java.util.Properties;
+import javajs.util.V3;
 
 /* 
  * 
@@ -96,7 +93,7 @@ public final class ModelLoader {
     ms = new ModelSet(vwr, modelSetName);
     JmolAdapter adapter = vwr.getModelAdapter();
     this.modelSet0 = modelSet0;
-    merging = (modelSet0 != null && modelSet0.ac > 0);
+    merging = (modelSet0 != null && (modelSet0.ac > 0 || modelSet0.vwr.getOperativeSymmetry() != null));
     if (merging) {
       ms.canSkipLoad = false;
     } else {
@@ -117,6 +114,9 @@ public final class ModelLoader {
     info.put("loadScript", loadScript);
     initializeInfo(adapter.getFileTypeName(asc).toLowerCase().intern(), info);
     createModelSet(adapter, asc, bsNew);
+    if (info.get("lowPrecision") != null) {
+      vwr.setBooleanPropertyTok("doublePrecision", T.doubleprecision, false);      
+    }    
     if (jbr != null)
       jbr.setLoader(null);
     jbr = null;
@@ -153,7 +153,6 @@ public final class ModelLoader {
   public Group[] groups;
   private int groupCount;
   private P3 modulationTUV;
-  private boolean isSupercell;
   private boolean noH;
   
 
@@ -205,7 +204,6 @@ public final class ModelLoader {
     ms.someModelsHaveSymmetry = ms.getMSInfoB(JC.getBoolName(JC.GLOBAL_SYMMETRY));
     someModelsHaveUnitcells = ms.getMSInfoB(JC.getBoolName(JC.GLOBAL_UNITCELLS));
     someModelsAreModulated = ms.getMSInfoB(JC.getBoolName(JC.GLOBAL_MODULATED));
-    isSupercell = ms.getMSInfoB(JC.getBoolName(JC.GLOBAL_SUPERCELL));
     ms.someModelsHaveFractionalCoordinates = ms
         .getMSInfoB(JC.getBoolName(JC.GLOBAL_SYMMETRY));
     if (merging) {
@@ -353,7 +351,7 @@ public final class ModelLoader {
     // only now can we access all of the atom's properties
 
     if (is2D && doMinimize) {
-      applyStereochemistry();
+      setupMinimization();
     }
 
     if (doAddPDBHydrogens)
@@ -381,11 +379,6 @@ public final class ModelLoader {
     freeze();
     finalizeShapes();
     vwr.setModelSet(ms);
-    if (isSupercell && appendNew) {
-      for (int i = baseModelIndex; i < ms.mc; i++) {
-        vwr.assignSpaceGroup(null, "P1", i);
-      }
-    }
     setAtomProperties();
     if (adapter != null)
       adapter.finish(asc);    
@@ -530,10 +523,13 @@ public final class ModelLoader {
     Map<String, Object> info = modelSet0.getModelSetAuxiliaryInfo(null);
     String[] mergeGroup3Lists = (String[]) info.get("group3Lists");
     int[][] mergeGroup3Counts = (int[][]) info.get("group3Counts");
+    int nLists = (mergeGroup3Lists == null ? 0 : mergeGroup3Lists.length);
     if (mergeGroup3Lists != null) {
       for (int i = 0; i < baseModelCount; i++) {
-        group3Lists[i + 1] = mergeGroup3Lists[i + 1];
-        group3Counts[i + 1] = mergeGroup3Counts[i + 1];
+        if (i < nLists) {
+          group3Lists[i + 1] = mergeGroup3Lists[i + 1];
+          group3Counts[i + 1] = mergeGroup3Counts[i + 1];
+        }
         structuresDefinedInFile.set(i);
       }
       group3Lists[0] = mergeGroup3Lists[0];
@@ -587,7 +583,7 @@ public final class ModelLoader {
     vwr.setSmilesString((String) ms.msInfo.get("smilesString"));
     String loadState = (String) ms.msInfo.remove("loadState");
     SB loadScript = (SB)ms.msInfo.remove("loadScript");
-    if (loadScript.indexOf("Viewer.AddHydrogens") < 0 || !m.isModelKit) {
+    if (loadScript.indexOf(JC.ADD_HYDROGEN_TITLE) < 0 || !m.isModelKit) {
       String[] lines = PT.split(loadState, "\n");
       SB sb = new SB();
       for (int i = 0; i < lines.length; i++) {
@@ -660,9 +656,9 @@ public final class ModelLoader {
     codes = (String) ms.getInfo(modelIndex, "insertionCodes");
     models[modelIndex].insertionCount = (codes == null ? 0 : codes.length());
     boolean isModelKit = (ms.modelSetName != null
-        && ms.modelSetName.startsWith("Jmol Model Kit")
-        || modelName.startsWith("Jmol Model Kit") || "Jme".equals(ms.getInfo(
-        modelIndex, "fileType")) && is2D);
+        && ms.modelSetName.startsWith(JC.JMOL_MODEL_KIT)
+        || modelName.startsWith(JC.JMOL_MODEL_KIT) 
+        || "Jme".equals(ms.getInfo( modelIndex, "fileType")) && is2D);
     models[modelIndex].isModelKit = isModelKit;
   }
 
@@ -828,7 +824,7 @@ public final class ModelLoader {
       String group3 = iterAtom.getGroup3();
       int chainID = iterAtom.getChainID();
       checkNewGroup(adapter, chainID, group3, iterAtom.getSequenceNumber(),
-          iterAtom.getInsertionCode(), addH, isLegacyHAddition);
+            iterAtom.getInsertionCode(), addH, isLegacyHAddition);
       int isotope = iterAtom.getElementNumber();
       if (addH && Elements.getElementNumber(isotope) == 1)
         jbr.setHaveHsAlready(true);
@@ -870,7 +866,7 @@ public final class ModelLoader {
         iLast = atoms[i].mi;
         Model m = models[iLast];
         m.firstAtomIndex = i;
-        m.isOrderly = (m.act == m.bsAtoms.length()); 
+        m.isOrderly = (m.act == m.bsAtoms.length() - i); 
         VDW vdwtype = ms.getDefaultVdwType(iLast);
         if (vdwtype != vdwtypeLast) {
           Logger.info("Default Van der Waals type for model" + " set to "
@@ -982,7 +978,6 @@ public final class ModelLoader {
   private void checkNewGroup(JmolAdapter adapter, int chainID,
                              String group3, int groupSequenceNumber,
                              char groupInsertionCode, boolean addH, boolean isLegacyHAddition) {
-    String group3i = (group3 == null ? null : group3.intern());
     if (chainID != currentChainID) {
       currentChainID = chainID;
       currentChain = getOrAllocateChain(model, chainID);
@@ -991,6 +986,7 @@ public final class ModelLoader {
       currentGroup3 = "xxxx";
       isNewChain = true;
     }
+    String group3i = (group3 == null ? null : group3.intern());
     if (groupSequenceNumber != currentGroupSequenceNumber
         || groupInsertionCode != currentGroupInsertionCode
         || group3i != currentGroup3) {
@@ -1022,7 +1018,7 @@ public final class ModelLoader {
     if (chain != null)
       return chain;
     if (model.chainCount == model.chains.length)
-      model.chains = (Chain[])AU.doubleLength(model.chains);
+      model.chains = (model.chainCount == 0 ? new Chain[2] : (Chain[])AU.doubleLength(model.chains));
     return model.chains[model.chainCount++] = new Chain(model, chainID, (chainID == 0 || chainID == 32 ? 0 : ++iChain));
   }
 
@@ -1081,7 +1077,7 @@ public final class ModelLoader {
       if (m != lastModel) {
         lastModel = m;
         Map<String, Object> info = ms.getModelAuxiliaryInfo(m);
-        isMOL2D = (info != null && "2D".equals(info.get("dimension")));
+        isMOL2D = (is2D || info != null && "2D".equals(info.get("dimension")));
       }
       bond = ms.bondMutually(atom1, atom2,
           (isMOL2D ? order : Edge.BOND_AROMATIC_SINGLE),
@@ -1125,25 +1121,24 @@ public final class ModelLoader {
 
     if (someModelsAreModulated && ms.bsModulated == null)
       ms.bsModulated = new BS();
-    if (someModelsHaveUnitcells) {
+    boolean haveMergeCells = (modelSet0 != null
+        && modelSet0.unitCells != null);
+    if (someModelsHaveUnitcells || haveMergeCells) {
       ms.unitCells = new SymmetryInterface[ms.mc];
       ms.haveUnitCells = true;
-      boolean haveMergeCells = (modelSet0 != null
-          && modelSet0.unitCells != null);
       for (int i = 0, pt = 0; i < ms.mc; i++) {
         if (haveMergeCells && i < baseModelCount) {
           ms.unitCells[i] = modelSet0.unitCells[i];
-        } else {
+        } else if (ms.getModelAuxiliaryInfo(i).get(JC.INFO_SPACE_GROUP_INDEX) != null) {
           ms.unitCells[i] = Interface.getSymmetry(vwr, "file");
           float[] notionalCell = null;
           if (isTrajectory) {
             @SuppressWarnings("unchecked")
-            Lst<float[]> lst = (Lst<float[]>) ms.getInfoM("unitCells");
+            Lst<float[]> lst = (Lst<float[]>) ms.getInfoM(JC.INFO_UNIT_CELLS);
             if (lst != null)
               notionalCell = lst.get(pt++);
           }
-          ms.unitCells[i].setSymmetryInfo(i, ms.getModelAuxiliaryInfo(i),
-              notionalCell);
+          ((Symmetry) ms.unitCells[i]).setSymmetryInfoFromFile(ms, i, notionalCell);
         }
       }
     }
@@ -1170,7 +1165,7 @@ public final class ModelLoader {
       int modelIndex = -1;
       SymmetryInterface c = null;
       boolean isFractional = false;
-      boolean roundCoords = (!vwr.getBoolean(T.doubleprecision) && !vwr.getBoolean(T.legacyjavafloat));
+//      boolean roundCoords = (!vwr.getBoolean(T.doubleprecision) && !vwr.getBoolean(T.legacyjavafloat));
       for (int i = baseAtomIndex; i < ms.ac; i++) {
         if (atoms[i].mi != modelIndex) {
           modelIndex = atoms[i].mi;
@@ -1180,10 +1175,11 @@ public final class ModelLoader {
         if (isFractional) {
           // it is possible for atoms to have specific unit cells, not just models
           // this happens for commensurately modulated composite compounds
-          c = atoms[i].getUnitCell();
-          c.toCartesian(c.toSupercell(atoms[i]), false);
-          if (roundCoords)
-            PT.fixPtFloats(atoms[i], PT.CARTESIAN_PRECISION);
+          JmolModulationSet m = ms.getModulation(i);
+          SymmetryInterface uca = (m == null ? c : atoms[i].getUnitCell());
+          uca.toCartesian(uca.toSupercell(atoms[i]), false);
+//          if (roundCoords)
+//            PT.fixPtFloats(atoms[i], PT.CARTESIAN_PRECISION);
         }
       }
       for (int imodel = baseModelIndex; imodel < ms.mc; imodel++)
@@ -1424,12 +1420,10 @@ public final class ModelLoader {
     }
   }
 
-  private void applyStereochemistry() {
+  private void setupMinimization() {
 
-    int atomCount = ms.ac;//ms.am[ms.at[0].mi].bsAtoms.cardinality();
-    
     // 1) initialize average bond lengths
-    set2DLengths(baseAtomIndex, atomCount);
+    initialize2DMin();
     
     V3 v = new V3();
     
@@ -1458,13 +1452,12 @@ public final class ModelLoader {
     
     // 2) implicit stereochemistry 
     
-    set2dZ(baseAtomIndex, atomCount, v);
+    set2dZ(v);
 
     // 3) explicit stereochemistry
     
     if (vStereo != null) {
-      BS bsToTest = new BS();
-      bsToTest.setBits(baseAtomIndex, atomCount);
+      BS bsToTest = BSUtil.newBitSet2(baseAtomIndex, vwr.ms.ac);
       for (int i = vStereo.size(); --i >= 0;) {
         Bond b = vStereo.get(i);
         float dz2 = (b.order == Edge.BOND_STEREO_NEAR ? 3 : -3);
@@ -1486,11 +1479,19 @@ public final class ModelLoader {
     is2D = false;
   }
 
-  private void set2DLengths(int iatom1, int iatom2) {
+  private void initialize2DMin() {
     float scaling = 0;
     int n = 0;
-    for (int i = iatom1; i < iatom2; i++) {
+    int lastModel = -1;
+    int i0 = baseAtomIndex;
+    int i1 = vwr.ms.ac;
+    for (int i = i0; i < i1; i++) {
       Atom a = ms.at[i];
+      int m = a.getModelIndex();
+      if (m != lastModel) {
+        lastModel = m;
+        ms.setInfo(m, "dimension", "3D");
+      }
       Bond[] bonds = a.bonds;
       if (bonds == null)
         continue;
@@ -1507,12 +1508,14 @@ public final class ModelLoader {
     if (n == 0)
       return;
     scaling = 1.45f / (scaling/n);
-    for (int i = iatom1; i < iatom2; i++) {
+    for (int i = i0; i < i1; i++) {
       ms.at[i].scale(scaling);
     }
   }
 
-  private void set2dZ(int iatom1, int iatom2, V3 v) {
+  private void set2dZ(V3 v) {
+    int iatom1 = baseAtomIndex;
+    int iatom2 = vwr.ms.ac;
     BS atomlist = BS.newN(iatom2);
     BS bsBranch = new BS();
     V3 v0 = V3.new3(0, 1, 0);
@@ -1543,8 +1546,7 @@ public final class ModelLoader {
     BS bs = BS.newN(ms.ac);
     if (atomIndex < 0)
       return bs;
-    BS bsToTest = new BS();
-    bsToTest.or(bs0);
+    BS bsToTest = BSUtil.copy(bs0);
     if (atomIndexNot >= 0)
       bsToTest.clear(atomIndexNot);
     setBranch2dZ(ms.at[atomIndex], bs, bsToTest, v, v0, v1, dir);
