@@ -110,7 +110,6 @@ public class Text {
   private void set(Font font, short colix, int align,
                    float scalePixelsPerMicron) {
     this.scalePixelsPerMicron = scalePixelsPerMicron;
-    //this.isEcho = isEcho;
     this.colix = colix;
     this.align = align;
     this.setFont(font, !isEcho);
@@ -180,7 +179,7 @@ public class Text {
     if (fontScale == scale)
       return;
     fontScale = scale;
-    if (fontScale != 0)
+    if (fontScale != 0 && font != null)
       setFont(vwr.gdata.getFont3DScaled(font, scale), true);
   }
 
@@ -238,9 +237,11 @@ public class Text {
     xAdj = (fontScale >= 2 ? 8 : 4);
     yAdj = ascent - lineHeight + xAdj;
     if (!isEcho || pymolOffset != null) {
+      
       boxXY[0] = movableX;
       boxXY[1] = movableY;
-      if (pymolOffset != null && pymolOffset[0] != 2 && pymolOffset[0] != 3) {
+      boolean isAng = !isPymolOffsetPixels();
+      if (pymolOffset != null && isAng) {
         // [1,2,3] are in Angstroms, not screen pixels
         float pixelsPerAngstrom = vwr.tm.scaleToScreen(z, 1000);
         float pz = pymolOffset[3];
@@ -281,6 +282,7 @@ public class Text {
           dy -= descent;
         }
 
+
         // xAdj and yAdj are the adjustments for the box itself relative to the text 
         xAdj = (fontScale >= 2 ? 8 : 4);
         yAdj = -descent;
@@ -323,21 +325,22 @@ public class Text {
       y0 += ascent + (lines.length - 1)/2f * lineHeight;
   }
 
-  private float getPymolXYOffset(float off, int width, float ppa) {
-    float f = (off < -1 ? -1 : off > 1 ? 0 : (off - 1) / 2);
-    // offset
-    // -3     -2
-    // -2     -1
-    // -1      0 absolute, -1 width
-    //-0.5    -3/4  width
-    //  0     -1/2 width
-    // 0.5    -1/4 width
-    //  1      0
-    //  2      1
-    //  3      2
-    off = (off < -1 || off > 1 ? off + (off < 0 ? 1 : -1) : 0);
-    return f * width + off * ppa;
-  }
+  private float getPymolXYOffset(float x, int width, float ppa) {
+	    float f = (x < -1 ? -1 : x > 1 ? 0 : (x - 1) / 2);
+	    //  x   f/width    offset/ppa
+	    // -3     -1         -2
+	    // -2     -1         -1
+	    // -1      0          0
+	    //-0.5    -3/4        0
+	    //  0     -1/2        0
+	    // 0.5    -1/4        0
+	    //  1      0          0
+	    //  2      1          1
+	    //  3      1          2
+	    
+	    float offset = (x < -1 || x > 1 ? x + (x < 0 ? 1 : -1) : 0);
+	    return f * width + offset * ppa;
+	  }
 
   private void setPos(float scale) {
     float xLeft, xCenter, xRight;
@@ -523,6 +526,37 @@ public class Text {
 
   // PyMOL-type offset
   // [mode, screenoffsetx,y,z (applied after tranform), positionOffsetx,y,z (applied before transform)]
+
+//
+//new feature: set labelOffset [mode sx sy sz ax ay az] (3.1.15, never documented)
+//new feature: PyMOL-like label offset options:
+//   
+//   set labelOffset [sx, sy, sz]
+//   set labelOffset [mode, sx, sy, sz, ax, ay, az]
+//   
+// where
+//   
+//   sx,sy,sz are screen coord offsets 
+//    -- applied after view rotation
+//    -- sy > 0 LOWERS label
+//   ax,ay,az are xyz position (in Angstroms; applied before view rotation)
+//   mode == 0 indicates xyz position is absolute and sx sy sz are Angstroms
+//   mode == 1 indicates xyz position is relative to atom position and sx sy sz are Angstroms
+//   mode == 2 indicates xyz is absolute, and sx sy sz positions are screen pixels
+//   mode == 3 indicates xyz is relative, and sx sy sz positions are screen pixels
+//   defaults: mode == 1; ax = ay = az = 0
+//   
+//   
+
+  public final static int PYMOL_LABEL_OFFSET_JMOL= -1;
+  public final static int PYMOL_LABEL_OFFSET_REL = 1;
+  public final static int PYMOL_LABEL_OFFSET_PIX = 2; // and not -1
+
+  public final static int PYMOL_LABEL_OFFSET_ABS_ANG = 0;
+  public final static int PYMOL_LABEL_OFFSET_REL_ANG = PYMOL_LABEL_OFFSET_REL; 
+  public final static int PYMOL_LABEL_OFFSET_ABS_PIX = PYMOL_LABEL_OFFSET_PIX;
+  public final static int PYMOL_LABEL_OFFSET_REL_PIX = PYMOL_LABEL_OFFSET_REL | PYMOL_LABEL_OFFSET_PIX;
+  
   public float[] pymolOffset;
 
   protected int windowWidth;
@@ -534,6 +568,7 @@ public class Text {
   public float boxY;
 
   public int modelIndex = -1;
+  public boolean thisModelOnly;
   public boolean visible = true;
   public boolean hidden = false;
 
@@ -623,12 +658,14 @@ public class Text {
   }
 
   public boolean setAlignmentLCR(String align) {
+    if (align != null) {
     if ("left".equals(align))
       return setAlignment(JC.TEXT_ALIGN_LEFT);
     if ("center".equals(align))
       return setAlignment(JC.TEXT_ALIGN_CENTER);
     if ("right".equals(align))
       return setAlignment(JC.TEXT_ALIGN_RIGHT);
+    }
     return false;
   }
 
@@ -682,27 +719,6 @@ public class Text {
         + boxHeight);
   }
 
-//  
-//  new feature: set labelOffset [mode sx sy sz ax ay az] (3.1.15, never documented)
-//  new feature: PyMOL-like label offset options:
-//     
-//     set labelOffset [sx, sy, sz]
-//     set labelOffset [mode, sx, sy, sz, ax, ay, az]
-//     
-//   where
-//     
-//     sx,sy,sz are screen coord offsets 
-//      -- applied after view rotation
-//      -- sy > 0 LOWERS label
-//     ax,ay,az are xyz position (in Angstroms; applied before view rotation)
-//     mode == 0 indicates xyz position is absolute and sx sy sz are Angstroms
-//     mode == 1 indicates xyz position is relative to atom position and sx sy sz are Angstroms
-//     mode == 2 indicates xyz is absolute, and sx sy sz positions are screen pixels
-//     mode == 3 indicates xyz is relative, and sx sy sz positions are screen pixels
-//     defaults: mode == 1; ax = ay = az = 0
-//     
-//     
-
   /**
    * PyMOL will use 1 here for pymolOffset[0] for relative, 0 or absolute. Jmol
    * set labelOffset or set echo offset or measure offset will set -1, when
@@ -716,21 +732,37 @@ public class Text {
    * @param sppm
    */
   public void getPymolScreenOffset(P3 atomPt, P3i screen, int zSlab, P3 pTemp,
-                                   float sppm) {
-    float mode = pymolOffset[0];
-    if (atomPt != null && (Math.abs(mode) % 2) == 1)
+                                   double sppm) {
+    boolean isPixel = isPymolOffsetPixels();
+    boolean isRelative = isPymolOffsetRelative();
+    if (atomPt != null && isRelative)
       pTemp.setT(atomPt);
     else
       pTemp.set(0, 0, 0);
     pTemp.add3(pymolOffset[4], pymolOffset[5], pymolOffset[6]);
     vwr.tm.transformPtScr(pTemp, screen);
-    if (mode == 2 || mode == 3) {
+    if (isPixel) {
       screen.x += pymolOffset[1];
       screen.y += pymolOffset[2];
       screen.z += pymolOffset[3];
     }
     setXYZs(screen.x, screen.y, screen.z, zSlab);
-    setScalePixelsPerMicron(sppm);
+    // commented out; this next was the problem with setFontScaling TRUE and a 3d echo with offset
+    //      setScalePixelsPerMicron(sppm);
+  }
+
+  private boolean isPymolOffsetRelative() {
+    int mode = (pymolOffset == null ? PYMOL_LABEL_OFFSET_JMOL : (int) pymolOffset[0]);
+    return ((mode & PYMOL_LABEL_OFFSET_REL) == PYMOL_LABEL_OFFSET_REL);
+  }
+
+  private boolean isPymolOffsetPixels() {
+    int mode = (pymolOffset == null ? PYMOL_LABEL_OFFSET_JMOL : (int) pymolOffset[0]);
+    return (mode != PYMOL_LABEL_OFFSET_JMOL && ((mode & PYMOL_LABEL_OFFSET_PIX) == PYMOL_LABEL_OFFSET_PIX));
+  }
+
+  public String getStateText() {
+    return (doFormatText ? PT.rep(text, "@{", "\\@{") : text);
   }
 
   @Override
