@@ -543,7 +543,7 @@ public class BioExt {
         if (pdbATOM == null)// || bsSelected != null && !bsSelected.get(a.getIndex()))
           continue;
         bsWritten.set(((Monomer) a.group).leadAtomIndex);
-        pdbATOM.append(ms.getLabeler().formatLabelAtomArray(vwr, a, tokens,
+        pdbATOM.append(LabelToken.formatLabelAtomArray(vwr, a, tokens,
             '\0', null, ptTemp));
         pdbATOM.append(
             PT.sprintf("%8.2f%8.2f%8.2f      %6.3f          %2s    %s\n", "ssF",
@@ -1327,7 +1327,7 @@ public class BioExt {
             AtomCollection.CALC_H_ALLOW_H);
         if (p.length == 1) {
           Lst<Atom> vConnections = new Lst<Atom>();
-          vConnections.add(a);
+          vConnections.addLast(a);
           Atom b = vwr.ms.addAtom(a.mi, a.group, 1, "H3", null, 0, a.getSeqID(), 0, p[0],
               Float.NaN, null, 0, 0, 1, 0, null, a.isHetero(), (byte) 0, null, Float.NaN);
           vwr.ms.bondAtoms(a, b, Edge.BOND_COVALENT_SINGLE, 
