@@ -76,9 +76,19 @@ public final class BioModel extends Model {
   boolean isMutated;
 
   String defaultStructure;
-  
-  BioModel(ModelSet modelSet, int modelIndex, int trajectoryBaseIndex, 
+
+  /**
+   * @param modelSet
+   * @param modelIndex
+   * @param trajectoryBaseIndex
+   * @param jmolData
+   * @param properties
+   * @param auxiliaryInfo
+   * @return this
+   */
+  BioModel setBioModel(ModelSet modelSet, int modelIndex, int trajectoryBaseIndex, 
       String jmolData, Properties properties, Map<String, Object> auxiliaryInfo) {
+    setupArrays();
     vwr = modelSet.vwr;
     set(modelSet, modelIndex, trajectoryBaseIndex, jmolData, properties, auxiliaryInfo);
     
@@ -87,6 +97,7 @@ public final class BioModel extends Model {
     clearBioPolymers();
     modelSet.am[modelIndex] = this;
     pdbID = (String) auxiliaryInfo.get("name");
+    return this;
   }
 
   int addBioPolymer(BioPolymer polymer) {
@@ -101,7 +112,7 @@ public final class BioModel extends Model {
 
 
   void addSecondaryStructure(STR type, String structureID,
-                                    int serialID, int strandCount,
+                                    String serialID, int strandCount,
                                     int startChainID, int startSeqcode,
                                     int endChainID, int endSeqcode, int istart,
                                     int iend, BS bsAssigned) {
@@ -212,50 +223,117 @@ public final class BioModel extends Model {
 
   /**
    * @param conformationIndex0
-   * @param doSet 
+   * @param doSet
    * @param bsAtoms
-   * @param bsRet
-   * @return true;
+   * @param bsRet to be set
    */
-  public boolean getConformation(int conformationIndex0, boolean doSet, BS bsAtoms, BS bsRet) {
-    if (conformationIndex0 >= 0) {
-      int nAltLocs = altLocCount;
-      if (nAltLocs > 0) {
-        Atom[] atoms = ms.at;
-        Group g = null;
-        char ch = '\0';
-        int conformationIndex = conformationIndex0;
-        BS bsFound = new BS();
-        for (int i = bsAtoms.nextSetBit(0); i >= 0; i = bsAtoms.nextSetBit(i + 1)) {
-            Atom atom = atoms[i];
-            char altloc = atom.altloc;
-            // ignore (include) atoms that have no designation
-            if (altloc == '\0')
-              continue;
-            if (atom.group != g) {
-              g = atom.group;
-              ch = '\0';
-              conformationIndex = conformationIndex0;
-              bsFound.clearAll();
-            }
-            // count down until we get the desired index into the list
-            if (conformationIndex >= 0 && altloc != ch && !bsFound.get(altloc)) {
-              ch = altloc;
-              conformationIndex--;
-              bsFound.set(altloc);
-            }
-            if (conformationIndex >= 0 || altloc != ch)
-              bsAtoms.clear(i);
+  public void getConformation(int conformationIndex0, boolean doSet,
+                                 BS bsAtoms, BS bsRet) {
+    boolean isConfig0 = (conformationIndex0 == 0);
+    if (altLocCount == 0) {
+      if (isConfig0)
+        bsRet.or(bsAtoms);
+      return;
+    }
+    boolean isFirst = (conformationIndex0 <= 0);
+    Atom[] atoms = ms.at;
+    boolean isSpace = (conformationIndex0 == -32);
+    char thisAltLoc = (isFirst && !isSpace ? (char) -conformationIndex0 : '\0');
+    if (isFirst) {
+      // config=n where n <= 0   atom-based
+      int lastChain = Integer.MIN_VALUE;
+      char lastIns = '\0';
+      int lastRes = Integer.MIN_VALUE;
+      Map<String, int[]> map = new Hashtable<String, int[]>(); // map name to [first loc, foundFlag]
+      for (int i = bsAtoms.nextSetBit(0); i >= 0; i = bsAtoms
+          .nextSetBit(i + 1)) {
+        Atom atom = atoms[i];
+        int chain = atom.getChainID();
+        int res = atom.getResno();
+        char ins = atom.getInsertionCode();
+        if (res != lastRes || ins != lastIns || chain != lastChain) {
+          lastChain = chain;
+          lastRes = res;
+          lastIns = ins;
+          map.clear();
+        }
+        String name = atom.getAtomName();
+        int[] locs = map.get(name);
+        if (locs == null)
+          map.put(name, locs = new int[1]);//first location or -1 (foundFlag)
+        int i0 = locs[0] - 1;
+        if (i0 < 0) {
+          if (i0 == -2) {
+            // desired loc already found - clear this atom and continue
+            bsAtoms.clear(i);
+            continue;
           }
+          // first for this atom
+          if (isSpace) {
+            // request is config=''
+            // clear it only if it is an altloc
+            if (atom.altloc != '\0') {
+              bsAtoms.clear(i);
+            }
+          } else {
+            // request is for a specific altloc
+            // just mark this as the first location and leave it selected
+            locs[0] = i + 1;
+          }
+          continue;
+        }
+        // not first for this atom 
+        if (isConfig0) {
+          // config=0 - clear this and set found flag
+          bsAtoms.clear(i);
+          locs[0] = -1;
+          // and clear i0
+        } else if (atom.altloc == thisAltLoc) {
+          // found - set flag
+          locs[0] = -1;
+          // and clear i0
+        } else {
+          // clear this atom, not first
+          i0 = i;
+        }
+        bsAtoms.clear(i0);
+      }
+    } else {
+      // config=n where n > 0   residue-based
+      conformationIndex0--;
+      Group g = null;
+      char ch = '\0';
+      int conformationIndex = conformationIndex0;
+      BS bsFound = new BS();
+      for (int i = bsAtoms.nextSetBit(0); i >= 0; i = bsAtoms
+          .nextSetBit(i + 1)) {
+        Atom atom = atoms[i];
+        char altloc = atom.altloc;
+        // ignore (include) atoms that have no designation
+        if (altloc == '\0')
+          continue;
+        if (atom.group != g) {
+          g = atom.group;
+          ch = '\0';
+          conformationIndex = conformationIndex0;
+          bsFound.clearAll();
+        }
+        // count down until we get the desired index into the list
+        if (conformationIndex >= 0 && altloc != ch && !bsFound.get(altloc)) {
+          ch = altloc;
+          conformationIndex--;
+          bsFound.set(altloc);
+        }
+        if (conformationIndex >= 0 || altloc != ch)
+          bsAtoms.clear(i);
       }
     }
     if (bsAtoms.nextSetBit(0) >= 0) {
-      bsRet.or(bsAtoms);      
+      bsRet.or(bsAtoms);
       if (doSet)
         for (int j = bioPolymerCount; --j >= 0;)
           bioPolymers[j].setConformation(bsAtoms);
     }
-    return true;
   }
 
   public void getDefaultLargePDBRendering(SB sb, int maxAtoms) {
