@@ -1,7 +1,7 @@
 /* $RCSfile$
  * $Author: hansonr $
- * $Date: 2022-04-14 15:08:02 -0500 (Thu, 14 Apr 2022) $
- * $Revision: 22400 $
+ * $Date: 2023-03-22 18:24:53 -0500 (Wed, 22 Mar 2023) $
+ * $Revision: 22512 $
  *
  * Copyright (C) 2002-2005  The Jmol Development Team
  *
@@ -76,7 +76,7 @@ public class LabelsRenderer extends FontLineShapeRenderer {
 
   private float scalePixelsPerMicron;
 
-  private int mode;
+  protected int mode;
 
   @Override
   protected boolean render() {
