@@ -1,7 +1,7 @@
 /* $RCSfile$
  * $Author: hansonr $
- * $Date: 2022-03-27 18:31:37 -0500 (Sun, 27 Mar 2022) $
- * $Revision: 22381 $
+ * $Date: 2023-03-22 18:24:53 -0500 (Wed, 22 Mar 2023) $
+ * $Revision: 22512 $
  *
  * Copyright (C) 2003-2005  The Jmol Development Team
  *
@@ -113,6 +113,7 @@ public class MeasuresRenderer extends LabelsRenderer {
         labelColix = colix;
       g3d.setC(colix);
       colixA = colixB = colix;
+      mode = (g3d.isAntialiased() ? TextRenderer.MODE_IS_ANTIALIASED : 0);
       renderMeasurement(showMeasurementLabels);
       //checkAtoms("m3");
     }
@@ -164,6 +165,7 @@ public class MeasuresRenderer extends LabelsRenderer {
       } else if (m.text != null) {
         m.text.setText(s);
         m.text.colix = labelColix;
+        m.text.setFontScale(imageFontScaling);
       }
     }
     if (m.mad == 0) {
