@@ -78,6 +78,8 @@ public class DrawRenderer extends MeshRenderer {
         System.out.println("DrawRenderer mesh is null?");
         return false;
       }
+      if (dmesh.thisModelOnly && vwr.am.cmi < 0)
+        continue;
       if (mesh.connectedAtoms != null) {
         if (mesh.connectedAtoms[0] < 0)
           continue;
@@ -97,7 +99,7 @@ public class DrawRenderer extends MeshRenderer {
         renderInfo();
       if (!isExport 
           && mesh.visibilityFlags != 0
-          && vwr.getPickingMode() == ActionManager.PICKING_DRAW) {
+          && (vwr.getDrawHover() || vwr.getPickingMode() == ActionManager.PICKING_DRAW)) {
         if (!g3d.setC(C.getColixTranslucent3(C.GOLD, true, 0.5f)))
           needTranslucent = true;
         else
@@ -139,26 +141,8 @@ public class DrawRenderer extends MeshRenderer {
       if (diameter == 0)
         diameter = 1;
     }
-    if (dmesh.haveXyPoints) {
-      if (dmesh.isVector) {
-        int ptXY = 0;
-        // [x y] or [x,y] refers to an xy point on the screen
-        // just a Point3f with z = Float.MAX_VALUE
-        //  [x y %] or [x,y %] refers to an xy point on the screen
-        // as a percent 
-        // just a Point3f with z = -Float.MAX_VALUE
-        for (int i = 0; i < 2; i++)
-          if (vertices[i].z == Float.MAX_VALUE
-              || vertices[i].z == -Float.MAX_VALUE)
-            ptXY += i + 1;
-        if (--ptXY < 2) {
-          renderXyArrow(ptXY);
+    if (dmesh.haveXyPoints && drawXYPoints())
           return;
-        }
-      } else if (drawType == Draw.EnumDrawType.POINT){
-        renderXyPoint();
-      }
-    }
     int tension = 5;
     switch (drawType) {
     case CYLINDER:
@@ -248,6 +232,28 @@ public class DrawRenderer extends MeshRenderer {
 
   }
 
+  private boolean drawXYPoints() {
+	    if (dmesh.isVector) {
+	      int ptXY = 0;
+	      // [x y] or [x,y] refers to an xy point on the screen
+	      // just a Point3f with z = Float.MAX_VALUE
+	      //  [x y %] or [x,y %] refers to an xy point on the screen
+	      // as a percent 
+	      // just a Point3f with z = -Float.MAX_VALUE
+	      for (int i = 0; i < 2; i++)
+	        if (Draw.is2DPoint(vertices[i]))
+	          ptXY += i + 1;
+	      if (--ptXY < 2) {
+	        renderXyArrow(ptXY);
+	        return true;
+	      }
+	    } else if (drawType == Draw.EnumDrawType.POINT || drawType == Draw.EnumDrawType.MULTIPLE){
+	      renderXyPoint();
+	      return true;
+	    }
+	    return false;
+	  }
+
   private int setArc(T3 v1, T3 v2, T3 ptRef, float nDegreesOffset,
                        float theta, float fractionalOffset, float scale) {
     vTemp.sub2(v2, v1);
@@ -369,6 +375,7 @@ public class DrawRenderer extends MeshRenderer {
 
   private void renderXyPoint() {
     // new in Jmol 14.5
+    // z == -Float.MAX_VALUE indicates percentage of width and height
     int f = (g3d.isAntialiased() ? 2 : 1);
     pt0.setT(vertices[0]);
     if (diameter == 0)
@@ -476,6 +483,7 @@ public class DrawRenderer extends MeshRenderer {
   }
 
   private final BS bsHandles = new BS();
+  private boolean haveNotifiedHandles;
   
   private void renderHandles() {
     int diameter = Math.round(10 * imageFontScaling);
@@ -498,8 +506,15 @@ public class DrawRenderer extends MeshRenderer {
           if (bsHandles.get(k))
             continue;
           bsHandles.set(k);
-          g3d.drawFilledCircle(C.GOLD, colixFill, diameter,
+          if (k >= screens.length) {
+            if (!haveNotifiedHandles) {
+              System.out.println("DrawRender handles k >= screens.length " + k + " ");
+              haveNotifiedHandles = true;
+            }
+          } else {
+            g3d.drawFilledCircle(C.GOLD, colixFill, diameter,
               screens[k].x, screens[k].y, screens[k].z);
+          }
         }
       }
       break;
@@ -519,6 +534,8 @@ public class DrawRenderer extends MeshRenderer {
         //just the first line of the title -- nothing fancy here.
         if (!haveFont || dmesh.fontID != lastFID) {
           f = (Font) vwr.shm.getShapePropertyIndex(JC.SHAPE_DRAW, "font", i);
+          if (f == null)
+            f = f0;
           lastFID = f.fid;
           vwr.gdata.setFont(imageFontScaling == 1 ? f : vwr.gdata.getFont3DFSS(f.fontFace, f.fontStyle, f.fontSize * imageFontScaling));
           haveFont = true;
