@@ -508,6 +508,7 @@ abstract class ScriptExpr extends ScriptParam {
           rpn.addOp(T.tokenRightParen);
         }
         break;
+      case T.atoms:
       case T.atomname:
       case T.atomtype:
       case T.branch:
@@ -862,7 +863,8 @@ abstract class ScriptExpr extends ScriptParam {
         rpn.addXBs(BSUtil.copy(vwr.slm.getHiddenSet()));
         break;
       case T.fixed:
-        rpn.addXBs(vwr.getMotionFixedAtoms());
+        // note - this is only for the current model
+        rpn.addXBs(vwr.getMotionFixedAtoms(null, null));
         break;
       case T.displayed:
         rpn.addXBs(BSUtil.copyInvert(vwr.slm.getHiddenSet(), ac));
@@ -1006,7 +1008,7 @@ abstract class ScriptExpr extends ScriptParam {
         rpn.addXNum(instruction);
         break;
       case T.bitset:
-        BS bs1 = BSUtil.copy((BS) value);
+        BS bs1 = (value instanceof BondSet ? (BondSet) value : BSUtil.copy((BS) value));
         rpn.addXBs(bs1);
         break;
       case T.point3f:
@@ -1113,7 +1115,8 @@ abstract class ScriptExpr extends ScriptParam {
           if (v.tok == T.varray)
             return getComparison(v, tokWhat, tokOp, strOp, data);
           comparisonInt = v.intValue;
-          val = (isStringProperty ? SV.sValue(v) : SV.nValue(v));
+          val = (isStringProperty || tokWhat == T.configuration && v.tok != T.integer ? SV.sValue(v) : SV.nValue(v));
+          t = v;
         }          
       }
     }
@@ -1142,7 +1145,9 @@ abstract class ScriptExpr extends ScriptParam {
         tokValue = T.integer;
         isIntProperty = true;
       } else if (!isStringProperty) {
-        if (tokWhat == T.structure || tokWhat == T.substructure
+        if (tokWhat == T.configuration) {
+          val = Integer.valueOf(t.tok == T.integer ? t.intValue : -1000 - (val + " ").codePointAt(0));
+        } else if (tokWhat == T.structure || tokWhat == T.substructure
             || tokWhat == T.element)
           isStringProperty = !(isIntProperty = (comparisonInt != Integer.MAX_VALUE));
         else
@@ -1241,7 +1246,7 @@ abstract class ScriptExpr extends ScriptParam {
           false, null, null, false);
       if (v.size() == 0)
         invArg();
-      ht.put(key, v.get(0));
+      ht.put(key, SV.copySafely(v.get(0)));
       i = iToken;
       if (tokAt(i) != T.comma)
         break;
@@ -1421,7 +1426,7 @@ abstract class ScriptExpr extends ScriptParam {
       case T.subsystem:
       case T.configuration:
         // these are all-inclusive; no need to do a by-atom comparison
-        return BSUtil.copy(vwr.ms.getConformation(-1, ival - 1, false, null));
+        return BSUtil.copy(vwr.ms.getConformation(-1, ival, false, null));
       case T.symop:
         propertyBitSet = atom.atomSymmetry;
         if (propertyBitSet == null)
@@ -1621,6 +1626,13 @@ abstract class ScriptExpr extends ScriptParam {
     case T.function:
     case T.distance:
       break;
+    case T.wyckoff:
+      isInt = (tokenValue == "color");
+      isString = !isInt;
+      break;
+    case T.rxyz:
+      isString = true;
+      break;
     default:
       isInt = T.tokAttr(tok, T.intproperty) && !T.tokAttr(tok, T.floatproperty);
       // occupancy and radius considered floats here
@@ -1645,11 +1657,15 @@ abstract class ScriptExpr extends ScriptParam {
     float[][] ffdata = null;
     switch (tok) {
     case T.atoms:
+      if (isAtoms)
+        break;
+      //$FALL-THROUGH$
     case T.bonds:
       if (chk)
         return bs;
-      bsNew = (tok == T.atoms ? (isAtoms ? bs : vwr.ms.getAtoms(T.bonds, bs))
-          : (isAtoms ? (BS) BondSet.newBS(vwr.getBondsForSelectedAtoms(bs), null)
+      bsNew = (tok == T.atoms ? vwr.ms.getAtoms(T.bonds, bs)
+          : (isAtoms
+              ? (BS) BondSet.newBS(vwr.getBondsForSelectedAtoms(bs))
               : bs));
       int i;
       switch (minmaxtype) {
@@ -1673,6 +1689,7 @@ abstract class ScriptExpr extends ScriptParam {
     case T.identify:
       switch (minmaxtype) {
       case 0:
+      case T.min: // single value
       case T.all:
         return getCmdExt().getBitsetIdent(bs, null, tokenValue, useAtomMap,
             index, isExplicitlyAll);
@@ -1812,8 +1829,24 @@ abstract class ScriptExpr extends ScriptParam {
           case T.cell:
             errorStr(ERROR_unrecognizedAtomProperty, T.nameOf(tok));
             break;
+          case T.wyckoff:
+            // a-z(1-26), ?(0), and A(27, space group 47 only
+            iv = atom.getWyckoffPosition(false).charAt(0);
+            switch (iv) {
+            case 63: // ?
+               iv = 0;
+               break;
+            case 65: // A
+              iv = 27;
+              break;
+          default:
+              iv = iv & 0x1F;
+              break;
+            }
+            break;
           default:
             iv = atom.atomPropertyInt(tok);
+            break;
           }
           switch (minmaxtype) {
           case T.min:
@@ -2177,7 +2210,7 @@ abstract class ScriptExpr extends ScriptParam {
       invArg();
     if (chk || v.get(0).tok == T.nada)
       return null;
-    SV tv = SV.selectItemVar(SV.newS("").setv(v.get(nv - 1)));
+    SV tv = SV.selectItemVar(SV.copySafely(v.get(nv - 1)));
     if (nv > 1) {
       SV sel = (nv > 2 ? v.get(1) : null);
       t = v.get(0);
@@ -2323,7 +2356,7 @@ abstract class ScriptExpr extends ScriptParam {
 
   private void setBitsetProperty(BS bs, int tok, int iValue, float fValue,
                                  T tokenValue) throws ScriptException {
-    if (chk || bs.cardinality() == 0)
+    if (chk || bs.isEmpty())
       return;
     String[] list = null;
     String sValue = null;
