@@ -26,8 +26,16 @@ package org.jmol.script;
 import java.util.Hashtable;
 import java.util.Map;
 
+import org.jmol.modelset.BondSet;
+import org.jmol.util.BSUtil;
+import org.jmol.util.BoxInfo;
+import org.jmol.util.Escape;
+import org.jmol.util.Logger;
+import org.jmol.viewer.Viewer;
+
 import javajs.util.A4;
 import javajs.util.AU;
+import javajs.util.BS;
 import javajs.util.CU;
 import javajs.util.DF;
 import javajs.util.Lst;
@@ -40,14 +48,6 @@ import javajs.util.Quat;
 import javajs.util.T3;
 import javajs.util.V3;
 
-import javajs.util.BS;
-import org.jmol.modelset.BondSet;
-import org.jmol.util.BSUtil;
-import org.jmol.util.BoxInfo;
-import org.jmol.util.Escape;
-import org.jmol.util.Logger;
-import org.jmol.viewer.Viewer;
-
 /**
  * Reverse Polish Notation Engine for IF, SET, and @{...}
  * 
@@ -663,16 +663,26 @@ public class ScriptMathProcessor {
     case T.opOr:
       if (!wasSyntaxCheck && xPt < 0)
         return false;
-      if (!wasSyntaxCheck && xStack[xPt].tok != T.bitset
-          && xStack[xPt].tok != T.varray) {
-        // check to see if we need to evaluate the second operand or not
-        // if not, then set this to syntax check in order to skip :)
-        // Jmol 12.0.4, Jmol 12.1.2
-        boolean tf = getX().asBoolean();
-        addX(SV.getBoolean(tf));
-        if (tf == (op.tok == T.opOr)) { // TRUE or.. FALSE and...
-          chk = true;
-          op = (op.tok == T.opOr ? T.tokenOrTRUE : T.tokenAndFALSE);
+      if (!wasSyntaxCheck) {
+        switch (xStack[xPt].tok) {
+        case T.bitset:
+        case T.varray:
+          break;
+        case T.matrix3f: 
+          if (op.tok == T.opOr)
+            break;
+          //$FALL-THROUGH$
+        default:
+          // check to see if we need to evaluate the second operand or not
+          // if not, then set this to syntax check in order to skip :)
+          // Jmol 12.0.4, Jmol 12.1.2
+          boolean tf = getX().asBoolean();
+          addX(SV.getBoolean(tf));
+          if (tf == (op.tok == T.opOr)) { // TRUE or.. FALSE and...
+            chk = true;
+            op = (op.tok == T.opOr ? T.tokenOrTRUE : T.tokenAndFALSE);
+          }
+          break;
         }
       }
       wasX = false;
@@ -912,13 +922,13 @@ public class ScriptMathProcessor {
         //System.out.println("ptx="+ ptx + " " + pto);
         if (ptx < pto) {
           // x++ must make a copy first
-          x1 = SV.newS("").setv(x2);
+          x1 = SV.copySafely(x2);
         }
         if (!x2.increment(op.tok == T.plusPlus ? 1 : -1))
           return false;
         if (ptx > pto) {
           // ++x must make a copy after
-          x1 = SV.newS("").setv(x2);
+          x1 = SV.copySafely(x2);
         }
       }
       wasX = false;
@@ -939,11 +949,11 @@ public class ScriptMathProcessor {
         return addXPt4(pt4);
       case T.matrix3f:
         m = M3.newM3((M3) x2.value);
-        m.invert();
+        m.transpose();
         return addXM3(m);
       case T.matrix4f:
         m4 = M4.newM4((M4) x2.value);
-        m4.invert();
+        m4.transpose();
         return addXM4(m4);
       case T.bitset:
         return addXBs(BSUtil.copyInvert((BS) x2.value,
@@ -963,8 +973,10 @@ public class ScriptMathProcessor {
       case T.matrix4f:
         return addXM4(M4.newM4((M4) x2.value).invert());
       case T.bitset:
-        return addXBs(BSUtil.copyInvert((BS) x2.value,
-            (x2.value instanceof BondSet ? vwr.ms.bondCount : vwr.ms.ac)));
+        boolean isBond = x2.value instanceof BondSet;
+        BS bs = BSUtil.copyInvert((BS) x2.value,
+            (isBond ? vwr.ms.bondCount : vwr.ms.ac));
+        return addXBs(isBond ? BondSet.newBS(bs) : bs);
       default:
         return addXBool(!x2.asBoolean());
       }
@@ -1078,7 +1090,6 @@ public class ScriptMathProcessor {
     M3 m;
     String s;
     float f;
-
     switch (op.tok) {
     case T.opAND:
     case T.opAnd:
@@ -1092,7 +1103,7 @@ public class ScriptMathProcessor {
         case T.bitset:
           bs = BSUtil.copy(bs);
           bs.and((BS) x2.value);
-          return addXBs(bs);
+          return addXBs(x1.value instanceof BondSet ? BondSet.newBS(bs) : bs);
         }
         break;
       }
@@ -1100,36 +1111,46 @@ public class ScriptMathProcessor {
     case T.opOr:
       switch (x1.tok) {
       case T.bitset:
-        BS bs = BSUtil.copy((BS) x1.value);
+        BS bs = null;
         switch (x2.tok) {
         case T.bitset:
+          bs = BSUtil.copy((BS) x1.value);
           bs.or((BS) x2.value);
-          return addXBs(bs);
+          break;
         case T.integer:
+          bs = BSUtil.copy((BS) x1.value);
           int x = x2.asInt();
-          if (x < 0)
-            break;
-          bs.set(x);
-          return addXBs(bs);
+          if (x >= 0) {
+            bs.set(x);
+          }
+          break;
         case T.varray:
+          bs = BSUtil.copy((BS) x1.value);
           Lst<SV> sv = (Lst<SV>) x2.value;
           for (int i = sv.size(); --i >= 0;) {
             int b = sv.get(i).asInt();
             if (b >= 0)
               bs.set(b);
           }
-          return addXBs(bs);
+          break;
         }
-        break;
+        if (bs == null) 
+          break;
+        return addXBs(x1.value instanceof BondSet ? BondSet.newBS(bs) : bs);
       case T.varray:
         return addX(SV.concatList(x1, x2, false));
+      case T.matrix3f:
+        if (x2.tok == T.point3f) {
+          // M3 | P3 => M4  (BCS nomenclature)
+          return addXM4(M4.newMV((M3) x1.value, (P3) x2.value));
+        }
       }
       return addXBool(x1.asBoolean() || x2.asBoolean());
     case T.opXor:
       if (x1.tok == T.bitset && x2.tok == T.bitset) {
         BS bs = BSUtil.copy((BS) x1.value);
         bs.xor((BS) x2.value);
-        return addXBs(bs);
+        return addXBs(x1.value instanceof BondSet ? BondSet.newBS(bs) : bs);
       }
       boolean a = x1.asBoolean();
       boolean b = x2.asBoolean();
@@ -1350,9 +1371,12 @@ public class ScriptMathProcessor {
           Lst<SV> l = x2.getList();
           Lst<P3> lnew = new Lst<P3>();
           for (int i = l.size(); --i >= 0;) {
-            P3 pt1 = P3.newP(SV.ptValue(l.get(i)));
+            P3 pt1 = SV.ptValue(l.get(i));
+            if (pt1 == null)
+              return false;
+            pt1 = P3.newP(pt1);
             m3.rotate(pt1);
-            lnew.add(pt1);
+            lnew.addLast(pt1);
           }
           return addXList(lnew);
         }
@@ -1574,7 +1598,17 @@ public class ScriptMathProcessor {
           return addXM3(q.getMatrix());
         default:
           return addXStr("NaN");
-        }
+        }        
+      case T.matrix3f:
+          M3 mm = (M3) x1.value;
+          switch (n) {
+          case 1:
+            return addXFloat(mm.m00 + mm.m11 + mm.m22);
+          case 2:
+            return addXFloat(mm.determinant3());
+          default:
+            return false;
+      }
       case T.matrix4f:
         M4 m4 = (M4) x1.value;
         switch (n) {
@@ -1611,13 +1645,13 @@ public class ScriptMathProcessor {
       return (P3) x.value;
     case T.bitset:
       BS bs = (BS) x.value;
-      if (bs.isEmpty())
+      if (bs.isEmpty() || bs instanceof BondSet)
         break;
       if (bsRestrict != null) {
         bs = BSUtil.copy(bs);
         bs.and(bsRestrict);
       }
-      Object o = (bs.cardinality() == 0 ? null : eval.getBitsetProperty(bs, null, T.xyz, null,
+      Object o = (bs.isEmpty() ? null : eval.getBitsetProperty(bs, null, T.xyz, null,
           null, x.value, null, false, Integer.MAX_VALUE, false));
 //      if (o != null && !(o instanceof P3))
 //        System.out.println("ScriptMathProc OHOH");
@@ -1693,7 +1727,7 @@ public class ScriptMathProcessor {
   }
 
   private boolean getBoundBox(SV x2) {
-    if (x2.tok != T.bitset)
+    if (x2.tok != T.bitset || x2.value instanceof BondSet)
       return false;
     BoxInfo b = vwr.ms.getBoxInfo((BS) x2.value, 1);
     P3[] pts = b.getBoundBoxPoints(true);
@@ -1758,6 +1792,8 @@ public class ScriptMathProcessor {
         return (op.intValue == T.fracxyz ? addXPt(ptf)
             : addXFloat(op.intValue == T.fracx ? ptf.x
                 : op.intValue == T.fracy ? ptf.y : ptf.z));
+      case T.rxyz:
+        return addXStr(vwr.getSymStatic().staticToRationalXYZ(P3.newP((P3) x2.value), " "));
       case T.fux:
       case T.fuy:
       case T.fuz:
@@ -1799,15 +1835,14 @@ public class ScriptMathProcessor {
       if (!isAtoms && x2.value instanceof BondSet)
         return addX(x2);
       BS bs = (BS) x2.value;
+      // return just value for @3.xxx
       if (isAtoms && bs.cardinality() == 1 && (op.intValue & T.minmaxmask) == 0)
         op.intValue |= T.min;
       Object val = eval.getBitsetProperty(bs, null, op.intValue, null,
           null, null, op.value, false, x2.index, true);
-      return (isAtoms ? addXObj(val) : addX(SV.newV(T.bitset, BondSet.newBS(
-          (BS) val, vwr.ms.getAtomIndices(bs)))));
+      return (isAtoms ? addXObj(val) : addXBs(BondSet.newBS((BS) val)));
     }
     return false;
   }
 
-  
 }
