@@ -106,20 +106,48 @@ abstract public class ScriptParam extends ScriptError {
   }
 
   public SV getContextVariableAsVariable(String var, boolean isLocal) {
-    if (var.equals("expressionBegin"))
+    if (var.length() == 0 || var.equals("expressionBegin"))
       return null;
-    if (var.equalsIgnoreCase("_caller")) {
+    if (var.charAt(0) == '_') {
+      boolean isCallers = var.equalsIgnoreCase("_callers");
+      if (isCallers || var.equalsIgnoreCase("_caller")) {
+        ScriptContext sc = thisContext;
+        Hashtable<String, Object> h0 = new Hashtable<String, Object>(),
+            h = null, h1;
+        while (sc != null) {
+          if (sc.isFunction) {
+            if (h == null) {
+              h = h0;
+            } else {
+              h1 = new Hashtable<String, Object>();
+              h.put("_caller", SV.newV(T.hash, h1));
+              h = h1;
+            }
+            h.putAll(sc.vars);
+            h.put("_name", SV.newS(sc.functionName));
+            if (!isCallers)
+              break;
+          }
+          sc = sc.parentContext;
+        }
+        return SV.newV(T.hash, h0);
+      }
+      if (var.equalsIgnoreCase("_name")) {
       ScriptContext sc = thisContext;
       while (sc != null) {
-        if (sc.isFunction)
-          return SV.newV(T.hash, sc.vars);
+          if (sc.isFunction) {
+            return SV.newS(SV.sValue(sc.statement[0]));
+          }
         sc = sc.parentContext;
       }
-      return SV.newV(T.hash, new Hashtable<String, Object>());
+        return SV.newS("");
+      }
     }
     var = var.toLowerCase();
-    return (contextVariables != null && contextVariables.containsKey(var) ? contextVariables
-        .get(var) : isLocal || thisContext == null ? null : thisContext.getVariable(var));
+    SV v = (contextVariables == null ? null : contextVariables.get(var));
+    if (v == null && !isLocal && thisContext != null)
+      v = thisContext.getVariable(var);
+    return (v != null && v.tok == T.array ? v.arrayToList(v) : v);
   }
   
   public String paramAsStr(int i) throws ScriptException {
@@ -242,18 +270,7 @@ abstract public class ScriptParam extends ScriptError {
     if (checkToken(i)) {
       switch (getToken(i).tok) {
       case T.unitcell:
-        center = new P3();
-        SymmetryInterface uc = vwr.getCurrentUnitCell();
-        if (uc != null) {
-          P3[] pts = uc.getUnitCellVerticesNoOffset();
-          P3 off = uc.getCartesianOffset();
-          for (int j = 0; j < 8; j++) {
-            center.add(pts[j]);
-            center.add(off);
-          }
-        }
-        center.scale(1f/8);
-        break;
+        return getUnitCellCenter();
       case T.dollarsign:
         String id = objectNameParameter(++i);
         int index = Integer.MIN_VALUE;
@@ -288,6 +305,21 @@ abstract public class ScriptParam extends ScriptError {
     return center;
   }
 
+  public P3 getUnitCellCenter() {
+    P3 center = new P3();
+    SymmetryInterface uc = vwr.getCurrentUnitCell();
+    if (uc != null) {
+      P3[] pts = uc.getUnitCellVerticesNoOffset();
+      P3 off = uc.getCartesianOffset();
+      for (int j = 0; j < 8; j++) {
+        center.add(pts[j]);
+        center.add(off);
+      }
+    }
+    center.scale(1f/8);
+    return center;
+  }
+
   public P4 planeParameter(int i, boolean isBest) throws ScriptException {
     V3 vTemp = new V3();
     V3 vTemp2 = new V3();
@@ -1010,7 +1042,7 @@ abstract public class ScriptParam extends ScriptError {
         invArg();
       return Quat.newP4(p4);
     case T.best:
-      return (chk ? null : (Quat) vwr.getOrientation(T.best, (divideByCurrent ? "best" : ""), bsAtoms));
+      return (chk ? null : (Quat) vwr.getOrientation(T.best, (divideByCurrent ? "best" : ""), bsAtoms, null));
     default:
       return Quat.newP4(getPoint4f(i));
     }
@@ -1142,11 +1174,12 @@ abstract public class ScriptParam extends ScriptError {
       Lst<P3> data = new Lst<P3>();
       P3 pt;
       Lst<SV> pts = ((SV) t).getList();
-      for (int j = 0; j < pts.size(); j++)
-        if ((pt = SV.ptValue(pts.get(j))) != null)
-          data.addLast(pt);
-        else
+      for (int j = 0; j < pts.size(); j++) {
+        SV v = pts.get(j);
+        if ((pt = (v.tok == T.bitset ? vwr.ms.getAtomSetCenter(SV.getBitSet(v, false)) : SV.ptValue(v))) == null)
           return null;
+          data.addLast(pt);
+      }
       return data;
     }
     if (i > 0)
@@ -1305,7 +1338,7 @@ abstract public class ScriptParam extends ScriptError {
     }
     if (tokAt(++index) == T.none) {
       tickInfo = new TickInfo(null);
-      tickInfo.type = str;
+      tickInfo.type = str.charAt(0);
       iToken = index;
       return tickInfo;
     }
@@ -1316,7 +1349,7 @@ abstract public class ScriptParam extends ScriptError {
     }
     if (tokAt(iToken + 1) == T.unitcell)
       iToken++;
-    tickInfo.type = str;
+    tickInfo.type = str.charAt(0);
     if (tokAt(iToken + 1) == T.format)
       tickInfo.tickLabelFormats = stringParameterSet(iToken + 2);
     if (!allowScale)
@@ -1473,6 +1506,38 @@ abstract public class ScriptParam extends ScriptError {
     return (pt instanceof P4 ? (P4) pt : null);
   }
 
+  /**
+   * @param params
+   * @param isCallback unused to date  
+   */
+  protected void setScriptArguments(Object params, boolean isCallback) {
+    if (contextVariables == null)
+      contextVariables = new Hashtable<String, SV>();
+    contextVariables.put("_arguments",
+        (params == null ? SV.getVariableAI(new int[] {})
+            : isCallback ? SV.newV(T.array,  params) : SV.getVariableList((Lst<?>)params)));
+    contextVariables.put("_argcount",
+        SV.newI(params == null ? 0 : isCallback ? ((Object[]) params).length : ((Lst<?>)params).size()));
+  }
 
-
+  public SV getCallbackParameter(int n) {
+	    SV v = (contextVariables == null ? null
+	        : contextVariables.get("_arguments"));
+	    if (v == null && (thisContext == null
+	        || (v = thisContext.getVariable("_arguments")) == null))
+	      return null;
+	    if (v.tok == T.array) {
+	      if (n == Integer.MIN_VALUE) {
+	        return v.arrayToList(v);
+	      }
+	      Object[] params = (Object[]) v.value;
+	      return (--n < 0 || n >= params.length ? null
+	          : (SV) (params[n] = SV.getVariable(params[n])));
+	    }
+	    Lst<SV> list = v.getList();
+	    return (n == Integer.MIN_VALUE ? v 
+	        : --n < 0 || n >= list.size() ? 
+	            SV.newV(T.varray,  new Lst<SV>()) : list.get(n));
+	  }
+	   
 }
