@@ -68,14 +68,14 @@ public class Draw extends MeshCollection {
   private final static int PT_MODEL_INDEX = 4;
   private final static int PT_MODEL_BASED_POINTS = 5;
 
+  DrawMesh[] dmeshes = new DrawMesh[4];
+  private DrawMesh thisMesh;
+  
   public Draw() {
     // from reflection
     htObjects = new Hashtable<String, Mesh>();
   }
 
-  DrawMesh[] dmeshes = new DrawMesh[4];
-  private DrawMesh thisMesh;
-  
   @Override
   public void allocMesh(String thisID, Mesh m) {
     int index = meshCount++;
@@ -95,10 +95,13 @@ public class Draw extends MeshCollection {
     setPropMC(propertyName, value, bs);
     thisMesh = (DrawMesh)currentMesh;  
   }
-  
+
+  /**
+   * @j2sOverride
+   */
  @Override
-public void initShape() {
-    super.initShape();
+  public void initShape() {
+    setMeshColor();
     myType = "draw";
   }
   
@@ -127,6 +130,7 @@ public void initShape() {
   private boolean noHead;
   private boolean isBarb;
   private int indicatedModelIndex = -1;
+  private boolean indicatedModelOnly;
   private int[] modelInfo;
   private boolean makePoints;
   //private int nidentifiers;
@@ -169,7 +173,7 @@ public void initShape() {
     }
     
     if ("length" == propertyName) {
-      length = ((Float) value).floatValue();
+      length = ((Number) value).floatValue();
       return;
     }
 
@@ -222,9 +226,11 @@ public void initShape() {
         return;
       vData.addLast(new Object[] { Integer.valueOf(PT_MODEL_INDEX),
           (modelInfo = new int[] { indicatedModelIndex, 0 }) });
-      return;
+      if (thisMesh.thisID.startsWith(JC.THIS_MODEL_ONLY))
+        indicatedModelOnly = true;
+        return;
     }
-
+    
     if ("planedef" == propertyName) {
       plane = (P4) value;
       if (intersectID != null || slabData != null)
@@ -338,12 +344,12 @@ public void initShape() {
     }
 
     if ("diameter" == propertyName) {
-      diameter = ((Float) value).intValue();
+      diameter = ((Number) value).intValue();
       return;
     }
 
     if ("width" == propertyName) {
-      width = ((Float) value).floatValue();
+      width = ((Number) value).floatValue();
       return;
     }
 
@@ -403,6 +409,11 @@ public void initShape() {
       addPoints(PT_MODEL_BASED_POINTS, value);
       return;
     }
+
+    if ("hoverlabel" == propertyName) {
+      propertyName = "title";
+      value = "<hover>" + value;
+    }
     
     if ("title" == propertyName) {
       if (thisMesh != null) {
@@ -449,7 +460,7 @@ public void initShape() {
       return;
     }
 
-    if (propertyName == "deleteModelAtoms") {
+    if (propertyName == JC.PROP_DELETE_MODEL_ATOMS) {
       deleteModels(((int[]) ((Object[]) value)[2])[0]);
       return;
     }
@@ -481,6 +492,25 @@ public void initShape() {
     resetObjects();
   }
 
+  @Override
+  protected String[] setTitle(Object value) {
+    if (value instanceof String) {
+      String lines = (String) value;
+        if (lines.startsWith("<hover>")) {
+          String s = lines.substring(7);
+          int i = s.indexOf("</hover>");
+          if (i >= 0) {
+            thisMesh.hoverLabel = s.substring(0, i);
+            value = s.substring(i + 8);
+          } else {
+            thisMesh.hoverLabel = s;
+            value = "";
+          }
+        }
+    }
+    return super.setTitle(value);
+  }  
+
  private void deleteMeshElement(int i) {
    if (meshes[i] == currentMesh)
      currentMesh = thisMesh = null;
@@ -496,6 +526,7 @@ private void initDraw() {
    explicitID = false;
    thisFontID = -1;
    indicatedModelIndex = -1;
+   indicatedModelOnly = false;
    intersectID = null;
    isCurve = isArc = isArrow = isPlane = isCircle = isCylinder = isLine = false;
    //isBest = 
@@ -683,6 +714,7 @@ private void initDraw() {
       thisMesh.vc = 0;
       if (indicatedModelIndex >= 0) {
         setPoints(-1, 0);
+        thisMesh.thisModelOnly = indicatedModelOnly;
         thisMesh.drawType = EnumDrawType.MULTIPLE;
         thisMesh.drawVertexCount = -1;
         thisMesh.modelFlags.set(indicatedModelIndex);
@@ -775,7 +807,7 @@ private void initDraw() {
       if (newPt == null || iModel >= 0 && !bsAllModels.get(iModel))
         return;
       ptList[nPoints] = P3.newP(newPt);
-      if (newPt.z == Float.MAX_VALUE || newPt.z == -Float.MAX_VALUE)
+      if (is2DPoint(newPt))
         thisMesh.haveXyPoints = true;
     } else if (iModel >= 0) {
       bsAllModels.set(iModel);
@@ -817,13 +849,15 @@ private void initDraw() {
         int n0 = thisMesh.vc;
         if (nPoints > 0) {
           int[] p = thisMesh.pis[modelIndex] = new int[nVertices];
+          P3 pt = null;
           for (int j = 0; j < nPoints; j++) {
             info = vData.get(++i);
-            p[j] = thisMesh.addV((P3) info[1], false);
+            p[j] = thisMesh.addV(pt = (P3) info[1], false);
           }
           for (int j = nPoints; j < 3; j++) {
             p[j] = n0 + nPoints - 1;
           }
+          thisMesh.haveXyPoints = is2DPoint(pt);
           thisMesh.drawTypes[modelIndex] = EnumDrawType.getType(nPoints);
           thisMesh.drawVertexCounts[modelIndex] = nPoints;
           thisMesh.modelFlags.set(modelIndex);
@@ -1215,7 +1249,6 @@ private void initDraw() {
         m.bsMeshesVisible.or(m.modelFlags);
         m.bsMeshesVisible.and(bsModels);
       }
-
     }
   }
   
@@ -1276,7 +1309,8 @@ private void initDraw() {
       return false;
     if (!findPickedObject(x, y, false, bsVisible))
       return false;
-    String s = (pickedMesh.title == null ? pickedMesh.thisID
+    String s = (((DrawMesh) pickedMesh).hoverLabel != null ? ((DrawMesh) pickedMesh).hoverLabel 
+        : pickedMesh.title == null ? pickedMesh.thisID
         : pickedMesh.title[0]);
     if (s.length() > 1 && s.charAt(0) == '>')
       s = s.substring(1);
@@ -1357,6 +1391,10 @@ private void initDraw() {
     mesh.setCenters();
   }
   
+  private Mesh pm2;
+  private int dmin22, pmod2, pickedVertex2;
+  private T3 pickedPoint2;
+
   /**
    * 
    * @param x
@@ -1367,50 +1405,89 @@ private void initDraw() {
    * @return true if found
    */
   private boolean findPickedObject(int x, int y, boolean isPicking, BS bsVisible) {
-    int dmin2 = MAX_OBJECT_CLICK_DISTANCE_SQUARED;
-    if (vwr.gdata.isAntialiased()) {
-      x <<= 1;
-      y <<= 1;
-      dmin2 <<= 1;
-    }
-    pickedModel = 0;
-    pickedVertex = 0;
-    pickedMesh = null;
-    for (int i = 0; i < meshCount; i++) {
-      DrawMesh m = dmeshes[i];
-      if (m.visibilityFlags != 0) {
-        int mCount = (m.isDrawPolygon ? m.pc
-            : m.modelFlags == null ? 1 : ms.mc);
-        for (int iModel = mCount; --iModel >= 0;) {
-          if (m.modelFlags != null
-              && !m.modelFlags.get(iModel)
-              || m.pis == null
-              || !m.isDrawPolygon
-              && (iModel >= m.pis.length || m.pis[iModel] == null))
-            continue;
-          for (int iVertex = (m.isDrawPolygon ? 3
-              : m.pis[iModel].length); --iVertex >= 0;) {
-            try {
-              int iv = m.pis[iModel][iVertex];
-              T3 pt = (m.altVertices == null ? m.vs[iv]
-                  : (P3) m.altVertices[iv]);
-              int d2 = coordinateInRange(x, y, pt, dmin2, ptXY);
-              if (d2 >= 0) {
-                pickedMesh = m;
-                dmin2 = d2;
-                pickedModel = iModel;
-                pickedVertex = iVertex;
-                pickedPt = pt;
-              }
-            } catch (Exception e) {
-              System.out.println(e);
-            }
-          }
-        }
-      }
-    }
-    return (pickedMesh != null);
-  }
+	    int dmin2 = MAX_OBJECT_CLICK_DISTANCE_SQUARED;
+	    if (vwr.gdata.isAntialiased()) {
+	      x <<= 1;
+	      y <<= 1;
+	      dmin2 <<= 1;
+	    }
+	    pickedModel = pickedVertex = 0;
+	    pickedMesh = null;
+	    pickedPt = null;
+	    pm2 = null;
+	    pickedPoint2 = null;
+	    dmin22 = pmod2 = pickedVertex2 = -1;
+	    for (int i = 0; i < meshCount; i++) {
+	      DrawMesh m = dmeshes[i];
+	      if (m.visibilityFlags == 0)
+	        continue;
+	      dmin2 = pickClosestPoint(dmin2, x, y, m, -1, -1, ptXY, null);
+	    }
+	    if (dmin2 != 0 && dmin22 == dmin2 && pickedMesh != null && pm2 != null) {
+	      V3 vTemp = new V3();
+	      int d1 = pickClosestPoint(Integer.MAX_VALUE, x, y, (DrawMesh) pickedMesh, pickedModel, pickedVertex, ptXY, vTemp);
+	      int d2 = pickClosestPoint(d1, x, y, (DrawMesh) pm2, pmod2, pickedVertex2, ptXY, vTemp);
+	      
+	      if (d2 < d1) {
+	        pickedMesh = pm2;
+	        pickedModel = pmod2;
+	        pickedVertex = pickedVertex2;
+	        pickedPt = pickedPoint2;
+	      }
+	    }
+	    return (pickedMesh != null);
+	  }
+
+	  private int pickClosestPoint(int dmin2, int x, int y, DrawMesh m, int model,
+	                               int vnot, P3i ptXY, V3 vTemp) {
+	    int mCount = (m.isDrawPolygon ? m.pc : m.modelFlags == null ? 1 : ms.mc);
+	    for (int iModel = mCount; --iModel >= 0;) {
+	      if (model >= 0 ? iModel != model
+	          : m.modelFlags != null && !m.modelFlags.get(iModel) || m.pis == null
+	              || !m.isDrawPolygon
+	                  && (iModel >= m.pis.length || m.pis[iModel] == null))
+	        continue;
+	      for (int iVertex = (m.isDrawPolygon ? 3
+	          : m.pis[iModel].length); --iVertex >= 0;) {
+	        if (iVertex == vnot)
+	          continue;
+	        try {
+	          int iv = m.pis[iModel][iVertex];
+	          T3 pt = (m.altVertices == null ? m.vs[iv] : (P3) m.altVertices[iv]);
+	          // just looking for the direction here. 
+	          if (vnot >= 0) {
+	            vTemp.sub2(pt, pickedPt);
+	            if (vTemp.length() == 0)
+	              continue;
+	            vTemp.normalize();
+	            vTemp.add(pickedPt);
+	          }
+	          int d2 = coordinateInRange(x, y, (vnot >= 0 ? vTemp : pt), dmin2,
+	              ptXY);
+	          if (d2 >= 0) {
+	            if (vnot < 0) {
+	              if (pm2 == null) {
+	                dmin22 = d2;
+	                pmod2 = pickedModel;
+	                pm2 = pickedMesh;
+	                pickedVertex2 = pickedVertex;
+	                pickedPoint2 = pickedPt;
+	              }
+	              pickedVertex = iVertex;
+	              pickedPt = pt;
+	              pickedMesh = m;
+	              pickedModel = iModel;
+	            }
+	            dmin2 = d2;
+	          }
+	        } catch (Exception e) {
+	          System.out.println(e);
+	        }
+	      }
+	    }
+	    return dmin2;
+	  }
+
 
   private String getCommand(Mesh mesh) {
     if (mesh != null)
@@ -1556,11 +1633,16 @@ private void initDraw() {
       str.append(" title color ")
           .append(Escape.escapeColor(dmesh.titleColor.intValue()));
     }
+    String title = "";
     if (dmesh.title != null) {
-      String s = "";
       for (int i = 0; i < dmesh.title.length; i++)
-        s += "|" + dmesh.title[i];
-      str.append(" ").append(PT.esc(s.substring(1)));
+        title += "|" + dmesh.title[i];
+    }
+    if (dmesh.hoverLabel != null) {
+      title = "|<hover>" + dmesh.hoverLabel + "</hover>" + (title == "" ? "" : title.substring(1));
+    }
+    if (title.length() > 1) {
+      str.append(" ").append(PT.esc(title.substring(1)));
     }
     if (dmesh.fontID >= 0) {
       str.append(" font " + Font.getFont3D(dmesh.fontID).getInfo());
@@ -1585,7 +1667,7 @@ private void initDraw() {
       boolean adjustPt = (mesh.isVector && mesh.drawType != EnumDrawType.ARC);
       for (int i = 0; i < nVertices; i++) {
         T3 pt = mesh.vs[mesh.pis[iModel][i]];
-        if (pt.z == Float.MAX_VALUE || pt.z == -Float.MAX_VALUE) {
+        if (is2DPoint(pt)) {
           str += (i == 0 ? " " : " ,") + "[" + (int) pt.x + " " + (int) pt.y + (pt.z < 0 ? " %]" : "]");
         } else if (adjustPt && i == 1){
           P3 pt1 = P3.newP(pt);
@@ -1601,6 +1683,10 @@ private void initDraw() {
     return str;
   }
   
+  public static boolean is2DPoint(T3 pt) {
+    return (pt.z == Float.MAX_VALUE || pt.z == -Float.MAX_VALUE);
+  }
+
   @Override
   public Object getShapeDetail() {
     Lst<Map<String, Object>> V = new  Lst<Map<String,Object>>();
