@@ -595,7 +595,7 @@ public class Isosurface extends MeshCollection implements MeshDataServer {
         BS bsSets;
         if (value instanceof BS) {
           bsSets = ((BS) value);
-          if (bsSets.cardinality() == 0)
+          if (bsSets.isEmpty())
             bsSets = null;
         } else {
           bsSets = new BS();
@@ -712,7 +712,7 @@ public class Isosurface extends MeshCollection implements MeshDataServer {
      * meshes[i].hideBackground = doHide; } return; }
      */
 
-    if (propertyName == "deleteModelAtoms") {
+    if (propertyName == JC.PROP_DELETE_MODEL_ATOMS) {
       int modelIndex = ((int[]) ((Object[]) value)[2])[0];
       int firstAtomDeleted = ((int[]) ((Object[]) value)[2])[1];
       int nAtomsDeleted = ((int[]) ((Object[]) value)[2])[2];
@@ -1120,7 +1120,7 @@ public class Isosurface extends MeshCollection implements MeshDataServer {
       cmd += " ATOMINDEX " + imesh.atomIndex;
     appendCmd(sb, cmd);
     String id = myType + " ID " + PT.esc(imesh.thisID);
-    if (imesh.jvxlData.thisSet != null && imesh.jvxlData.thisSet.cardinality() > 0) {
+    if (imesh.jvxlData.thisSet != null && !imesh.jvxlData.thisSet.isEmpty()) {
       appendCmd(sb, id + (imesh.jvxlData.thisSet.cardinality() == 1 ? " set " + (imesh.jvxlData.thisSet.nextSetBit(0)+1)
       : " subset " + imesh.jvxlData.thisSet));
     }
