@@ -1,7 +1,7 @@
 /* $RCSfile$
  * $Author: hansonr $
- * $Date: 2022-03-13 21:21:32 -0500 (Sun, 13 Mar 2022) $
- * $Revision: 22369 $
+ * $Date: 2023-01-25 16:14:30 -0600 (Wed, 25 Jan 2023) $
+ * $Revision: 22508 $
  *
  * Copyright (C) 2005  The Jmol Development Team
  *
@@ -195,6 +195,8 @@ public class SmilesAtom extends P3 implements Node {
   private int missingHydrogenCount;
   private int cipChirality;
 
+  boolean hasDoubleBond;
+
   public SmilesAtom setTopoAtom(int iComponent, int ptAtom, String symbol,
       int charge, int patternIndex) {
     component = iComponent;
@@ -221,7 +223,8 @@ public class SmilesAtom extends P3 implements Node {
     if (explicitHydrogenCount != Integer.MIN_VALUE)
       return true;
     // Determining max count
-    int count = getDefaultCount(elementNumber, isAromatic);
+    int charge = getFormalCharge();
+    int count = getDefaultHCount(elementNumber, isAromatic, charge == Integer.MIN_VALUE ? 0 : charge);
     if (count < 0) {
       missingHydrogenCount = 0;
       return (count == -1);
@@ -264,7 +267,7 @@ public class SmilesAtom extends P3 implements Node {
     return true;
   }
 
-  static int getDefaultCount(int elementNumber, boolean isAromatic) {
+  static int getDefaultHCount(int elementNumber, boolean isAromatic, int charge) {
     // not a complete set...
     // B, C, N, O, P, S, F, Cl, Br, and I
     // B (3), C (4), N (3,5), O (2), P (3,5), S (2,4,6), and 1 for the halogens
@@ -277,14 +280,14 @@ public class SmilesAtom extends P3 implements Node {
     case 1:
       return 1;
     case 6: // C
-      return (isAromatic ? 3 : 4);
+      return (isAromatic ? 3 : 4) + (charge == 0 ? 0 : 1);
     case 8: // O
     case 16: // S
-      return 2;
+      return 2 + charge;
     case 7: // N
       // note -- it is necessary to indicate explicitly
       // single bonds to aromatic n if a proper MF is desired
-      return (isAromatic ? 2 : 3);
+      return (isAromatic ? 2 : 3) + charge;
     case 5: // B
     case 15: // P
       return 3;
@@ -292,12 +295,11 @@ public class SmilesAtom extends P3 implements Node {
     case 17: // Cl
     case 35: // Br
     case 53: // I
-      return 1;
+      return 1 + charge;
     default:
       // unbracketed atom
       return 0; 
     }
-    //return -2;
   }
 
   /**
@@ -536,6 +538,9 @@ public class SmilesAtom extends P3 implements Node {
     //if (Logger.debugging)
     //Logger.debug("adding bond " + bondCount + " to " + this + ": " + bond.atom1 + " " + bond.atom2);
     bonds[bondCount] = bond;
+    if (bond.order == 2) {
+      hasDoubleBond = true;
+    }
     bondCount++;
   }
 
@@ -546,8 +551,11 @@ public class SmilesAtom extends P3 implements Node {
       subAtoms = (SmilesAtom[]) AU.arrayCopyObject(subAtoms, subAtoms.length);
     for (int i = 0; i < bonds.length; i++) {
       SmilesBond b = bonds[i];
-      if (isBioAtom && b.getBondType() == SmilesBond.TYPE_AROMATIC)
+      if (isBioAtom && b.getBondType() == SmilesBond.TYPE_AROMATIC) {
         b.order = SmilesBond.TYPE_BIO_CROSSLINK;
+      } else if (b.order == 2) {
+        b.atom1.hasDoubleBond = b.atom2.hasDoubleBond = true;
+      }
       if (b.atom1.index > b.atom2.index) {
         // it is possible, particularly for a connection to a an atom 
         // with a branch:   C1(CCCN1)
@@ -781,13 +789,13 @@ public class SmilesAtom extends P3 implements Node {
                              String stereo, boolean is2D) {
     String sym = Elements.elementSymbolFromNumber(atomicNumber);
     if (atomicNumber == 1 
-        || isAromatic && !(sym = sym.toLowerCase()).equals("c")) {
+        || isAromatic && !(sym = sym.toLowerCase()).equals("c") && !sym.equals("o") && !sym.equals("s")) {
       // force [n]
       valence = Integer.MAX_VALUE;
     }
     boolean simple = (valence != Integer.MAX_VALUE && isotopeNumber <= 0 
         && charge == 0 && Float.isNaN(osclass) && (stereo == null || stereo.length() == 0)); 
-    int norm = getDefaultCount(atomicNumber, false);
+    int norm = getDefaultHCount(atomicNumber, false, charge == Integer.MIN_VALUE ? 0 : charge);
     if (is2D && nH == 0) {
       if (simple && atomicNumber == 6)
         return sym;    
@@ -800,8 +808,8 @@ public class SmilesAtom extends P3 implements Node {
         + sym
         + (stereo == null ? "" : stereo)
         + (nH > 1 ? "H" + nH : nH == 1 ? "H" : "")
-        + (charge < 0 && charge != Integer.MIN_VALUE ? "" + charge 
-            : charge > 0 ? "+" + charge : "") 
+        + (charge == Integer.MIN_VALUE || charge ==0 ? "" : charge < 0 ? "" + charge 
+                : "+" + charge) 
         + (Float.isNaN(osclass) ? "" : ":" + (int) osclass)
         + "]");
   }
