@@ -106,6 +106,8 @@ public class SmilesGenerator {
   private int iHypervalent;
   private boolean is2D;
   private boolean haveSmilesAtoms;
+  private boolean noBranches; // testing only -- leave false
+  private boolean allComponents;
 
   // generation of SMILES strings
 
@@ -188,6 +190,8 @@ public class SmilesGenerator {
     isPolyhedral = ((flags
         & JC.SMILES_GEN_POLYHEDRAL) == JC.SMILES_GEN_POLYHEDRAL);
     is2D = ((flags & JC.SMILES_2D) == JC.SMILES_2D);
+    noBranches = false;//((flags & JC.SMILES_GEN_NO_BRANCHES) == JC.SMILES_GEN_NO_BRANCHES);
+    allComponents = ((flags & JC.SMILES_GEN_ALL_COMPONENTS) == JC.SMILES_GEN_ALL_COMPONENTS);
     return getSmilesComponent(atoms[ipt], bsSelected, true, false, false);
   }
 
@@ -352,6 +356,12 @@ public class SmilesGenerator {
                                     boolean allowConnectionsToOutsideWorld,
                                     boolean forceBrackets)
       throws InvalidSmilesException {
+	  
+	    String ret = "";
+
+	    while (true) {
+	        if (atom == null)
+	            atom = atoms[bs.nextSetBit(0)];
 
     atom = checkFirstAtom(atom);
     
@@ -360,21 +370,23 @@ public class SmilesGenerator {
     bs.andNot(bsSelected);
     iHypervalent = -1; // this needs to be a bitset
     
-    for (int i = bsSelected.nextSetBit(0); i >= 0 && iHypervalent < 0; i = bsSelected.nextSetBit(i + 1))
-      if (atoms[i].getCovalentBondCount() > 4 || isPolyhedral)
+    for (int i = bsSelected.nextSetBit(0); i >= 0
+        && iHypervalent < 0; i = bsSelected.nextSetBit(i + 1)) {
+      if (atoms[i].getCovalentBondCount() > 4 || isPolyhedral) {
         iHypervalent = i;
+      }
+    }
     bsIncludingH = BSUtil.copy(bsSelected);
-    if (explicitHydrogen == 0)
+    if (explicitHydrogen == 0) {
       for (int j = bsSelected.nextSetBit(0); j >= 0; j = bsSelected
           .nextSetBit(j + 1)) {
         Node a = atoms[j];
-        if (a.getAtomicAndIsotopeNumber() == 1 
-            && a.getBondCount() > 0 
+        if (a.getAtomicAndIsotopeNumber() == 1 && a.getBondCount() > 0
             && a.getBondedAtomIndex(0) != iHypervalent
-            && !isExplicitOnly(atoms[a.getBondedAtomIndex(0)])
-            )
+            && !isExplicitOnly(atoms[a.getBondedAtomIndex(0)]))
           bsSelected.clear(j);
       }
+    }
     bsAromatic = new BS();
     if (!topologyOnly && bsSelected.cardinality() > 2) {
       generateRingData();
@@ -384,18 +396,20 @@ public class SmilesGenerator {
     SB sb = new SB();
 
     // The idea hear is to allow a hypervalent atom to be listed first
-    for (int i = bsToDo.nextSetBit(0); i >= 0; i = bsToDo.nextSetBit(i + 1))
-      if (atoms[i].getCovalentBondCount() > 4 || isPolyhedral) {
+    for (int i = bsToDo.nextSetBit(0); i >= 0; i = bsToDo.nextSetBit(i + 1)) {
+      if (atoms[i].getCovalentBondCount() > 4 || isPolyhedral || noBranches) {
         if (atom == null)
           sb.append(".");
-        getSmilesAt(sb, atoms[i], allowConnectionsToOutsideWorld, false, forceBrackets);
+        getSmilesAt(sb, atoms[i], allowConnectionsToOutsideWorld, false,
+            forceBrackets, false);
         atom = null;
       }
+    }
     if (atom != null)
-      while ((atom = getSmilesAt(sb, atom, allowConnectionsToOutsideWorld,
-          true, forceBrackets)) != null) {
+      while ((atom = getSmilesAt(sb, atom, allowConnectionsToOutsideWorld, true,
+          forceBrackets, false)) != null) {
       }
-    while (bsToDo.cardinality() > 0 || !htRings.isEmpty()) {
+    while (!bsToDo.isEmpty() || !htRings.isEmpty()) {
       Iterator<Object[]> e = htRings.values().iterator();
       if (e.hasNext()) {
         atom = atoms[((Integer) e.next()[1]).intValue()];
@@ -407,8 +421,8 @@ public class SmilesGenerator {
       sb.append(".");
       prevSp2Atoms = alleneStereo = null;
       prevAtom = null;
-      while ((atom = getSmilesAt(sb, atom, allowConnectionsToOutsideWorld,
-          true, forceBrackets)) != null) {
+      while ((atom = getSmilesAt(sb, atom, allowConnectionsToOutsideWorld, true,
+          forceBrackets, false)) != null) {
       }
     }
     if (!htRings.isEmpty()) {
@@ -432,8 +446,17 @@ public class SmilesGenerator {
         s = s0;
       }
     }
-    return s;
+    ret += s;
+
+    int ipt = bs.nextSetBit(0);
+    if (ipt < 0 || !allComponents)
+      break;
+    ret += ".";
+    atom = null;
   }
+  return ret;
+}
+
 
   /**
    * Don't start with H or central cumulene (in case there is symmetry)
@@ -473,16 +496,16 @@ public class SmilesGenerator {
   private void generateRingData() throws InvalidSmilesException {
     // we are not actually running this search, just getting preliminary data    
     SmilesSearch search = SmilesParser.newSearch("[r500]", true, true);
-    search.targetAtoms = atoms;
-    search.setSelected(bsSelected);
+    search.target.setAtoms(atoms, ac, bsSelected);
     search.setFlags(flags);
-    search.targetAtomCount = ac;
+    if (!search.needRingData && search.target.hasDoubleBonds())
+      search.needRingData = true;
     search.ringDataMax = 7;
     search.flags = flags;
     Lst<BS>[] vRings = AU.createArrayOfArrayList(4);
     search.setRingData(null, vRings, true);
-    bsAromatic = search.bsAromatic;
-    ringSets = search.ringSets;
+    bsAromatic = search.target.bsAromatic;
+    ringSets = search.target.ringSets;
     aromaticRings = vRings[3];
   }
 
@@ -670,20 +693,19 @@ public class SmilesGenerator {
   
   private Node getSmilesAt(SB sb, SimpleNode atom,
                            boolean allowConnectionsToOutsideWorld,
-                           boolean allowBranches, boolean forceBrackets) {
+                           boolean allowBranches, boolean forceBrackets,
+                           boolean isBranch) {
     int atomIndex = atom.getIndex();
     if (!bsToDo.get(atomIndex))
       return null;
     ptAtom++;
     bsToDo.clear(atomIndex);
-    boolean includeHs = (
-        atomIndex == iHypervalent 
-        || explicitHydrogen != 0 && !bsIgnoreHydrogen.get(atomIndex)
-        );
+    boolean includeHs = (atomIndex == iHypervalent
+        || explicitHydrogen != 0 && !bsIgnoreHydrogen.get(atomIndex));
     boolean explicitHs = isExplicitOnly(atom);
+    boolean isAromatic = bsAromatic.get(atomIndex);
     boolean isExtension = (!bsSelected.get(atomIndex));
     int prevIndex = (prevAtom == null ? -1 : prevAtom.getIndex());
-    boolean isAromatic = bsAromatic.get(atomIndex);
     // prevSp2Atoms is for allene ABC=C=CDE
     SimpleNode[] sp2Atoms = prevSp2Atoms;
     boolean havePreviousSp2Atoms = (sp2Atoms != null);
@@ -695,7 +717,7 @@ public class SmilesGenerator {
     Edge bondNext = null;
     Edge bondPrev = null;
     Edge[] bonds = (Edge[]) atom.getEdges();
-    if (polySmilesCenter != null) {
+    if (!isBranch && polySmilesCenter != null) {
       allowBranches = false;
       sortPolyBonds(atom, prevAtom, polySmilesCenter);
     }
@@ -707,8 +729,8 @@ public class SmilesGenerator {
     // first look through the bonds for the best 
     // continuation -- bond0 -- and count hydrogens
     // and create a list of bonds to process.
-    if (bonds != null)
-      for (int i = bonds.length; --i >= 0;) {
+    if (bonds != null) {
+      for (int i = 0, nb = bonds.length; i < nb; i++) {
         Edge bond = bonds[i];
         if (!bond.isCovalent())
           continue;
@@ -722,11 +744,10 @@ public class SmilesGenerator {
         boolean isH = !includeHs && !explicitHs
             && (atom1.getElementNumber() == 1 && atom1.getIsotopeNumber() <= 0);
         if (!bsIncludingH.get(index1)) {
-          if (!isH && allowConnectionsToOutsideWorld
-              && bsSelected.get(atomIndex))
-            bsToDo.set(index1);
-          else
+          if (isH || !allowConnectionsToOutsideWorld
+              || !bsSelected.get(atomIndex))
             continue;
+            bsToDo.set(index1);
         }
         if (isH) {
           aH = atom1;
@@ -737,6 +758,7 @@ public class SmilesGenerator {
           v.addLast(bonds[i]);
         }
       }
+    }
 
     // order of listing is critical for stereochemistry:
     //
@@ -782,20 +804,26 @@ public class SmilesGenerator {
       for (int i = 0; i < nBonds; i++) {
         Edge bond = v.get(i);
         SimpleNode a = bond.getOtherNode(atom);
-        int n = a.getCovalentBondCount()
-            - (includeHs || isExplicitOnly(a) ? 0 : ((Node) a).getCovalentHydrogenCount());
+        int n = a.getCovalentBondCount() - (includeHs || isExplicitOnly(a) ? 0
+            : ((Node) a).getCovalentHydrogenCount());
         int order = bond.getCovalentOrder();
-        if (n == 1 && (bondNext != null || i < nBonds - 1)) {
+        // single-connected atoms such as OH or (=O) or CH3 first up
+        boolean isEndPoint = htRings
+            .containsKey(getRingKey(a.getIndex(), atomIndex));
+        if (isEndPoint)
+          continue;
+        boolean check = (bondNext != null || i < nBonds - 1);
+        if (n == 1 && check) {
           bsBranches.set(bond.index);
-        } else if ((order > 1 || n > nMax)
-            && !htRings.containsKey(getRingKey(a.getIndex(), atomIndex))) {
+        } else if ((order > 1 || n > nMax)) {
           nMax = (order > 1 ? 1000 + order : n);
           bondNext = bond;
         }
       }
     }
-    Node atomNext = (bondNext == null ? null : (Node) bondNext
-        .getOtherNode(atom));
+
+    Node atomNext = (bondNext == null ? null
+        : (Node) bondNext.getOtherNode(atom));
     int orderNext = (bondNext == null ? 0 : bondNext.getCovalentOrder());
 
     // initialize stereo[] for stereochemistry 
@@ -855,7 +883,7 @@ public class SmilesGenerator {
       int ptAtomt = ptAtom;
       // next call re-enters this method.
       getSmilesAt(s2, a, allowConnectionsToOutsideWorld, allowBranches,
-          forceBrackets);
+          forceBrackets, true);
       bondNext = bond0t;
       ptAtom = ptAtomt;
       ptSp2Atom0 = ptSp2Atom0t;
@@ -890,13 +918,14 @@ public class SmilesGenerator {
       if (bond == bondNext)
         continue;
       SimpleNode a = bond.getOtherNode(atom);
-      strPrev = getBondOrder(bond, atomIndex, a.getIndex(), isAromatic);
+      int ia = a.getIndex();
+      strPrev = getBondOrder(bond, atomIndex, ia, isAromatic);
       chBond = getBondStereochemistry(bond, atom);
       if (!deferStereo && chBond != '\0') {
           strPrev = "" + chBond;
       }
       sbRings.append(strPrev);
-      sbRings.append(getRingCache(atomIndex, a.getIndex(), htRings));
+      sbRings.append(getRingCache(atomIndex, ia, htRings));
       if (stereoFlag < 7)
         stereo[stereoFlag++] = a;
       if (sp2Atoms != null && nSp2Atoms < 5)
@@ -980,11 +1009,12 @@ public class SmilesGenerator {
     // .[CYS.SG#16] could match either the atom number or the element number 
     if (addAtomComment)
       sb.append("\n//* " + atom.toString() + " *//\t");
-    if (topologyOnly)
+    if (topologyOnly) {
       sb.append("*");
-    else if (isExtension && groupType.length() != 0 && atomName.length() != 0)
+    } else if (isExtension && groupType.length() != 0
+        && atomName.length() != 0) {
       addBracketedBioName(sb, (Node) atom, "." + atomName, false);
-    else
+    } else {
       sb.append(SmilesAtom.getAtomLabel(
           atomicNumber,
           isotope,
@@ -995,7 +1025,7 @@ public class SmilesGenerator {
           isAromatic,
           atat != null ? atat : noStereo ? null : checkStereoPairs(atom,
               alleneStereo == null ? atomIndex : -1, stereo, stereoFlag, prevIndex == -1), is2D));
-
+    }
     // add the rings...
 
     sb.appendSB(sbRings);
@@ -1036,8 +1066,14 @@ public class SmilesGenerator {
   }
 
   private boolean isExplicitOnly(SimpleNode atom) {
-    // imine hack here
-    return (atom.getElementNumber() == 7 || atom.getElementNumber() == 6 && atom.getIsotopeNumber() == 17);
+    // imine hack here "17C"
+    //  -- {thismodel}.inchi("SMILES/imine") will convert InChI N to [17C] or [17c] 
+    //      with [5H] to preserve stereochemistry when loading.  
+    // -- MolReader will convert [17C,17c] to imine N automatically 
+    // and remove any [5H] 
+
+    return (atom.getElementNumber() == 7 && atom.getCovalentBondCount() == 2 
+        || atom.getElementNumber() == 6 && atom.getIsotopeNumber() == 17);
   }
 
   private SimpleNode[] atemp;
@@ -1078,7 +1114,7 @@ public class SmilesGenerator {
     if ((bondPrev.order & Edge.TYPE_ATROPISOMER) == Edge.TYPE_ATROPISOMER) {
       return "^-";
     }
-    int border = bondPrev.getCovalentOrder();
+    int border = Math.max(bondPrev.isPartial() ? 1 : 0, bondPrev.getCovalentOrder());
     return (!isAromatic || !bsAromatic.get(prevIndex) ? SmilesBond
         .getBondOrderString(border) : border == 1
         && !isSameAromaticRing(atomIndex, prevIndex) ? 
@@ -1211,17 +1247,20 @@ public class SmilesGenerator {
   }
 
   private String checkStereoPairs(SimpleNode atom, int atomIndex,
-                                  SimpleNode[] stereo, int stereoFlag, boolean isFirst) {
+                                  SimpleNode[] stereo, int stereoFlag,
+                                  boolean isFirst) {
     if (stereoFlag == 10 || stereoFlag < (is2D ? 3 : 4))
       return "";
-    if (explicitHydrogen == 0 && atomIndex >= 0 && stereoFlag == 4
-        && (atom.getElementNumber()) == 6) {
-      // do a quick check for two of the same group for tetrahedral carbon only
-      String s = "";
-      for (int i = 0; i < 4; i++) {
-        if ((s = addStereoCheck(0, atomIndex, stereo[i], s,
-            BSUtil.newAndSetBit(atomIndex))) == null) {
-          return "";
+    if (explicitHydrogen == 0 && atomIndex >= 0 && stereoFlag == 4) {
+      if (atom.getElementNumber() <= 17) {
+        // anything up to Cl. 
+        // do a quick check for two of the same group for tetrahedral sulfurcarbon only
+        String s = "";
+        for (int i = 0; i < 4; i++) {
+          if ((s = addStereoCheck(0, atomIndex, stereo[i], s,
+              BSUtil.newAndSetBit(atomIndex))) == null) {
+            return "";
+          }
         }
       }
     }
@@ -1304,16 +1343,19 @@ public class SmilesGenerator {
     if (bsDone != null)
       bsDone.set(atomIndex);
     int n = ((Node)atom).getAtomicAndIsotopeNumber();
+    if (n == 1 && explicitHydrogen == 0)
+      return "H";
     int nx = atom.getCovalentBondCount();
-    int nh = (n == 6 && explicitHydrogen != 0 ? ((Node) atom).getCovalentHydrogenCount() : 0);
+    int nhcov = ((Node) atom).getCovalentHydrogenCount();
+    int nh = (n == 6 && explicitHydrogen != 0 ? nhcov : 0);
     // only carbon or singly-connected atoms are checked
-    // for C we use nh -- CH3, for example.
+    // for C or Si we use nh -- CH3, for example.
     // for other atoms, we use number of bonds.
     // just checking for tetrahedral CH3)
-    if (n == 6 ? nx != 4 : n == 1 || nx > 1)
+    if (n == 6 || n == 16 ? nx != 4 : n == 1 || nx > 1)
       return s + (++chainCheck);
     String sa = ";" + level + "/" + n + "/" + nh + "/" + nx + (level == 0 ? "," : "_");
-    if (n == 6) {
+    if (n == 6 || n == 16) {
       switch (nh) {
       case 1:
         return s + sa + (++chainCheck);
@@ -1327,8 +1369,9 @@ public class SmilesGenerator {
         int nunique = (nh == 2 ? 0 : 3);
         for (int j = atom.getBondCount(); --j >= 0;) {
           SimpleNode a2 = edges[j].getOtherNode(atom);
+          //System.out.println(level + " " + atom + " " + a2);
           int i2 = a2.getIndex();
-          if (bsDone.get(i2) || !edges[j].isCovalent() || a2.getElementNumber() == 1)
+          if (bsDone.get(i2) || !edges[j].isCovalent())
             continue;
           bsDone.set(i2);
           sa2 = addStereoCheck(level + 1, atom.getIndex(), a2, "", (BS) bsDone.clone());
@@ -1336,8 +1379,9 @@ public class SmilesGenerator {
             nunique--;
           s2 += sa2;
         }
-        if (nunique == 3)
+        if (nunique == 3) {
           return s + sa + (++chainCheck);
+        }
         sa = (sa + s2).replace(',','_');
         if (level > 0)
           return s + sa;
@@ -1347,7 +1391,7 @@ public class SmilesGenerator {
       }
     }
     if (s.indexOf(sa) >= 0) {
-      if (nh == 3) {
+      if (nhcov == 3) {
         // must check isotopes for CH3
         int ndt = 0;
         for (int j = 0; j < nx && ndt < 3; j++) {
