@@ -24,6 +24,8 @@
 
 package org.jmol.util;
 
+import org.jmol.viewer.Viewer;
+
 import javajs.util.AU;
 import javajs.util.M4;
 import javajs.util.P3;
@@ -31,6 +33,7 @@ import javajs.util.P3i;
 import javajs.util.P4;
 import javajs.util.PT;
 import javajs.util.T3;
+import javajs.util.T4;
 import javajs.util.V3;
 
 
@@ -44,24 +47,75 @@ import javajs.util.V3;
  * 
  */
 
-public class SimpleUnitCell {
+public class SimpleUnitCell { 
 
+  public static final int PARAM_STD       = 6;  // a b c alpha beta gamma [0-5]
+  public static final int PARAM_VAX       = 6;  // ax ay az [6-8]
+  public static final int PARAM_VBX       = 9;  // bx by bz [9-11]
+  public static final int PARAM_VCX       = 12; // cx cy cz [12-14]
+  public static final int PARAM_VCZ       = 14; // last vector value - test for vabc
+  public static final int PARAM_OXYZ      = 15; // ox oy oz [15-17]
+  
+  public static final int PARAM_M4        = 6;  // m00 m01 ... m44 (16 entries, [6-21)
+  public static final int PARAM_M33       = 21; // last matrix value - test for 4x4 matrix
+  public static final int PARAM_SUPERCELL = 22; // na nb nc
+  public static final int PARAM_SCALE = 25;     // scale
+  public static final int PARAM_SLOP = 26;      // slop
+  public static final int PARAM_COUNT = 27;
 
-  public static final int PARAM_STD = 6;
-  public static final int PARAM_VABC = 6;
-  public static final int PARAM_M4 = 6;
-  public static final int PARAM_SUPERCELL = 22;
-  public static final int PARAM_SCALE = 25;
-  public static final int PARAM_COUNT = 26;
+  public final static int INFO_IS_RHOMBOHEDRAL = 8;
+  public final static int INFO_IS_HEXAGONAL = 7;  
+  public final static int INFO_DIMENSIONS = 6;
+  public final static int INFO_GAMMA = 5;
+  public final static int INFO_BETA = 4;
+  public final static int INFO_ALPHA = 3;
+  public final static int INFO_C = 2;
+  public final static int INFO_B = 1;
+  public final static int INFO_A = 0;
+
+  public static final String HEX_TO_RHOMB = "2/3a+1/3b+1/3c,-1/3a+1/3b+1/3c,-1/3a-2/3b+1/3c";
+  public static final String RHOMB_TO_HEX = "a-b,b-c,a+b+c";
+
+  protected final static double toRadians = Math.PI * 2 / 360;
+
+ /**
+   * allowance for rounding in [0,1)
+   */
+  public final static float SLOPSP = 0.0001f;
+  public final static float SLOPDP = 0.000000000001f;
+  public static final float SLOP_PARAMS = 0.001f; // generous
 
   protected float[] unitCellParams;
+  
+  /**
+   * initial value is set in subclass UnitCell
+   */
+  protected float slop = (Viewer.isHighPrecision ? SLOPDP : SLOPSP);
+  
+  public float getPrecision() {
+    return slop;
+  }
+
+  public void setPrecision(float slop) {
+    unitCellParams[PARAM_SLOP] = this.slop = (!Float.isNaN(slop) ? slop 
+        : !Float.isNaN(unitCellParams[PARAM_SLOP]) ? unitCellParams[PARAM_SLOP] 
+            : Viewer.isHighPrecision ? SLOPDP : SLOPSP);
+  }
+
   public M4 matrixCartesianToFractional;
   public M4 matrixFractionalToCartesian;
+  protected M4 matrixCtoFNoOffset;
+  protected M4 matrixFtoCNoOffset;  
+  
   public double volume;
-
-  protected final static float toRadians = (float) Math.PI * 2 / 360;
-
+  
+  protected int dimension = 3;
+  
+  private P3 fractionalOrigin;
+  
+  
   private int na, nb, nc;
+  
   public boolean isSupercell() {
     return (na > 1 || nb > 1 || nc > 1);
   }
@@ -73,12 +127,11 @@ public class SimpleUnitCell {
   protected double cA_, cB_;
   protected double a_;
   protected double b_, c_;
-  protected int dimension = 3;
-  private P3 fractionalOrigin;
 
+  
   public static boolean isValid(float[] parameters) {
-    return (parameters != null && (parameters[0] > 0 || parameters.length > 14
-        && !Float.isNaN(parameters[14])));
+    return (parameters != null && (parameters[0] > 0 || parameters.length > PARAM_VCZ
+        && !Float.isNaN(parameters[PARAM_VCZ])));
   }
 
   protected SimpleUnitCell() {
@@ -107,6 +160,7 @@ public class SimpleUnitCell {
    * 
    *        and/or len = 26 [...................... na nb nc scale] // scaled supercell
    * 
+   *        and/or len = 27 [..................................... slop] // precision
    * @return a simple unit cell
    */
   public static SimpleUnitCell newA(float[] params) {
@@ -115,26 +169,23 @@ public class SimpleUnitCell {
     return c;
   }
   
+  public static int getDimensionFromParams(float[] params) {
+	    return (params[0] <= 0 ? 3 : params[1] < 0 ? 1 : params[2] < 0 ? 2 : 3);
+  }
+
+
   protected void init(float[] params) {
     if (params == null)
       params = new float[] { 1, 1, 1, 90, 90, 90 };
     if (!isValid(params))
       return;
-    params = unitCellParams = AU.arrayCopyF(params, params.length);
-
+    unitCellParams = newParams(params, Float.NaN);
     boolean rotateHex = false; // special gamma = -1 indicates hex rotation for AFLOW
 
+    dimension = getDimensionFromParams(params);
     a = params[0];
     b = params[1];
     c = params[2];
-    if (a > 0) {
-      if (b < 0) {
-        dimension = 1;
-      } else if (c < 0) {
-        dimension = 2;
-      }
-    }
-
     alpha = params[3];
     beta = params[4];
     gamma = params[5];
@@ -142,8 +193,15 @@ public class SimpleUnitCell {
       rotateHex = true;
       gamma = 120;
     }
+    if (params.length > PARAM_SLOP) {
+      if (Float.isNaN(params[PARAM_SLOP])) {
+        params[PARAM_SLOP] = slop;
+      } else {
+        slop = params[PARAM_SLOP];
+      }
+    }
 
-    // (int) Float.NaN == 0 (but not in JavaScript!)
+    // (int) double.NaN == 0 (but not in JavaScript!)
     // supercell
     float fa = na = Math.max(1,
         params.length > PARAM_SUPERCELL + 2
@@ -173,12 +231,9 @@ public class SimpleUnitCell {
 
     if (a <= 0 && c <= 0) {
       // must calculate a, b, c alpha beta gamma from Cartesian vectors;
-      V3 va = V3.new3(params[PARAM_VABC], params[PARAM_VABC + 1],
-          params[PARAM_VABC + 2]);
-      V3 vb = V3.new3(params[PARAM_VABC + 3], params[PARAM_VABC + 4],
-          params[PARAM_VABC + 5]);
-      V3 vc = V3.new3(params[PARAM_VABC + 6], params[PARAM_VABC + 7],
-          params[PARAM_VABC + 8]);
+      V3 va = newV(params, PARAM_VAX);
+      V3 vb = newV(params, PARAM_VBX);
+      V3 vc = newV(params, PARAM_VCX);
       setABC(va, vb, vc);
       if (c < 0) {
         float[] n = AU.arrayCopyF(params, -1);
@@ -197,7 +252,7 @@ public class SimpleUnitCell {
           vc.normalize();
           n[12] = vc.x;
           n[13] = vc.y;
-          n[14] = vc.z;
+          n[PARAM_VCZ] = vc.z;
         }
         params = n;
       }
@@ -208,18 +263,16 @@ public class SimpleUnitCell {
     a *= fa;
     if (b <= 0) {
       b = c = 1;
-      dimension = 1;
     } else if (c <= 0) {
       c = 1;
       b *= fb;
-      dimension = 2;
     } else {
       b *= fb;
       c *= fc;
     }
     setCellParams();
 
-    if (params.length > 21 && !Float.isNaN(params[21])) {
+    if (params.length > PARAM_M33 && !Float.isNaN(params[PARAM_M33])) {
       // parameters with a 4x4 matrix
       // [a b c alpha beta gamma m00 m01 m02 m03 m10 m11.... m20...]
       // this is for PDB and CIF reader
@@ -240,7 +293,7 @@ public class SimpleUnitCell {
           f = 1;
           break;
         }
-        scaleMatrix[i] = params[6 + i] * f;
+        scaleMatrix[i] = params[PARAM_M4 + i] * f;
       }
       matrixCartesianToFractional = M4.newA16(scaleMatrix);
       matrixCartesianToFractional.getTranslation(fractionalOrigin);
@@ -248,14 +301,23 @@ public class SimpleUnitCell {
           .invert();
       if (params[0] == 1)
         setParamsFromMatrix();
-    } else if (params.length > 14 && !Float.isNaN(params[14])) {
+    } else if (params.length > PARAM_VCZ && !Float.isNaN(params[PARAM_VCZ])) {
       // parameters with a 3 vectors
       // [a b c alpha beta gamma ax ay az bx by bz cx cy cz...]
       M4 m = matrixFractionalToCartesian = new M4();
-      m.setColumn4(0, params[6] * fa, params[7] * fa, params[8] * fa, 0);
-      m.setColumn4(1, params[9] * fb, params[10] * fb, params[11] * fb, 0);
-      m.setColumn4(2, params[12] * fc, params[13] * fc, params[14] * fc, 0);
-      m.setColumn4(3, 0, 0, 0, 1);
+      m.setColumn4(0, params[PARAM_VAX] * fa, params[PARAM_VAX + 1] * fa,
+          params[PARAM_VAX + 2] * fa, 0);
+      m.setColumn4(1, params[PARAM_VBX] * fb, params[PARAM_VBX + 1] * fb,
+          params[PARAM_VBX + 2] * fb, 0);
+      m.setColumn4(2, params[PARAM_VCX] * fc, params[PARAM_VCX + 1] * fc,
+          params[PARAM_VCX + 2] * fc, 0);
+      if (params.length > PARAM_OXYZ + 2 && !Float.isNaN(params[PARAM_OXYZ + 2])) {
+        // cartesian offset
+        m.setColumn4(3, params[PARAM_OXYZ], params[PARAM_OXYZ + 1],
+            params[PARAM_OXYZ + 2], 1);
+      } else {
+        m.setColumn4(3, 0, 0, 0, 1);
+      }
       matrixCartesianToFractional = M4.newM4(matrixFractionalToCartesian)
           .invert();
     } else {
@@ -285,11 +347,25 @@ public class SimpleUnitCell {
     matrixFtoCNoOffset = matrixFractionalToCartesian;
   }
 
+  private static V3 newV(float[] p, int i) {
+    return V3.new3(p[i++], p[i++],p[i]);
+  }
+
+  public static float[] newParams(float[] params, float slop) {
+    float[] p = new float[PARAM_COUNT];
+    int n = params.length;
+    for (int i = 0; i < PARAM_COUNT; i++)
+      p[i] = (i < n ? params[i] : Float.NaN);
+    if (n < PARAM_COUNT)
+      p[PARAM_SLOP] = slop;
+    return p;
+  }
+
   public static void addVectors(float[] params) {
     SimpleUnitCell c = SimpleUnitCell.newA(params);
     M4 m = c.matrixFractionalToCartesian;
     for (int i = 0; i < 9; i++)
-    params[PARAM_VABC + i] = m.getElement(i%3, i/3);
+    params[PARAM_M4 + i] = m.getElement(i%3, i/3);
   }
 
 
@@ -305,29 +381,40 @@ public class SimpleUnitCell {
   }
 
   private void setABC(V3 va, V3 vb, V3 vc) {
-    a = va.length();
-    b = vb.length();
-    c = vc.length();
+    fillParams(va, vb, vc, unitCellParams);
+    float[] p = unitCellParams;
+    a = p[0];
+    b = p[1];
+    c = p[2];
+    alpha = p[3];
+    beta = p[4];
+    gamma = p[5];
+  }
+
+  public static void fillParams(V3 va, V3 vb, V3 vc, float[] p) {
+    if (va == null) {
+      va = newV(p, PARAM_VAX);
+      vb = newV(p, PARAM_VBX);
+      vc = newV(p, PARAM_VCX);
+    }
+    float a = va.length();
+    float b = vb.length();
+    float c = vc.length();
     if (a == 0)
       return;
     if (b == 0)
       b = c = -1; //polymer
     else if (c == 0)
       c = -1; //slab
-    alpha = (b < 0 || c < 0 ? 90 : vb.angle(vc) / toRadians);
-    beta = (c < 0 ? 90 : va.angle(vc) / toRadians);
-    gamma = (b < 0 ? 90 : va.angle(vb) / toRadians);
-    float[] p = unitCellParams;
     p[0] = a;
     p[1] = b;
     p[2] = c;
-    p[3] = alpha;
-    p[4] = beta;
-    p[5] = gamma;
+    p[3] = (float) (b < 0 || c < 0 ? 90 : vb.angle(vc) / toRadians);
+    p[4] = (float) (c < 0 ? 90 : va.angle(vc) / toRadians);
+    p[5] = (float) (b < 0 ? 90 : va.angle(vb) / toRadians);
   }
 
   private void setCellParams() {
-    //System.out.println("unitcell " + a + " " + b + " " + c);
     cosAlpha = Math.cos(toRadians * alpha);
     sinAlpha = Math.sin(toRadians * alpha);
     cosBeta = Math.cos(toRadians * beta);
@@ -345,21 +432,17 @@ public class SimpleUnitCell {
     c_ = a * b * sinGamma / volume;
   }
 
-  public T3 getFractionalOrigin() {
-    return fractionalOrigin ;
+  /**
+   * Get the fractional origin for the UccageRenderer.
+   * 
+   * NOTE: This is NOT a copy
+   * 
+   * @return fractionalOrigin
+   */
+  public P3 getFractionalOrigin() {
+    return fractionalOrigin;
   }
 
-  protected M4 matrixCtoFNoOffset;
-  protected M4 matrixFtoCNoOffset;
-  
-  public final static int INFO_DIMENSIONS = 6;
-  public final static int INFO_GAMMA = 5;
-  public final static int INFO_BETA = 4;
-  public final static int INFO_ALPHA = 3;
-  public final static int INFO_C = 2;
-  public final static int INFO_B = 1;
-  public final static int INFO_A = 0;
-
   /**
    * convenience return only after changing fpt
    * 
@@ -438,30 +521,14 @@ public class SimpleUnitCell {
       return gamma;
     case INFO_DIMENSIONS:
       return dimension;
+    case INFO_IS_HEXAGONAL:
+      return (isHexagonal(unitCellParams) ? 1 : 0);
+    case INFO_IS_RHOMBOHEDRAL:
+      return (isRhombohedral(unitCellParams) ? 1 : 0);      
     }
     return Float.NaN;
   }
 
-  public final static float SLOP = 0.02f;
-  private final static float SLOP1 = 1 - SLOP;
-
-  /**
-   * calculate weighting of 1 (interior), 0.5 (face), 0.25 (edge), or 0.125 (vertex)
-   * @param pt
-//   * @param tolerance fractional allowance to consider this on an edge
-   * @return weighting
-   */
-  public static float getCellWeight(P3 pt) {
-    float f = 1;
-    if (pt.x <= SLOP || pt.x >= SLOP1)
-      f /= 2;
-    if (pt.y <= SLOP || pt.y >= SLOP1)
-      f /= 2;
-    if (pt.z <= SLOP || pt.z >= SLOP1)
-      f /= 2;
-    return f;
-  }
-  
   /**
    * Generate the reciprocal unit cell, scaled as desired
    * 
@@ -471,36 +538,39 @@ public class SimpleUnitCell {
    * @return oabc
    */
   public static T3[] getReciprocal(T3[] abc, T3[] ret, float scale) {
-    if (scale == 0)
-      scale = (float)(2 * Math.PI);
-    P3[] rabc = new P3[4];
     int off = (abc.length == 4 ? 1 : 0);
+    P3[] rabc = new P3[4];
     rabc[0] = (off == 1 ? P3.newP(abc[0]) : new P3()); // origin
+    if (scale == 0)
+      scale = (float) (2 * Math.PI);
     // a' = 2pi/V * b x c  = 2pi * (b x c) / (a . (b x c))
     // b' = 2pi/V * c x a 
     // c' = 2pi/V * a x b 
     for (int i = 0; i < 3; i++) {
       P3 v = rabc[i + 1] = new P3();
       v.cross(abc[((i + 1) % 3) + off], abc[((i + 2) % 3) + off]);
-      v.scale(scale / abc[i + off].dot(v));
+      float vol = abc[i + off].dot(v);
+      if (scale == -1)
+        scale = (float) Math.sqrt(vol);
+      v.scale(scale / vol);
     }
     if (ret == null)
       return rabc;
-    for (int i = 0; i < 4; i++)
+    for (int i = 0; i < 4; i++) {
       ret[i] = rabc[i];
+    }
     return ret;
   }
 
   /**
-   * set cell vectors by string
-   * 
+   * set cell vectors by string. Does not set origin.
    * 
    * @param abcabg "a=...,b=...,c=...,alpha=...,beta=..., gamma=..." or null  
    * @param params to use if not null 
    * @param ucnew  to create and return; null if only to set params
-   * @return T3[4] origin, a, b c
+   * @return T3[4] ucnew as [origin, a, b, c], with origin unchanged or {0 0 0}
    */
-  public static T3[] setOabc(String abcabg, float[] params, T3[] ucnew) {
+  public static T3[] setAbc(String abcabg, float[] params, T3[] ucnew) {
     if (abcabg != null) {
       if (params == null)
         params = new float[6];
@@ -511,108 +581,115 @@ public class SimpleUnitCell {
     }
     if (ucnew == null)
       return null;
+    return setAbcFromParams(params, ucnew);
+  }
+
+  public static T3[] setAbcFromParams(float[] params, T3[] ucnew) {
     float[] f = newA(params).getUnitCellAsArray(true);
-      ucnew[1].set(f[0], f[1], f[2]);
-      ucnew[2].set(f[3], f[4], f[5]);
-      ucnew[3].set(f[6], f[7], f[8]);
+    ucnew[1].set(f[0], f[1], f[2]);
+    ucnew[2].set(f[3], f[4], f[5]);
+    ucnew[3].set(f[6], f[7], f[8]);
     return ucnew;
   }
 
-  public static void unitizeDim(int dimension, T3 pt) {
+  /**
+   * Used for just about everything; via UnitCell
+   * including SpaceGroup.getSiteMultiplicity, Symmetry.getInvariantSymops, 
+   * Symmetry.removeDuplicates, Symmetry.unitize, Symmetry.toUnitCell, SymmetryDesc.getTransform.
+   * 
+   * Low precision here unless SET HIGHPRECISION TRUE has been issued.
+   * 
+   * @param dimension
+   * @param pt
+   */
+  public void unitizeDim(int dimension, T3 pt) {
     switch (dimension) {
     case 3:
-      pt.z = unitizeX(pt.z);  
+      pt.z = unitizeX(pt.z, slop);  
       //$FALL-THROUGH$
     case 2:
-      pt.y = unitizeX(pt.y);
+      pt.y = unitizeX(pt.y, slop);
+      //$FALL-THROUGH$
+    case 1: 
+      pt.x = unitizeX(pt.x, slop);
+	    }
+	  }
+
+  /**
+   * Only used for getting equivalent points and checking for duplicates. 
+   * Presumption here is that two points should not be close together
+   * regardless of lattice distances.
+   * 
+   * @param dimension
+   * @param pt
+   * @param slop 
+   */
+  public static void unitizeDimRnd(int dimension, T3 pt, float slop) {
+    switch (dimension) {
+    case 3:
+      pt.z = unitizeXRnd(pt.z, slop);  
+      //$FALL-THROUGH$
+    case 2:
+      pt.y = unitizeXRnd(pt.y, slop);
       //$FALL-THROUGH$
     case 1:
-      pt.x = unitizeX(pt.x);
+      pt.x = unitizeXRnd(pt.x, slop);
     }
   }
 
-  public static void unitizeDimRnd(int dimension, T3 pt) {
-    switch (dimension) {
-    case 3:
-      pt.z = unitizeXRnd(pt.z);  
-      //$FALL-THROUGH$
-    case 2:
-      pt.y = unitizeXRnd(pt.y);
-      //$FALL-THROUGH$
-    case 1:
-      pt.x = unitizeXRnd(pt.x);
-    }
-  }
-
-  public static float unitizeX(float x) {
-    // introduced in Jmol 11.7.36
+  public static float unitizeX(float x, float slop) {
+	    // introduced in Jmol 11.7.36
     x = (float) (x - Math.floor(x));
     // question - does this cause problems with dragatom?
-    if (x > 0.999f || x < 0.001f)  // 0.9999, 0.0001 was just too tight ams/jolliffeite
+    if (x > 1 - slop || x < slop)  // 0.9999, 0.0001 was just too tight ams/jolliffeite
       x = 0;
     return x;
   }
 
-  public static float unitizeXRnd(float x) {
+  /**
+   * Slightly higher precision -- only used for cmdAssignSpaceGroup checking for duplicates
+   * @param x
+   * @param slop 
+   * @return rounded value +/-slop
+   */
+  public static float unitizeXRnd(float x, float slop) {
     // introduced in Jmol 11.7.36
-    x = (float) (x - Math.floor(x));
-    if (x > 0.9999f || x < 0.0001f) 
+    x = (x - (float) Math.floor(x));
+    if (x > 1 - slop || x < slop) 
       x = 0;
     return x;
   }
   
-
-
-  public static float normalizeXRnd(float x) {
-    return Math.round(x*12) / 12;
+  public int twelfthsOf(float f) {
+    if (f == 0)
+      return 0;
+    f = Math.abs(f * 12);
+    int i = Math.round(f);
+    return (i <= 12 && Math.abs(f - i) < slop * 12 ? i : -1);
   }
-
   
-  /**
-   * allowance for rounding in [0,1)
-   */
-  private final static float SLOP2 = 0.0001f;
-  
-  /**
-   * check atom position for range [0, 1) allowing for rounding
-
-   * @param pt 
-   * @return true if in [0, 1)
-   */
-  public boolean checkPeriodic(P3 pt) {
+  public void twelfthify(P3 pt) {
     switch (dimension) {
     case 3:
-      if (pt.z < -SLOP2 || pt.z > 1 - SLOP2)
-        return false;
+      pt.z = setTwelfths(pt.z);
       //$FALL-THROUGH$
     case 2:
-      if (pt.y < -SLOP2 || pt.y > 1 - SLOP2)
-        return false;
+      pt.y = setTwelfths(pt.y);
       //$FALL-THROUGH$
     case 1:
-    if (pt.x < -SLOP2 || pt.x > 1 - SLOP2)
-      return false;
+      pt.x = setTwelfths(pt.x);
+      break;
     }
-    return true;
   }
 
-  public boolean isWithinUnitCell(double a, double b, double c, P3 pt) {
-    switch (dimension) {
-    case 3:
-      if (pt.z < c - 1d - SLOP || pt.z > c + SLOP)
-        return false;
-      //$FALL-THROUGH$
-    case 2:
-      if (pt.y < b - 1d - SLOP || pt.y > b + SLOP)
-        return false;
-      //$FALL-THROUGH$
-    case 1:
-    if (pt.x < a - 1d - SLOP || pt.x > a + SLOP)
-      return false;
-    }
-    return true;
+  private float setTwelfths(float x) {
+    int i = twelfthsOf(x);
+    return (i >= 0 ? i / 12f : x);
   }
 
+
+
+
   ////// lattice methods //////
   
   /**
@@ -624,6 +701,10 @@ public class SimpleUnitCell {
    * 1000000000 - 1999999999 --> center 500, 500, 500; range 0 to 999 or -500 to
    * +499
    * 
+   * for example, a 3x3x3 block of 27 cells:
+   * 
+   * {444 666 1} or {1494949 1515151 1} or {1499499499 1501501501 1}
+   * 
    * @param nnn
    * @param cell
    * @param offset
@@ -634,7 +715,6 @@ public class SimpleUnitCell {
    *        precision issues so we use P4 {1iiijjjkkk 1iiijjjkkk scale,
    *        1kkkkkk}. Here, our offset -- initially 0 or 1 from the uccage
    *        renderer, but later -500 or -499 -- tells us which code we are
-   *        looking at, the first one or the second one.
    * 
    */
   public static void ijkToPoint3f(int nnn, P3 cell, int offset, int kcode) {
@@ -723,10 +803,52 @@ public class SimpleUnitCell {
     }
   }
   
+  /**
+   * @param params
+   * @return true if approximately hexagonal
+   */
+  public static boolean isHexagonal(float[] params) {
+    // a == b && alpha = beta = 90 && gamma = 120 (gamma -1 is a "rotateHexCell" indicator for AFLOW
+    return (approx0(params[0] - params[1]) 
+        && approx0(params[3] - 90) && approx0(params[4] - 90) && (approx0(params[5] - 120) 
+            || params[5] == -1));
+  }
+
+  /**
+   * @param params
+   * @return true if approximately rhombohedral
+   */
+  public static boolean isRhombohedral(float[] params) {
+    // a = b = c and alpha = beta = gamma and alpha != 90
+    return (approx0(params[0] - params[1]) && approx0(params[1] - params[2])
+        && !approx0(params[3] - 90) && approx0(params[3] - params[4])
+        && approx0(params[4] - params[5]));
+  }
+
+  /**
+   * 
+   * @param f
+   * @return true or false
+   */
+  protected static boolean approx0(float f) {
+    return (Math.abs(f) < SLOP_PARAMS);
+  }
   
+  public static int getCellRange(T3 fset, P3[] cellRange) {
+	    int t3w = (fset instanceof T4 ? (int) ((T4) fset).w : 0);
+	    SimpleUnitCell.ijkToPoint3f((int) fset.x, cellRange[0], 0, t3w);
+	    SimpleUnitCell.ijkToPoint3f((int) fset.y, cellRange[1], 1, t3w);
+	    if (fset.z < 0) {
+	      cellRange[0].scale(-1 / fset.z);
+	      cellRange[1].scale(-1 / fset.z);
+	    }
+	    return t3w;
+	  }
+
   @Override
   public String toString() {
     return "[" + a + " " + b + " " + c + " " + alpha + " " + beta + " " + gamma + "]";
   }
+
   
 }
