@@ -447,10 +447,11 @@ public class ActionManager implements EventManager {
   public final static int PICKING_IDENTIFY_BOND = 35;
   public final static int PICKING_DRAG_LIGAND = 36;
   public final static int PICKING_DRAG_MODEL = 37;
+  public final static int PICKING_SYMOP = 38;
 
   private final static String[] pickingModeNames;
   static {
-    pickingModeNames = "off identify label center draw spin symmetry deleteatom deletebond atom group chain molecule polymer structure site model element measure distance angle torsion sequence navigate connect struts dragselected dragmolecule dragatom dragminimize dragminimizemolecule invertstereo assignatom assignbond rotatebond identifybond dragligand dragmodel".split(" ");
+    pickingModeNames = "off identify label center draw spin symmetry deleteatom deletebond atom group chain molecule polymer structure site model element measure distance angle torsion sequence navigate connect struts dragselected dragmolecule dragatom dragminimize dragminimizemolecule invertstereo assignatom assignbond rotatebond identifybond dragligand dragmodel symop".split(" ");
   }
   
   public final static String getPickingModeName(int pickingMode) {
@@ -460,8 +461,11 @@ public class ActionManager implements EventManager {
 
   public final static int getPickingMode(String str) {
     for (int i = pickingModeNames.length; --i >= 0;)
-      if (str.equalsIgnoreCase(pickingModeNames[i]))
+      if (str.equalsIgnoreCase(pickingModeNames[i])) {
+        if (i == PICKING_SYMOP)
+          i=PICKING_SYMMETRY; 
         return i;
+      }
     return -1;
   }
 
@@ -546,12 +550,12 @@ public class ActionManager implements EventManager {
     String script = ";set modelkitMode " + vwr.getBoolean(T.modelkitmode)
         + ";set picking " + getPickingModeName(apm);
     if (apm == PICKING_ASSIGN_ATOM)
-      script += "_" + vwr.getModelkitPropertySafely("atomType");
+      script += "_" + vwr.getModelkitPropertySafely(JC.MODELKIT_ATOMTYPE);
     script += ";";
     if (bondPickingMode != PICKING_OFF)
       script += "set picking " + getPickingModeName(bondPickingMode);
     if (bondPickingMode == PICKING_ASSIGN_BOND)
-      script += "_" + vwr.getModelkitPropertySafely("bondType");
+      script += "_" + vwr.getModelkitPropertySafely(JC.MODELKIT_BONDTYPE);
     script += ";";
     return script;
   }
@@ -793,8 +797,7 @@ public class ActionManager implements EventManager {
     switch (key) {
     case Event.VK_ALT:
       if (dragSelectedMode && isAltKeyReleased)
-        vwr.moveSelected(Integer.MIN_VALUE, 0, Integer.MIN_VALUE,
-            Integer.MIN_VALUE, Integer.MIN_VALUE, null, false, false, modifiers);
+        vwr.moveSelectedXY(Integer.MIN_VALUE, 0, modifiers);
       isAltKeyReleased = false;
       moved.modifiers |= Binding.ALT;
       break;
@@ -848,9 +851,7 @@ public class ActionManager implements EventManager {
     case Event.VK_ALT:
       moved.modifiers &= ~Binding.ALT;
       if (dragSelectedMode)
-        vwr.moveSelected(Integer.MAX_VALUE, 0, Integer.MIN_VALUE,
-            Integer.MIN_VALUE, Integer.MIN_VALUE, null, false, false,
-            moved.modifiers);
+        vwr.moveSelectedXY(Integer.MAX_VALUE, 0, moved.modifiers);
       isAltKeyReleased = true;
       break;
     case Event.VK_SHIFT:
@@ -1089,7 +1090,7 @@ public class ActionManager implements EventManager {
           updateModelkitBranch(bi, true);
         }
       }
-      int[] xy = (int[]) vwr.getModelkitPropertySafely("screenXY");
+      int[] xy = (int[]) vwr.getModelkitPropertySafely(JC.MODELKIT_SCREENXY);
         mkBondPressed = (xy != null && pressed.inRange(10, xy[0], xy[1]));
         
       return;
@@ -1102,8 +1103,7 @@ public class ActionManager implements EventManager {
       haveSelection = (!isDragSelectedAction || vwr
           .findNearestAtomIndexMovable(x, y, true) >= 0);
       if (haveSelection && bnd(dragAction, ACTION_dragSelected, ACTION_dragZ))
-        vwr.moveSelected(Integer.MIN_VALUE, 0, Integer.MIN_VALUE,
-            Integer.MIN_VALUE, Integer.MIN_VALUE, null, false, false, buttonMods);
+        vwr.moveSelectedXY(Integer.MIN_VALUE, 0, buttonMods);
       return;
     }
     //   if (vwr.g.useArcBall)
@@ -1140,7 +1140,7 @@ public class ActionManager implements EventManager {
         if (dragAtomIndex >= 0) {
           updateModelkitBranch(bi, false);
         }
-        vwr.moveSelected(deltaX, deltaY, Integer.MIN_VALUE, x, y, null, false,
+        vwr.moveSelected(deltaX, deltaY, Integer.MIN_VALUE, x, y, null, null, null, false,
             false, dragAtomIndex >= 0 ? 0 : Event.VK_SHIFT);
         return;
       }
@@ -1183,7 +1183,8 @@ public class ActionManager implements EventManager {
           vwr.moveAtomWithHydrogens(dragAtomIndex, deltaX, deltaY,
               (bnd(dragWheelAction, ACTION_dragZ) ? -deltaY
                   : Integer.MIN_VALUE),
-              bs);
+              null, bs);
+          vwr.checkCoordinatesChanged(null);
         }
         // NAH! if (atomPickingMode == PICKING_DRAG_MINIMIZE_MOLECULE && (dragGesture.getPointCount() % 5 == 0))
         //  minimize(false);
@@ -1234,7 +1235,7 @@ public class ActionManager implements EventManager {
         vwr.undoMoveActionClear(iatom, AtomCollection.TAINT_COORD, true);
       else
         vwr.moveSelected(Integer.MAX_VALUE, 0, Integer.MIN_VALUE,
-            Integer.MIN_VALUE, Integer.MIN_VALUE, null, false, false,
+            Integer.MIN_VALUE, Integer.MIN_VALUE, null, null, null, false, false,
             buttonmods);
       dragSelected(dragWheelAction, deltaX, deltaY, false);
       return;
@@ -1299,7 +1300,7 @@ public class ActionManager implements EventManager {
   }
 
   private void updateModelkitBranch(int bondIndex, boolean isClick) {
-    vwr.setModelkitPropertySafely(isClick? "branchAtomClicked" : "branchAtomDragged", Integer.valueOf(dragAtomIndex));
+    vwr.setModelkitPropertySafely(isClick? JC.MODELKIT_BRANCH_ATOM_PICKED : JC.MODELKIT_BRANCH_ATOM_DRAGGED, Integer.valueOf(dragAtomIndex));
     if (measurementQueued == null || measurementQueued.numSet == 0 || mp == null) {
       vwr.setPendingMeasurement(vwr.getModelkit(false).setBondMeasure(bondIndex,
           measurementQueued = mp = getMP()));
@@ -1351,7 +1352,7 @@ public class ActionManager implements EventManager {
           deltaX,
           deltaY,
           (isPickingDrag && bnd(a, ACTION_dragZ) ? -deltaY : Integer.MIN_VALUE),
-          Integer.MIN_VALUE, Integer.MIN_VALUE, null, true, false, dragged.modifiers);
+          Integer.MIN_VALUE, Integer.MIN_VALUE, null, null, null, true, false, dragged.modifiers);
   }
 
 
@@ -1376,7 +1377,7 @@ public class ActionManager implements EventManager {
         exitMeasurementMode(null);
         return;
       } else if (bondPickingMode == PICKING_ROTATE_BOND) {
-        vwr.setModelkitPropertySafely("bondAtomIndex", Integer.valueOf(dragAtomIndex));
+        vwr.setModelkitPropertySafely(JC.MODELKIT_ROTATE_BOND_ATOM_INDEX, Integer.valueOf(dragAtomIndex));
         exitMeasurementMode(null);
         return;
       }
@@ -1399,8 +1400,7 @@ public class ActionManager implements EventManager {
       return;
     }
     if (haveSelection && dragSelectedMode && bnd(dragAction, ACTION_dragSelected))
-      vwr.moveSelected(Integer.MAX_VALUE, 0, Integer.MIN_VALUE,
-          Integer.MIN_VALUE, Integer.MIN_VALUE, null, false, false, dragged.modifiers);
+      vwr.moveSelectedXY(Integer.MAX_VALUE, 0, dragged.modifiers);
 
     if (dragRelease
         && checkUserAction(pressAction, x, y, 0, 0, time, Event.RELEASED))
@@ -1424,6 +1424,14 @@ public class ActionManager implements EventManager {
     // points are always picked up first, then atoms
     // so that atom picking can be superceded by draw picking
     // Binding.MOVED is used for some vwr methods.
+    
+    if (clickedCount > 1 && (apm == PICKING_SPIN || apm == PICKING_SYMMETRY)) {
+      resetMeasurement();
+      runScript("draw ID sym* delete;spin off;");
+      // no double clicks if spinning or selecting atoms for symmetry description
+      return;
+    }
+        
     if (clickedCount > 0) {
       if (checkUserAction(clickAction, x, y, 0, 0, time, Binding.CLICK))
         return;
@@ -1555,7 +1563,7 @@ public class ActionManager implements EventManager {
   }
 
   private void doPopup(int x, int y) {
-    vwr.popupMenu(x, y, vwr.getBoolean(T.modelkitmode) ? 'm' : 'j');
+    vwr.popupMenu(x, y, vwr.isModelKitOpen() ? 'm' : 'j');
   }
 
   private void pickLabel(int iatom) {
@@ -1940,8 +1948,10 @@ public class ActionManager implements EventManager {
       return;
     case PICKING_SPIN:
     case PICKING_SYMMETRY:
-      if (bnd(clickAction, ACTION_pickAtom))
+      if (bnd(clickAction, ACTION_pickAtom)) {
         checkTwoAtomAction(ptClicked, atomIndex);
+        return;
+      }
     }
     if (ptClicked != null)
       return;
@@ -2029,7 +2039,7 @@ public class ActionManager implements EventManager {
     switch (bondPickingMode) {
     case PICKING_ASSIGN_BOND:
       vwr.undoMoveActionClear(-1, T.save, true);
-      vwr.setModelkitPropertySafely("scriptAssignBond", Integer.valueOf(index));
+      vwr.setModelkitPropertySafely(JC.MODELKIT_ASSIGN_BOND, Integer.valueOf(index));
       break;
     case PICKING_ROTATE_BOND:
       // done separately
@@ -2041,6 +2051,9 @@ public class ActionManager implements EventManager {
 
   private void checkTwoAtomAction(Point3fi ptClicked, int atomIndex) {
     boolean isSpin = (apm == PICKING_SPIN);
+    boolean isSymmetry = (apm == PICKING_SYMMETRY);
+    if (!isSpin && !isSymmetry)
+      return;
     if (vwr.tm.spinOn || vwr.tm.navOn
         || vwr.getPendingMeasurement() != null) {
       resetMeasurement();
@@ -2060,6 +2073,8 @@ public class ActionManager implements EventManager {
           return;
       }
     }
+    if (!isSpin && !isSymmetry)
+      return;
     if (atomIndex >= 0 || ptClicked != null)
       queuedAtomCount = queueAtom(atomIndex, ptClicked);
     if (queuedAtomCount < 2) {
@@ -2076,10 +2091,11 @@ public class ActionManager implements EventManager {
       return;
     }
     String s = measurementQueued.getMeasurementScript(" ", false);
+    resetMeasurement();
     if (isSpin)
       runScript("spin" + s + " " + vwr.getInt(T.pickingspinrate));
     else
-      runScript("draw symop " + s + ";show symop " + s);
+      runScript("draw symop " + s + ";print 'all:';show symop " + s);
   }
 
   private void reset() {
