@@ -1,7 +1,7 @@
 /* $RCSfile$
  * $Author: hansonr $
- * $Date: 2023-01-05 08:42:06 -0600 (Thu, 05 Jan 2023) $
- * $Revision: 22500 $
+ * $Date: 2024-04-16 21:43:34 -0500 (Tue, 16 Apr 2024) $
+ * $Revision: 22613 $
  *
  * Copyright (C) 2003-2005  Miguel, Jmol Development Team
  *  
@@ -146,7 +146,8 @@ public class FileManager implements BytePoster {
 
   public String getFullPathName(boolean orPrevious) {
     String f =(fullPathName != null ? fullPathName : nameAsGiven);
-    return (!orPrevious || !f.equals(JC.ZAP_TITLE) ? f : lastFullPathName != null ? lastFullPathName : lastNameAsGiven);
+    return (!orPrevious || !f.equals(JC.ZAP_TITLE) && !f.equals("string") ? f 
+        : lastFullPathName != null ? lastFullPathName : lastNameAsGiven);
   }
 
   private String lastFileType;
@@ -683,7 +684,7 @@ public class FileManager implements BytePoster {
       } else if (Rdr.isTar(bis)) {
          o = vwr.getJzt().getZipFileDirectory(bis, subFileList, 1,
                   forceInputStream);
-      } else if (Rdr.isBinary(bis, 16)) {
+      } else if (Rdr.isBinary(bis, 32)) {
         return bis;
       }
       if (o != null) {
@@ -837,6 +838,10 @@ public class FileManager implements BytePoster {
       return false;
     }
     try {
+      if (t instanceof BufferedInputStream) {
+        if (name.toUpperCase().endsWith(".PNG") || isEmbeddable(name))
+          t = Rdr.getBufferedReader((BufferedInputStream) t, null);
+      }
       return Rdr.readAllAsString((BufferedReader) t, nBytesMax, allowBinary,
           data, 1);
     } catch (Exception e) {
@@ -950,6 +955,8 @@ public class FileManager implements BytePoster {
       return names;
     }
     name = vwr.resolveDatabaseFormat(name);
+    if (name == null)
+      return new String[] { null };
     if (name.indexOf(":") < 0 && name.indexOf("/") != 0)
       name = addDirectory(vwr.getDefaultDirectory(), name);
     if (appletDocumentBaseURL == null) {
@@ -1112,7 +1119,7 @@ public class FileManager implements BytePoster {
       vwr.setStringProperty("defaultDirectoryLocal", path);
   }
 
-  public static String getLocalPathForWritingFile(Viewer vwr, String file) {
+  public static String getLocalPathForWritingFile(Viewer vwr, String file, boolean forDialog) {
     if (file.startsWith("http://") || file.startsWith("https://"))
       return file;
     file = PT.rep(file, "?", "");
@@ -1122,7 +1129,7 @@ public class FileManager implements BytePoster {
       return file;
     GenericFileInterface dir = null;
     try {
-      dir = getLocalDirectory(vwr, false);
+      dir = getLocalDirectory(vwr, forDialog);
     } catch (Exception e) {
       // access control for unsigned applet
     }
@@ -1182,8 +1189,11 @@ public class FileManager implements BytePoster {
           byte[] buf = new byte[300];
           is.read(buf, 0, 300);
           is.reset();
-          if ((buf[0] & 0xFF) == 0x83) // Finite map(3)
+          if ((buf[0] & 0xFF) == 0x83) {
+            // Finite map(3)
+            if (buf[10] == 'D' && buf[11] == 'e' && buf[12] == 'n')
             return "BCifDensity";
+          }
           if (buf[0] == 'P' && buf[1] == 'M' && buf[2] == 1 && buf[3] == 0)//          "PM\1\0"
             return "Pmesh";
           if (buf[208] == 'M' && buf[209] == 'A' && buf[210] == 'P')//          "MAP" at 208
