@@ -77,6 +77,7 @@ public class GlobalSettings {
       legacyAutoBonding = g.legacyAutoBonding;
       legacyHAddition = g.legacyHAddition;
       legacyJavaFloat = g.legacyJavaFloat;
+      minimizationReportSteps = g.minimizationReportSteps;
       doublePrecision = g.doublePrecision;
       bondingVersion = g.bondingVersion;
       platformSpeed = g.platformSpeed;
@@ -255,6 +256,7 @@ public class GlobalSettings {
     setF("drawFontSize", drawFontSize);
     setB("drawPicking", drawPicking);
     setB("dsspCalculateHydrogenAlways", dsspCalcHydrogen);
+    setB("elementkey", elementKey);
 //    setO("edsUrlFormat", edsUrlFormat);
 //    setO("edsUrlFormatDiff", edsUrlFormatDiff);
 //    //setParameterValue("edsUrlOptions", edsUrlOptions);
@@ -316,6 +318,7 @@ public class GlobalSettings {
     setB("messageStyleChime", messageStyleChime);
     setF("minBondDistance", minBondDistance);
     setI("minPixelSelRadius", minPixelSelRadius);
+    setI("minimizationReportSteps", minimizationReportSteps);
     setI("minimizationSteps", minimizationSteps);
     setB("minimizationRefresh", minimizationRefresh);
     setB("minimizationSilent", minimizationSilent);
@@ -390,6 +393,7 @@ public class GlobalSettings {
     setF("strutLengthMaximum", strutLengthMaximum);
     setI("strutSpacing", strutSpacing);
     setB("strutsMultiple", strutsMultiple);
+    setB("symmetryhermannmauguin", symmetryHermannMauguin);
     setB("testFlag1", testFlag1);
     setB("testFlag2", testFlag2);
     setB("testFlag3", testFlag3);
@@ -483,7 +487,6 @@ public class GlobalSettings {
   boolean pdbSequential = false; // true for no bonding check
   int percentVdwAtom = JC.DEFAULT_PERCENT_VDW_ATOM;
   int smallMoleculeMaxAtoms = 40000;
-  int minimizationMaxAtoms = 200;
   boolean smartAromatic = true;
   boolean zeroBasedXyzRasmol = false;
   boolean legacyAutoBonding = false;
@@ -521,6 +524,8 @@ public class GlobalSettings {
   boolean minimizationRefresh = true;
   boolean minimizationSilent = false;
   float minimizationCriterion = 0.001f;
+  int minimizationMaxAtoms = 200;
+  int minimizationReportSteps = 10;
 
   //rendering
 
@@ -626,6 +631,7 @@ public class GlobalSettings {
   boolean drawHover = false;
   boolean drawPicking = false;
   boolean dsspCalcHydrogen = true;
+  boolean elementKey = false;
   public String energyUnits = "kJ";
   float exportScale = 0f;
   String helpPath = JC.DEFAULT_HELP_PATH;
@@ -666,6 +672,7 @@ public class GlobalSettings {
   float strutLengthMaximum = 7.0f;
   float strutDefaultRadius = JC.DEFAULT_STRUT_RADIUS;
   boolean strutsMultiple = false; //on a single position    
+  boolean symmetryHermannMauguin = false;
   //boolean useArcBall = false;
   boolean useMinimizationThread = true;
   boolean useNumberLocalization = true;
@@ -996,8 +1003,7 @@ public class GlobalSettings {
       + ";selecthetero;selecthydrogen"
       // removed in Jmol 14.32.68
       + ";pointgrouplineartolerance;pointgroupdistancetolerance"//;pointgroupmmaxatoms"
-      
-      + ";")
+      + ";minimizationreportsteps;elementkey;symmetryhermannmauguin;")
       .toLowerCase();
 
   Object getAllVariables() {
