@@ -1,7 +1,7 @@
 /* $RCSfile$
  * $Author: hansonr $
- * $Date: 2022-06-03 05:24:02 -0500 (Fri, 03 Jun 2022) $
- * $Revision: 22425 $
+ * $Date: 2024-05-02 13:36:04 -0500 (Thu, 02 May 2024) $
+ * $Revision: 22623 $
 
  *
  * Copyright (C) 2003-2005  Miguel, Jmol Development, www.jmol.org
@@ -30,6 +30,7 @@ import java.util.Hashtable;
 import java.util.Map;
 import java.util.Properties;
 
+import org.jmol.i18n.GT;
 import org.jmol.script.T;
 import org.jmol.util.Elements;
 import org.jmol.util.Logger;
@@ -45,6 +46,13 @@ import javajs.J2SRequireImport;
 @J2SRequireImport({ javajs.util.SB.class })
 public final class JC {
 
+  public final static int AXIS_A = 6;
+  public final static String[] axisLabels = { "+X", "+Y", "+Z", null, null,
+      null, "a", "b", "c", "X", "Y", "Z", null, null, null, "X", null, "Z",
+      null, "(Y)", null };
+
+  public final static String[] axesTypes = { "a", "b", "c", "x", "y", "z" };
+
   public static final String NBO_TYPES = ";" + "AO;;;;" // 31
       + "PNAO;;" // 32
       + "NAO;;;" // 33
@@ -186,15 +194,18 @@ public final class JC {
   public static final String defaultMacroDirectory = "https://chemapps.stolaf.edu/jmol/macros";
 
   private final static String[] databaseArray = {
-      // still http:
+      "itatable", 
+      "https://www.cryst.ehu.es/cgi-bin/cryst/programs/nph-getgen?gnum=$FILE&what=gp&list=Standard%2FDefault+Setting",
+      "itadiagram",
+      "https://onlinelibrary.wiley.com/iucr/itc/Ac/ch2o3v0001/sgtable2o3o%FILE",
       "aflowbin",
       "http://aflowlib.mems.duke.edu/users/jmolers/binary_new/%FILE.aflow_binary",
-      "aflow",
-      "http://aflowlib.mems.duke.edu/users/jmolers/binary_new/%FILE.aflow_binary",
+      "aflowlib", "https://aflow.org/p/%FILE.cif", // updated 2024.04.22
+      "aflowpro", "$aflowlib",
       // _#DOCACHE_ flag indicates that the loaded file should be saved in any state in full
       // ' at start indicates a Jmol script evaluation
       "ams",
-      "'http://rruff.geo.arizona.edu/AMS/viewJmol.php?'+(0+'%file'==0? 'mineral':('%file'.length==7? 'amcsd':'id'))+'=%file&action=showcif#_DOCACHE_'",
+      "'https://rruff.geo.arizona.edu/AMS/viewJmol.php?'+(0+'%file'==0? 'mineral':('%file'.length==7? 'amcsd':'id'))+'=%file&action=showcif#_DOCACHE_'",
       "dssr", "http://dssr-jmol.x3dna.org/report.php?id=%FILE&opts=--json=ebi", //for debugging: -blocks",
       "dssrModel",
       "http://dssr-jmol.x3dna.org/report.php?POST?opts=--json=ebi&model=", // called in DSSR1.java
@@ -208,40 +219,107 @@ public final class JC {
       "magndata", "http://webbdcrista1.ehu.es/magndata/mcif/%FILE.mcif",
       "rna3d", "http://rna.bgsu.edu/rna3dhub/%TYPE/download/%FILE",
       // now https:
-      "mmtf", "https://mmtf.rcsb.org/v1.0/full/%FILE", // new Jmol 14.5.4 4/2016
+      "mmtf", "https://mmtf.rcsb.org/v1.0/full/%FILE", // new Jmol 14.5.4 4/2016 MMTF phase out in 2024.06
+      "bcif", "https://models.rcsb.org/%file.bcif", // new Jmol 16.1.52 2024.01.25
       "chebi",
       "https://www.ebi.ac.uk/chebi/saveStructure.do?defaultImage=true&chebiId=%file%2D%",
       "ligand", "https://files.rcsb.org/ligands/download/%FILE.cif", "mp",
       "https://www.materialsproject.org/materials/mp-%FILE/cif#_DOCACHE_", // e.g. https://materialsproject.org/rest/v1/materials/mp-24972/cif 
-      "nci", "https://cactus.nci.nih.gov/chemical/structure", 
-      "pdb", "https://files.rcsb.org/download/%FILE.pdb", // new Jmol 14.4.4 3/2016
+      "nci", "https://cactus.nci.nih.gov/chemical/structure/", "pdb",
+      "https://files.rcsb.org/download/%FILE.pdb", // new Jmol 14.4.4 3/2016
       "pdb0", "https://files.rcsb.org/download/%FILE.pdb", // used in JSmol
       "pdbe", "https://www.ebi.ac.uk/pdbe/entry-files/download/%FILE.cif",
       "pdbe2", "https://www.ebi.ac.uk/pdbe/static/entry/%FILE_updated.cif",
-      "pubchem", "https://pubchem.ncbi.nlm.nih.gov/rest/pug/compound/%FILE/SDF?record_type=3d",
-      "map", "https://www.ebi.ac.uk/pdbe/api/%TYPE/%FILE?pretty=false&metadata=true",
+      "pubchem",
+      "https://pubchem.ncbi.nlm.nih.gov/rest/pug/compound/%FILE/SDF?record_type=3d",
+      "map",
+      "https://www.ebi.ac.uk/pdbe/api/%TYPE/%FILE?pretty=false&metadata=true",
       "pdbemap", "https://www.ebi.ac.uk/pdbe/coordinates/files/%file.ccp4",
-      "pdbemapdiff", "https://www.ebi.ac.uk/pdbe/coordinates/files/%file_diff.ccp4",
-      "pdbemapserver", "https://www.ebi.ac.uk/pdbe/densities/x-ray/%file/box/0,0,0/0,0,0?detail=6&space=cartesian&encoding=bcif",
-      "pdbemapdiffserver", "https://www.ebi.ac.uk/pdbe/densities/x-ray/%file/box/0,0,0/0,0,0?detail=6&space=cartesian&encoding=bcif&diff=1", // last bit is just mine
+      "pdbemapdiff",
+      "https://www.ebi.ac.uk/pdbe/coordinates/files/%file_diff.ccp4",
+      "pdbemapserver",
+      "https://www.ebi.ac.uk/pdbe/densities/x-ray/%file/box/0,0,0/0,0,0?detail=6&space=cartesian&encoding=bcif",
+      "pdbemapdiffserver",
+      "https://www.ebi.ac.uk/pdbe/densities/x-ray/%file/box/0,0,0/0,0,0?detail=6&space=cartesian&encoding=bcif&diff=1", // last bit is just mine
       //"emdbmap", "https://ftp.ebi.ac.uk/pub/databases/emdb/structures/EMD-%file/map/emd_%file.map.gz", // https did not work in Java due to certificate issues
       // was considerably slower
-      "emdbmap", "https://www.ebi.ac.uk/pdbe/densities/emd/emd-%file/cell?detail=6&space=cartesian&encoding=bcif",
-      "emdbquery", "https://www.ebi.ac.uk/emdb/api/search/fitted_pdbs:%file?fl=emdb_id,map_contour_level_value&wt=csv", // to get the EMDB id from the PDB id
-      "emdbmapserver", "https://www.ebi.ac.uk/pdbe/densities/emd/emd-%file/box/0,0,0/0,0,0?detail=6&space=cartesian&encoding=bcif",
-      "resolverResolver", "https://chemapps.stolaf.edu/resolver",
-  };
+      "emdbmap",
+      "https://www.ebi.ac.uk/pdbe/densities/emd/emd-%file/cell?detail=6&space=cartesian&encoding=bcif",
+      "emdbquery",
+      "https://www.ebi.ac.uk/emdb/api/search/fitted_pdbs:%file?fl=emdb_id,map_contour_level_value&wt=csv", // to get the EMDB id from the PDB id
+      "emdbmapserver",
+      "https://www.ebi.ac.uk/pdbe/densities/emd/emd-%file/box/0,0,0/0,0,0?detail=6&space=cartesian&encoding=bcif",
+      "xxxresolverResolver", "https://chemapps.stolaf.edu/resolver", "smiles2d",
+      "https://cirx.chemicalcreatures.com/chemical/structure/%FILE/file?format=sdf&get3d=false",
+      "smiles3d",
+      "https://cirx.chemicalcreatures.com/chemical/structure/%FILE/file?format=sdf&get3d=true", };
+
+  private final static String defaultOptimadeFieldsStr = ",chemical_formula_descriptive"
+      + ",dimension_types" + ",lattice_vectors" + ",cartesian_site_positions"
+      + ",species_at_sites" + ",species,";
+
+  /**
+   * Get all necessary response fields if an optimade call. Note that the only
+   * conditions we have here is that this is http and "optimade" is in the URL.
+   * Obviously not exactly correct.
+   * 
+   * @param url
+   * @return optimade query
+   */
+  static String fixOptimadeCall(String url) {
+    int pt = url.indexOf("response_fields=") + 16;
+    String a = defaultOptimadeFieldsStr;
+    if (pt < 16) {
+      int ptQ = url.indexOf("?");
+      url += (ptQ < 0 ? "?" : "&") + "response_fields="
+          + a.substring(1, a.length() - 1);
+    } else {
+      String fields = "," + url.substring(pt, (url + "&").indexOf('&', pt))
+          + ",";
+      int flen = fields.length();
+      for (int i = 0, b = 0; i >= 0; i = b) {
+        b = a.indexOf(',', i + 1);
+        if (b < 0)
+          break;
+        String k = a.substring(i, b + 1);
+        if (fields.indexOf(k) < 0) {
+          fields += k.substring(1);
+        }
+      }
+      url = url.substring(0, pt) + fields.substring(1, fields.length() - 1)
+          + url.substring(pt + flen - 2);
+    }
+    return url;
+  }
+
+  //  static {
+  //System.out.println(fixOptimadeCall("test?filter&response_fields=lkadfs"));
+  //System.out.println(fixOptimadeCall("test?response_fields=species"));
+  //System.out.println(fixOptimadeCall("test?response_fields=lkadfs,species"));
+  //System.out.println(fixOptimadeCall("test?response_fields=lkadfs,species_at_sites,species&more"));
+  //System.out.println(fixOptimadeCall("test?filter&response_fields=species,lkadfs&more"));
+  //System.out.println(fixOptimadeCall("test?response_fields=species,lkadfs&more"));
+  //    
+  //    
+  //  }
 
   final static String legacyResolver = "cactus.nci.nih.gov/chemical/structure";
 
   final static Map<String, String> databases = new Hashtable<String, String>();
 
   static {
-    for (int i = 0; i < databaseArray.length; i += 2)
-      databases.put(databaseArray[i].toLowerCase(), databaseArray[i + 1]);
+    for (int i = 0; i < databaseArray.length; i += 2) {
+      String target = databaseArray[i + 1];
+      if (target.charAt(0) == '$') {
+        // alias
+        target = databases.get(target.substring(1));
+      }
+      databases.put(databaseArray[i].toLowerCase(), target);
+    }
   }
 
-  static String resolveDataBase(String database, String id, String format) {
+  public static String resolveDataBase(String database, String id,
+                                       String format) {
     if (format == null) {
       if ((format = databases.get(database.toLowerCase())) == null)
         return null;
@@ -290,81 +368,18 @@ public final class JC {
             ? resolveDataBase(name.indexOf("/ligand/") >= 0 ? "ligand" : "pdb",
                 name.substring(name.lastIndexOf("/") + 1), null)
             : name.indexOf("http://www.ebi") == 0
+                || name.indexOf("http://rruff") == 0
                 || name.indexOf("http://pubchem") == 0
                 || name.indexOf("http://cactus") == 0
                 || name.indexOf("http://www.materialsproject") == 0
                     ? "https://" + name.substring(7)
-             : name.indexOf("optimade") > 0 ? JC.fixOptimadeCall(name)
-             : name);
+                    : name.indexOf("optimade") > 0 ? JC.fixOptimadeCall(name)
+                        : name);
     if (newname != name)
       Logger.info("JC.fixProtocol " + name + " --> " + newname);
     return newname;
   }
 
-  private final static String defaultOptimadeFieldsStr = 
-      ",chemical_formula_descriptive"
-    + ",dimension_types"
-    + ",lattice_vectors"
-    + ",cartesian_site_positions"
-    + ",species_at_sites"
-    + ",species,"; 
-  
-  /**
-   * Get all necessary response fields if an optimade call. Note that the only
-   * conditions we have here is that this is http and "optimade" is in the URL.
-   * Obviously not exactly correct.
-   * 
-   * @param url
-   * @return call with added necessary response_fields
-   */
-  static String fixOptimadeCall(String url) {
-    int pt = url.indexOf("response_fields=") + 16;
-    String a = defaultOptimadeFieldsStr;
-    if (pt < 16) {
-      int ptQ = url.indexOf("?");
-      url += (ptQ < 0 ? "?" : "&") + "response_fields="
-          + a.substring(1, a.length() - 1);
-    } else {
-      String fields = "," + url.substring(pt, (url + "&").indexOf('&', pt))
-          + ",";
-      int flen = fields.length();
-      for (int i = 0, b = 0; i >= 0; i = b) {
-        b = a.indexOf(',', i + 1);
-        if (b < 0)
-          break;
-        String k = a.substring(i, b + 1);
-        if (fields.indexOf(k) < 0) {
-          fields += k.substring(1);
-        }
-      }
-      url = url.substring(0, pt) + fields.substring(1, fields.length() - 1)
-          + url.substring(pt + flen - 2);
-    }
-    return url;
-  }
-
-//static {
-//  
-//  System.out.println(fixOptimadeCall("test?filter&response_fields=lkadfs"));
-//  System.out.println(fixOptimadeCall("test?response_fields=species"));
-//  System.out.println(fixOptimadeCall("test?response_fields=lkadfs,species"));
-//  System.out.println(fixOptimadeCall("test?response_fields=lkadfs,species_at_sites,species&more"));
-//  System.out.println(fixOptimadeCall("test?filter&response_fields=species,lkadfs&more"));
-//  System.out.println(fixOptimadeCall("test?response_fields=species,lkadfs&more"));
-//  
-//  
-//}
-
-  public static String[] macros = { "aflow",
-      "https://chemapps.stolaf.edu/jmol/macros/AFLOW.spt", "AFLOW macros", "bz",
-      "https://chemapps.stolaf.edu/jmol/macros/bz.spt",
-      "Brillouin Zone/Wigner-Seitz macros", "topology",
-      "https://chemapps.stolaf.edu/jmol/macros/topology.spt",
-      "Topology CIF macros", "topond",
-      "https://chemapps.stolaf.edu/jmol/macros/topond.spt",
-      "CRYSTAL/TOPOND macros", "crystal",
-      "https://chemapps.stolaf.edu/jmol/macros/crystal.spt", "CRYSTAL macros" };
-
   // unused
 
   //  public static String[] macros = {
@@ -445,7 +460,7 @@ public final class JC {
     if (tmpDate != null) {
       tmpDate = tmpDate.substring(7, 23);
       // NOTE : date is updated in the properties by SVN, and is in the format
-      // "$Date: 2022-06-03 05:24:02 -0500 (Fri, 03 Jun 2022) $"
+      // "$Date: 2024-05-02 13:36:04 -0500 (Thu, 02 May 2024) $"
       //  0         1         2
       //  012345678901234567890123456789
     }
@@ -511,17 +526,27 @@ public final class JC {
   public final static String NOTE_SCRIPT_FILE = "NOTE: file recognized as a script file: ";
 
   public static final String SCRIPT_EXT = "\1##";
+  public final static String SCRIPT_GUI = "; ## GUI ##";
+  public static final String SCRIPT_QUIET = "#quiet";
   public static final String SCRIPT_STEP = SCRIPT_EXT + "SCRIPT_STEP";
   public static final String SCRIPT_START = SCRIPT_EXT + "SCRIPT_START";
   public static final String SCRIPT_NOENDCHECK = SCRIPT_EXT + "NOENDCHECK";
   public final static String SCRIPT_ISEDITOR = SCRIPT_EXT + " ISEDITOR";
-  public final static String SCRIPT_EDITOR_IGNORE = SCRIPT_EXT + " EDITOR_IGNORE ##";
+  public final static String SCRIPT_EDITOR_IGNORE = SCRIPT_EXT
+      + " EDITOR_IGNORE ##";
+  public final static String SCRIPT_CONSOLE = SCRIPT_GUI + SCRIPT_EDITOR_IGNORE
+      + SCRIPT_GUI;
+
   public final static String REPAINT_IGNORE = SCRIPT_EXT + " REPAINT_IGNORE ##";
-  public final static String SCRIPT_GUI = "; ## GUI ##";
+
+  public static final int SG_AS_STRING = 1;
+  public static final int SG_IS_ASSIGN = 2;
+  public static final int SG_FROM_SCRATCH = 4;
+  public static final int SG_CHECK_SUPERCELL = 8;
 
   public final static String LOAD_ATOM_DATA_TYPES = ";xyz;vxyz;vibration;temperature;occupancy;partialcharge;";
 
-  public final static float radiansPerDegree = (float) (Math.PI / 180);
+  public final static double radiansPerDegree = Math.PI / 180;
 
   public final static String allowedQuaternionFrames = "RC;RP;a;b;c;n;p;q;x;";
 
@@ -574,6 +599,8 @@ public final class JC {
   /* .cube files need this */
   public final static float ANGSTROMS_PER_BOHR = 0.5291772f;
 
+  public static final int COLOR_CONTRAST = 0xFFfedcba;
+
   public final static int[] altArgbsCpk = { 0xFFFF1493, // Xx 0
       0xFFBFA6A6, // Al 13
       0xFFFFFF30, // S  16
@@ -586,6 +613,9 @@ public final class JC {
       0xFF105050, // 15N  7 - darker
   };
 
+  public final static int FORMAL_CHARGE_COLIX_RED = Elements.elementSymbols.length
+      + altArgbsCpk.length;
+
   // hmmm ... what is shapely backbone? seems interesting
   //public final static int argbShapelyBackbone = 0xFFB8B8B8;
   //public final static int argbShapelySpecial =  0xFF5E005E;
@@ -605,6 +635,9 @@ public final class JC {
       0xFF0000FF, // 7
   };
 
+  public final static int PARTIAL_CHARGE_COLIX_RED = FORMAL_CHARGE_COLIX_RED
+      + argbsFormalCharge.length;
+
   public final static int[] argbsRwbScale = { 0xFFFF0000, // red
       0xFFFF1010, //
       0xFFFF2020, //
@@ -638,10 +671,6 @@ public final class JC {
       0xFF0000FF, // blue
   };
 
-  public final static int FORMAL_CHARGE_COLIX_RED = Elements.elementSymbols.length
-      + altArgbsCpk.length;
-  public final static int PARTIAL_CHARGE_COLIX_RED = FORMAL_CHARGE_COLIX_RED
-      + argbsFormalCharge.length;
   public final static int PARTIAL_CHARGE_RANGE_SIZE = argbsRwbScale.length;
 
   //  $ print  color("red","blue", 33,true)
@@ -909,8 +938,7 @@ public final class JC {
 
       //periodic table
       "@nonmetal _H,_He,_B,_C,_N,_O,_F,_Ne,_Si,_P,_S,_Cl,_Ar,_As,_Se,_Br,_Kr,_Te,_I,_Xe,_At,_Rn",
-      "@metal !nonmetal && !_Xx",
-      "@alkaliMetal _Li,_Na,_K,_Rb,_Cs,_Fr",
+      "@metal !nonmetal && !_Xx", "@alkaliMetal _Li,_Na,_K,_Rb,_Cs,_Fr",
       "@alkalineEarth _Be,_Mg,_Ca,_Sr,_Ba,_Ra",
       "@nobleGas _He,_Ne,_Ar,_Kr,_Xe,_Rn", "@metalloid _B,_Si,_Ge,_As,_Sb,_Te",
       // added La, Ac as per Frank Weinhold - these two are not f-block
@@ -924,10 +952,75 @@ public final class JC {
 
   };
 
+  /**
+   * specifically for ECHO and DRAW to have these specific for a given model and
+   * only appearing when there is only one model showing (see MODELKIT SET KEY
+   * ON)
+   */
+  public static final String THIS_MODEL_ONLY = "_!_";
+  public static final String MODELKIT_ELEMENT_KEY_ID = THIS_MODEL_ONLY
+      + "elkey_";
+
+  public static final String MODELKIT_SET_ELEMENT_KEY = "setelementkey";
+  public static final String MODELKIT_ELEMENT_KEY = "elementkey";
+  public static final String MODELKIT_KEY = "key";
+  public static final String MODELKIT_NEW_MODEL_ATOM_KEYS = "newmodelatomkeys";
+  public static final String MODELKIT_FRAME_RESIZED = "frameresized";
+  public static final String MODELKIT_UDPATE_KEY_STATE = "updatekeysfromstate";
+  public static final String MODELKIT_UPDATE_MODEL_KEYS = "updatemodelkeys";
+  public static final String MODELKIT_UPDATE_ATOM_KEYS = "updateatomkeys";
+  public static final String MODELKIT_ASSIGN_BOND = "assignBond";
+  public static final String MODELKIT_ROTATE_BOND_ATOM_INDEX = "rotateBond";
+  public static final String MODELKIT_BRANCH_ATOM_PICKED = "branchatomclicked";
+  public static final String MODELKIT_BRANCH_ATOM_DRAGGED = "branchatomdragged";
+
+  public static final String MODELKIT_CENTER = "center";
+  public static final String MODELKIT_HIDEMENU = "hidemenu";
+  public static final String MODELKIT_CONSTRAINT = "constraint";
+  public static final String MODELKIT_EXISTS = "exists";
+  public static final String MODELKIT_ISMOLECULAR = "ismolecular";
+  public static final String MODELKIT_ALLOPERATORS = "alloperators";
+  public static final String MODELKIT_DATA = "data";
+  public static final String MODELKIT_MINIMIZING = "minimizing";
+  public static final String MODELKIT_RESET = "reset";
+  public static final String MODELKIT_ATOMPICKINGMODE = "atompickingmode";
+  public static final String MODELKIT_BONDPICKINGMODE = "bondpickingmode";
+  public static final String MODELKIT_HIGHLIGHT = "highlight";
+  public static final String MODELKIT_MODE = "mode";
+  public static final String MODELKIT_APPLYLOCAL = "applylocal";
+  public static final String MODELKIT_RETAINLOCAL = "retainlocal";
+  public static final String MODELKIT_APPLYFULL = "applyfull";
+  public static final String MODELKIT_UNITCELL = "unitcell";
+  public static final String MODELKIT_ADDHYDROGEN = "addhydrogen";
+  public static final String MODELKIT_ADDHYDROGENS = "addhydrogens";
+  public static final String MODELKIT_AUTOBOND = "autobond";
+  public static final String MODELKIT_CLICKTOSETELEMENT = "clicktosetelement";
+  public static final String MODELKIT_HIDDEN = "hidden";
+  public static final String MODELKIT_SHOWSYMOPINFO = "showsymopinfo";
+  public static final String MODELKIT_SYMOP = "symop";
+  public static final String MODELKIT_SYMMETRY = "symmetry";
+  public static final String MODELKIT_HOVERLABEL = "hoverlabel";
+  public static final String MODELKIT_ATOMTYPE = "atomtype";
+  public static final String MODELKIT_BONDTYPE = "bondtype";
+  public static final String MODELKIT_BONDINDEX = "bondindex";
+  public static final String MODELKIT_ROTATEBONDINDEX = "rotatebondindex";
+  public static final String MODELKIT_OFFSET = "offset";
+  public static final String MODELKIT_SCREENXY = "screenxy";
+  public static final String MODELKIT_INVARIANT = "invariant";
+  public static final String MODELKIT_DISTANCE = "distance";
+  public static final String MODELKIT_ADDCONSTRAINT = "addconstraint";
+  public static final String MODELKIT_REMOVECONSTRAINT = "removeconstraint";
+  public static final String MODELKIT_REMOVEALLCONSTRAINTS = "removeallconstraints";
+  public static final String MODELKIT_VIBRATION = "vibration";
+  public static final String MODELKIT_INITIALIZE_MODEL = "initializemodel";
+
+  public static final String MODELKIT_DRAGATOM = "dragatom";
+  public static final String MODELKIT_DELETE_BOND = "deletebond";
   public final static String MODELKIT_ZAP_STRING = "5\n\nC 0 0 0\nH .63 .63 .63\nH -.63 -.63 .63\nH -.63 .63 -.63\nH .63 -.63 -.63";
   public final static String MODELKIT_ZAP_TITLE = "Jmol Model Kit";//do not ever change this -- it is in the state
   public final static String ZAP_TITLE = "zapped";//do not ever change this -- it is in the state
   public final static String ADD_HYDROGEN_TITLE = "Viewer.AddHydrogens"; //do not ever change this -- it is in the state
+  public static final String JMOL_MODEL_KIT = "Jmol Model Kit";
 
   ////////////////////////////////////////////////////////////////
   // font-related
@@ -1228,7 +1321,7 @@ public final class JC {
   public final static int LABEL_ZPOS_GROUP = 0x010;
   public final static int LABEL_ZPOS_FRONT = 0x020;
 
-  public final static int LABEL_EXPLICIT             = 0x040;
+  public final static int LABEL_EXPLICIT = 0x040;
 
   private final static int LABEL_CENTERED = 0x100;
 
@@ -1403,9 +1496,10 @@ public final class JC {
   public static final int SMILES_GEN_EXPLICIT_H_ALL = 0x00001000; // SmilesExt -> generator
   public static final int SMILES_GEN_EXPLICIT_H2_ONLY = 0x00002000; // SmilesExt -> generator
   public static final int SMILES_GEN_TOPOLOGY = 0x00004000; // SmilesExt -> generator
+  public static final int SMILES_GEN_ALL_COMPONENTS = 0x00008000; // SmilesExt -> generator
   public static final int SMILES_GEN_POLYHEDRAL = 0x00010000; // polyhedron -> generator
   public static final int SMILES_GEN_ATOM_COMMENT = 0x00020000; // polyhedron,Viewer -> generator
-  public static final int SMILES_GEN_NO_BRANCHES               = 0x00040000; // MathExt -> Generator
+  public static final int SMILES_GEN_NO_BRANCHES = 0x00040000; // MathExt -> Generator
 
   public static final int SMILES_GEN_BIO = 0x00100000; // MathExt -> generator
   public static final int SMILES_GEN_BIO_ALLOW_UNMATCHED_RINGS = 0x00300000; // MathExt -> generator
@@ -1450,7 +1544,7 @@ public final class JC {
   public final static int UNITID_TRIM = 16;
 
   public static final String DEFAULT_DRAG_DROP_SCRIPT = "zap; load SYNC \"%FILE\";if (%ALLOWCARTOONS && _loadScript == '' && defaultLoadScript == '' && _filetype == 'Pdb') {if ({(protein or nucleic)&*/1.1} && {*/1.1}[1].groupindex != {*/1.1}[0].groupindex){select protein or nucleic;cartoons only;}if ({visible && cartoons > 0}){color structure}else{wireframe -0.1};if (!{visible}){spacefill 23%};select *}";
-  public static final float UC_TOLERANCE2 =  0.0014f*0.0014f; // always use check for LT this, not LTE
+  public static final float UC_TOLERANCE2 = 0.0014f * 0.0014f; // always use check for LT this, not LTE
 
   /**
    * Get a unitID type
@@ -1475,18 +1569,11 @@ public final class JC {
   }
 
   public final static String[] globalBooleans = {
-  "someModelsHaveFractionalCoordinates", 
-  "someModelsHaveSymmetry",
-  "someModelsHaveUnitcells", 
-  "someModelsHaveCONECT", 
-  "isPDB", 
-  "someModelsHaveDomains", 
-  "someModelsHaveValidations", 
-  "isSupercell",
-  "someModelsHaveAromaticBonds",
-  "someModelsAreModulated"
-  };
-  
+      "someModelsHaveFractionalCoordinates", "someModelsHaveSymmetry",
+      "someModelsHaveUnitcells", "someModelsHaveCONECT", "isPDB",
+      "someModelsHaveDomains", "someModelsHaveValidations", "isSupercell",
+      "someModelsHaveAromaticBonds", "someModelsAreModulated" };
+
   public final static int GLOBAL_FRACTCOORD = 0;
   public final static int GLOBAL_SYMMETRY = 1;
   public final static int GLOBAL_UNITCELLS = 2;
@@ -1501,6 +1588,62 @@ public final class JC {
   public static String getBoolName(int g) {
     return globalBooleans[g];
   }
-  
+
+  public final static float FLOAT_MIN_SAFE = 2E-45f;
+  // Float.MIN_VALUE (1.45E-45) is not reliable with JavaScript because of the float/double difference there
+
+  public static final String INFO_SPACE_GROUP = "spaceGroup";
+  public static final String INFO_SPACE_GROUP_ASSIGNED = "spaceGroupAssigned";
+  public static final String INFO_SPACE_GROUP_INFO = "spaceGroupInfo";
+  public static final String INFO_SPACE_GROUP_INDEX = "spaceGroupIndex";
+  public static final String INFO_SPACE_GROUP_TITLE = "spaceGroupTitle";
+  public static final String INFO_SPACE_GROUP_NAME = "spaceGroupName";
+  public static final String INFO_SPACE_GROUP_NOTE = "spaceGroupNote";
+  public static final String INFO_UNIT_CELL_PARAMS = "unitCellParams";
+  public static final String INFO_UNIT_CELL_OFFSET = "unitCellOffset";
+  public static final String INFO_UNIT_CELL_CONVENTIONAL = "unitcell_conventional";
+  /**
+   * used to set atom symmetry 555 556 etc.
+   */
+  public static final String INFO_UNIT_CELL_RANGE = "ML_unitCellRange";
+  //  public static final String INFO_UNIT_CELL_TRANSLATIONS = "unitCellTranslations";
+  public static final String INFO_UNIT_CELLS = "unitCells";
+  public static final String INFO_UNIT_CELL = "unitcell";
+
+  /**
+   * When UNITCELL NONE is given, clear out all space group and unit cell keys
+   * from model info.
+   * 
+   * @param key
+   * @return true to delete
+   */
+  public static boolean isSpaceGroupInfoKey(String key) {
+    return (key.indexOf("nitCell") >= 0
+        || key.equals("coordinatesAreFractional")
+        || key.startsWith("spaceGroup") || key.indexOf("ymmet") >= 0
+        || key.startsWith("lattice") || key.startsWith("intlTable"));
+  }
+
+  public static final String PROP_DELETE_MODEL_ATOMS = "deleteModelAtoms";
+  public static final String PROP_ATOMS_DELETED = "atomsDeleted";
+  public static final String PROP_ATOMS_MOVED = "atomsMoved";
+
+  public static String getMenuScript(String type) {
+    if (type == "openPDB") {
+      return "var x__id__ = _modelTitle; if (x__id__.length != 4) { x__id__ = '1crn'};x__id__ = prompt('"
+          + GT.$(
+              "Enter a four-digit PDB model ID or \"=\" and a three-digit ligand ID")
+          + "',x__id__);if (!x__id__) { quit }; load @{'=' + x__id__}";
+
+    }
+    if (type == "openMOL") {
+      return "var x__id__ = _smilesString; if (!x__id__) { x__id__ = 'tylenol'};x__id__ = prompt('"
+          + GT.$(
+              "Enter the name or identifier (SMILES, InChI, CAS) of a compound. Preface with \":\" to load from PubChem; otherwise Jmol will use the NCI/NIH Resolver.")
+          + "',x__id__);if (!x__id__) { quit }; load @{(x__id__[1]==':' ? x__id__ : '$' + x__id__)}";
+
+    }
+    return null;
+  }
 
 }
