@@ -633,7 +633,10 @@ abstract class OutputManager {
       fileName = fileName.substring(1);
     useDialog |= (vwr.isApplet && fileName.indexOf("http:") != 0
         && fileName.indexOf("https:") != 0);
-    fileName = FileManager.getLocalPathForWritingFile(vwr, fileName);
+    fileName = FileManager.getLocalPathForWritingFile(vwr, fileName, useDialog);
+    if (vwr.isApplet) {
+      fileName = fileName.substring(fileName.lastIndexOf("/") + 1);
+    }
     if (useDialog)
       fileName = vwr.dialogAsk(
           quality == Integer.MIN_VALUE ? "Save" : "Save Image", fileName,
