@@ -1,7 +1,7 @@
 /* $RCSfile$
  * $Author: hansonr $
- * $Date: 2023-01-05 08:42:06 -0600 (Thu, 05 Jan 2023) $
- * $Revision: 22500 $
+ * $Date: 2023-11-13 16:26:50 -0600 (Mon, 13 Nov 2023) $
+ * $Revision: 22583 $
  *
  * Copyright (C) 2003-2005  The Jmol Development Team
  *
@@ -24,6 +24,7 @@
 package org.jmol.viewer;
 
 import java.util.Arrays;
+import java.util.Calendar;
 import java.util.Enumeration;
 import java.util.Hashtable;
 import java.util.Map;
@@ -599,7 +600,7 @@ public class PropertyManager implements JmolPropertyManager {
                         v = null;
                     }
                     if (v == null)
-                      v2.remove(j);
+                      v2.removeItemAt(j);
                     else
                       v2.set(j, v);
                   }
@@ -1896,7 +1897,8 @@ public class PropertyManager implements JmolPropertyManager {
     BS bsAtoms = null;
     BS bsWritten = new BS();
     char ctype = '\0';
-    LabelToken[] tokens = vwr.ms.getLabeler().compile(vwr,
+    vwr.ms.getLabeler();
+    LabelToken[] tokens = LabelToken.compile(vwr,
         "ATOM  %-6i%4a%1A%3.-3n %1c%4R%1E   ", '\0', null);
     if (parameters == null) {
       ctype = (type.length() > 11 && type.indexOf("quaternion ") >= 0 ? type
@@ -2086,4 +2088,35 @@ public class PropertyManager implements JmolPropertyManager {
   }
 
 
+  public static String getSDFDateLine(String version, boolean is2d) {
+    SB mol = SB.newS((version + "          ").substring(0, 10));
+    int cMM, cDD, cYYYY, cHH, cmm;
+    /**
+     * @j2sNative
+     * 
+     *            var c = new Date(); cMM = c.getMonth(); cDD = c.getDate();
+     *            cYYYY = c.getFullYear(); cHH = c.getHours(); cmm =
+     *            c.getMinutes();
+     */
+    {
+      Calendar c = Calendar.getInstance();
+      cMM = c.get(Calendar.MONTH);
+      cDD = c.get(Calendar.DAY_OF_MONTH);
+      cYYYY = c.get(Calendar.YEAR);
+      cHH = c.get(Calendar.HOUR_OF_DAY);
+      cmm = c.get(Calendar.MINUTE);
+    }
+    PT.rightJustify(mol, "00", "" + (1 + cMM));
+    PT.rightJustify(mol, "00", "" + cDD);
+    mol.append(("" + cYYYY).substring(2, 4));
+    PT.rightJustify(mol, "00", "" + cHH);
+    PT.rightJustify(mol, "00", "" + cmm);
+    mol.append(is2d ? "2" : "3").append("D 1   1.00000     0.00000     0");
+    //       This line has the format:
+    //  IIPPPPPPPPMMDDYYHHmmddSSssssssssssEEEEEEEEEEEERRRRRR
+    //  A2<--A8--><---A10-->A2I2<--F10.5-><---F12.5--><-I6->
+    mol.append("\n");
+    return mol.toString();
+  }
+
 }
