@@ -273,6 +273,42 @@ public class StateCreator extends JmolStateCreator {
       }
 
       SB sb = new SB();
+      
+      boolean loadUC = false;
+      if (ms.unitCells != null) {
+        boolean haveModulation = false;
+        for (int i = 0; i < modelCount; i++) {
+          SymmetryInterface symmetry = ms.getUnitCell(i);
+          if (symmetry == null)
+            continue;
+          sb.setLength(0);
+          if (symmetry.getState(ms, i, sb)) {
+            loadUC = true;
+            commands.append("  frame ").append(ms.getModelNumberDotted(i))
+                .appendSB(sb).append(";\n");
+          }
+          haveModulation |= (vwr.ms.getLastVibrationVector(i, T.modulation) >= 0);
+        }
+        if (loadUC)
+          vwr.shm.loadShape(JC.SHAPE_UCCAGE); // just in case
+        getShapeStatePriv(commands, isAll, JC.SHAPE_UCCAGE);
+        if (haveModulation) {
+          Map<String, BS> temp = new Hashtable<String, BS>();
+          int ivib;
+          for (int i = modelCount; --i >= 0;) {
+            if ((ivib = vwr.ms.getLastVibrationVector(i, T.modulation)) >= 0)
+              for (int j = models[i].firstAtomIndex; j <= ivib; j++) {
+                JmolModulationSet mset = ms.getModulation(j);
+                if (mset != null)
+                  BSUtil.setMapBitSet(temp, j, j, mset.getState());
+              }
+          }
+          commands.append(getCommands(temp, null, "select"));
+        }
+      }
+
+      sb.setLength(0);
+
       for (int i = 0; i < ms.bondCount; i++)
         if (!models[bonds[i].atom1.mi].isModelKit)
           if (bonds[i].isHydrogen() || (bonds[i].order & Edge.BOND_NEW) != 0) {
@@ -360,38 +396,6 @@ public class StateCreator extends JmolStateCreator {
               .appendSB(sb);
       }
 
-      boolean loadUC = false;
-      if (ms.unitCells != null) {
-        boolean haveModulation = false;
-        for (int i = 0; i < modelCount; i++) {
-          SymmetryInterface symmetry = ms.getUnitCell(i);
-          if (symmetry == null)
-            continue;
-          sb.setLength(0);
-          if (symmetry.getState(ms, i, sb)) {
-            loadUC = true;
-            commands.append("  frame ").append(ms.getModelNumberDotted(i))
-                .appendSB(sb).append(";\n");
-          }
-          haveModulation |= (vwr.ms.getLastVibrationVector(i, T.modulation) >= 0);
-        }
-        if (loadUC)
-          vwr.shm.loadShape(JC.SHAPE_UCCAGE); // just in case
-        getShapeStatePriv(commands, isAll, JC.SHAPE_UCCAGE);
-        if (haveModulation) {
-          Map<String, BS> temp = new Hashtable<String, BS>();
-          int ivib;
-          for (int i = modelCount; --i >= 0;) {
-            if ((ivib = vwr.ms.getLastVibrationVector(i, T.modulation)) >= 0)
-              for (int j = models[i].firstAtomIndex; j <= ivib; j++) {
-                JmolModulationSet mset = ms.getModulation(j);
-                if (mset != null)
-                  BSUtil.setMapBitSet(temp, j, j, mset.getState());
-              }
-          }
-          commands.append(getCommands(temp, null, "select"));
-        }
-      }
       commands.append("  set fontScaling " + vwr.getBoolean(T.fontscaling)
           + ";\n");
       //      if (vwr.getBoolean(T.modelkitmode))
@@ -782,7 +786,7 @@ public class StateCreator extends JmolStateCreator {
       app(commands, "set navigationMode true");
     app(commands, vwr.ms.getBoundBoxCommand(false));
     app(commands, "center " + Escape.eP(tm.fixedRotationCenter));
-    commands.append(vwr.getOrientation(T.name, null, null).toString());
+    commands.append(vwr.getOrientation(T.name, null, null, null).toString());
 
     app(commands, moveToText);
 // stereo mode should not be in the state - just a display option
@@ -944,7 +948,7 @@ public class StateCreator extends JmolStateCreator {
   }
 
   private static void addTickInfo(SB sb, TickInfo tickInfo, boolean addFirst) {
-    sb.append(" ticks ").append(tickInfo.type).append(" ").append(
+    sb.append(" ticks ").appendC(tickInfo.type).append(" ").append(
         Escape.eP(tickInfo.ticks));
     boolean isUnitCell = (tickInfo.scale != null && Float
         .isNaN(tickInfo.scale.x));
@@ -1000,10 +1004,13 @@ public class StateCreator extends JmolStateCreator {
         sb.append(" " + m.property + " value " + (Double.isNaN(m.value) ? 0d : m.value))
          .append(" " + PT.esc(m.getString()));
       } else if (count == 2) {
-        String s = m.getStrFormatForState();
+        String s = m.getDistanceFormatForState();
         if (s != null)
           sb.append(" ").append(PT.esc(s));
       }
+      if (!isProperty) {
+        sb.append("; # " + shape.getInfoAsString(i));
+      }
       app(commands, sb.toString());
     }
     app(commands, Shape.getFontCommand("measures", font3d));
@@ -1163,9 +1170,12 @@ public class StateCreator extends JmolStateCreator {
       SB sb = new SB();
       sb.append("\n  set echo off;set echo \"%SCALE\" off;\n");
       getEchoState(sb, es.scaleObject);
+      boolean haveElementKey = false;
       for (Text t : es.objects.values()) {
-        getEchoState(sb, t);
+        haveElementKey |= getEchoState(sb, t);
       }
+      if (haveElementKey)
+        sb.append("  modelkit OFF;//to check for element keys\n");
       s = sb.toString();
       break;
     case JC.SHAPE_HALOS:
@@ -1344,12 +1354,12 @@ public class StateCreator extends JmolStateCreator {
     return s;
   }
 
-  private void getEchoState(SB sb, Text t) {
+  private boolean getEchoState(SB sb, Text t) {
     // ECHO "%SCALE" uses unformatted text
     boolean isScale = (t != null && t.barPixels > 0);
-    String text = (t == null ? null : isScale ? t.textUnformatted : t.text); 
+    String text = (t == null ? null : isScale ? t.textUnformatted : t.getStateText()); 
     if (text == null || !t.isEcho || t.target.equals("error"))
-      return;
+      return false;
     //set echo top left
     //set echo ID myecho x y
     //echo .....
@@ -1439,6 +1449,7 @@ public class StateCreator extends JmolStateCreator {
       sb.append("  set echo ID ").append(PT.esc(t.target))
           .append(" hidden;\n");
     }
+    return (t.target.startsWith(JC.MODELKIT_ELEMENT_KEY_ID));
   }
 
   @Override
@@ -1618,7 +1629,7 @@ public class StateCreator extends JmolStateCreator {
           else if (Float.isNaN(v.modScale))
             s.appendF(v.x).append(" ").appendF(v.y).append(" ").appendF(v.z);
           else
-            s.appendF(PT.FLOAT_MIN_SAFE).append(" ").appendF(PT.FLOAT_MIN_SAFE).append(" ").appendF(v.modScale);
+            s.appendF(JC.FLOAT_MIN_SAFE).append(" ").appendF(JC.FLOAT_MIN_SAFE).append(" ").appendF(v.modScale);
           break;
         case AtomCollection.TAINT_SITE:
           s.appendI(atoms[i].getAtomSite());
@@ -1702,12 +1713,8 @@ public class StateCreator extends JmolStateCreator {
     // called by actionManager
     if (!vwr.g.preserveState)
       return;
-    int modelIndex = (taintedAtom >= 0 ? vwr.ms.at[taintedAtom].mi
+    int modelIndex = (taintedAtom >= 0 && vwr.ms.at[taintedAtom] != null ? vwr.ms.at[taintedAtom].mi
         : vwr.ms.mc - 1);
-    //System.out.print("undoAction " + type + " " + taintedAtom + " modelkit?"
-    //    + modelSet.models[modelIndex].isModelkit());
-    //System.out.println(" " + type + " size=" + actionStates.size() + " "
-    //    + +actionStatesRedo.size());
     switch (type) {
     case T.redomove:
     case T.undomove:
