@@ -29,6 +29,7 @@ import java.util.Iterator;
 import java.util.Map;
 
 import org.jmol.api.JmolSceneGenerator;
+import org.jmol.api.SymmetryInterface;
 import javajs.util.BS;
 import javajs.util.Lst;
 
@@ -40,7 +41,10 @@ import org.jmol.script.T;
 
 import org.jmol.util.BSUtil;
 import org.jmol.util.Elements;
+import org.jmol.util.Escape;
 
+import javajs.util.BS;
+import javajs.util.Lst;
 import javajs.util.SB;
 
 import java.util.Arrays;
@@ -124,6 +128,7 @@ public class StateManager {
   private String lastState = "";
   private String lastShape = "";
   private String lastCoordinates = "";
+  private String lastUnitcell = "";
 
   StateManager(Viewer vwr) {
     this.vwr = vwr;
@@ -252,6 +257,36 @@ public class StateManager {
     return true;
   }
 
+  public void saveUnitCell(String saveName) {
+    if (saveName.equalsIgnoreCase("DELETE")) {
+      deleteSavedType("Unitcell_");
+      return;
+    }
+    saveName = lastUnitcell = "Unitcell_" + saveName;
+    SymmetryInterface uc = vwr.getCurrentUnitCell();
+    if (uc != null) {
+      String state = "UNITCELL " + Escape.e(uc.getUnitCellVectors());
+      saved.put(saveName, "unitcell reset;" + state);
+    }
+  }
+
+  public String getSavedUnitCell(String saveName) {
+    String name = (saveName.length() > 0 ? "Unitcell_" + saveName
+        : lastUnitcell);
+    String ucstate = (String) getNoCase(saved, name);
+    return ucstate;
+  }
+  
+  public boolean restoreUnitCell(String saveName) {
+    String ucstate = getSavedUnitCell(saveName);
+    if (ucstate == null) {
+      vwr.ms.setModelCagePts(-1, null, null);
+      return false;
+    }
+    vwr.runScript(ucstate);
+    return true;
+  }
+  
   public void saveState(String saveName) {
     if (saveName.equalsIgnoreCase("DELETE")) {
       deleteSavedType("State_");
@@ -288,7 +323,7 @@ public class StateManager {
    */
   public String popStack(int type) {
     Lst<String> stack = getStack(type);
-    String state = (stack.size() > 0 ? stack.remove(stack.size() - 1) : null);
+    String state = (stack.size() > 0 ? stack.removeItemAt(stack.size() - 1) : null);
     //System.out.println("STM pop " + stack.size() + " " + (state != null) + " " + (type == T.undo));
     if (state != null) {
       appendState(getStack(type == T.undo ? T.redo : T.undo));
