@@ -260,48 +260,12 @@ public class TransformManager {
     unTransformPoint(pt2, pt2);
     vwr.setInMotion(false);
     rotateAboutPointsInternal(null, pt2, pt1, 10 * speed, Float.NaN, false,
-        true, null, true, null, null, null, null);
+        true, null, true, null, null, null, null, false);
   }
 
-  //  final V3 arcBall0 = new V3();
-  //  final V3 arcBall1 = new V3();
-  //  final V3 arcBallAxis = new V3();
-  //  final M3 arcBall0Rotation = new M3();
-
-  //  void rotateArcBall(float x, float y, float factor) {
-  //    // radius is half the screen pixel count. 
-  //    float radius2 = (screenPixelCount >> 2) * screenPixelCount;
-  //    x -= fixedTranslation.x;
-  //    y -= fixedTranslation.y;
-  //    float z = radius2 - x * x - y * y;
-  //    z = (z < 0 ? -1 : 1) * (float) Math.sqrt(Math.abs(z));
-  //    if (factor == 0) {
-  //      // mouse down sets the initial rotation and point on the sphere
-  //      arcBall0Rotation.setM3(matrixRotate);
-  //      arcBall0.set(x, -y, z);
-  //      if (!Float.isNaN(z))
-  //        arcBall0.normalize();
-  //      return;
-  //    }
-  //    if (Float.isNaN(arcBall0.z) || Float.isNaN(z))
-  //      return;
-  //    arcBall1.set(x, -y, z);
-  //    arcBall1.normalize();
-  //    arcBallAxis.cross(arcBall0, arcBall1);
-  //    axisangleT.setVA(arcBallAxis, factor
-  //        * (float) Math.acos(arcBall0.dot(arcBall1)));
-  //    setRotation(arcBall0Rotation);
-  //    rotateAxisAngle2(axisangleT, null);
-  //  }
-
   protected void rotateXYBy(float degX, float degY, BS bsAtoms) {
     // from mouse action
-    //if (vwr.getTestFlag(2)) {
-    //  rotateXRadians(degY * JC.radiansPerDegree, bsAtoms);
-    //  rotateYRadians(degX * JC.radiansPerDegree, bsAtoms);
-    //} else {
     rotate3DBall(degX, degY, bsAtoms);
-    //}
   }
 
   void rotateZBy(int zDelta, int x, int y) {
@@ -311,13 +275,13 @@ public class TransformManager {
   }
 
   private void applyRotation(M3 mNew, boolean isInternal, BS bsAtoms,
-                             V3 translation, boolean translationOnly, M4 m4) {
+                             V3 translation, boolean translationOnly, M4 m4, boolean useModelKit) {
     if (bsAtoms == null) {
       matrixRotate.mul2(mNew, matrixRotate);
       return;
     }
     vwr.moveAtoms(m4, mNew, matrixRotate, translation, internalRotationCenter,
-        isInternal, bsAtoms, translationOnly);
+        isInternal, bsAtoms, translationOnly, useModelKit);
     if (translation != null) {
       internalRotationCenter.add(translation);
     }
@@ -334,23 +298,23 @@ public class TransformManager {
     // probably has to do with the fact that we are changing
     // the signs of both screen Y and screen Z in the end.
 
-    if (matrixTemp3.setAsBallRotation(JC.radiansPerDegree, -yDeg, -xDeg))
-      applyRotation(matrixTemp3, false, bsAtoms, null, false, null);
+    if (matrixTemp3.setAsBallRotation((float) JC.radiansPerDegree, -yDeg, -xDeg))
+      applyRotation(matrixTemp3, false, bsAtoms, null, false, null, false);
   }
 
   public synchronized void rotateXRadians(float angleRadians, BS bsAtoms) {
     applyRotation(matrixTemp3.setAsXRotation(angleRadians), false, bsAtoms,
-        null, false, null);
+        null, false, null, false);
   }
 
   public synchronized void rotateYRadians(float angleRadians, BS bsAtoms) {
     applyRotation(matrixTemp3.setAsYRotation(angleRadians), false, bsAtoms,
-        null, false, null);
+        null, false, null, false);
   }
 
   public synchronized void rotateZRadians(float angleRadians) {
     applyRotation(matrixTemp3.setAsZRotation(angleRadians), false, null, null,
-        false, null);
+        false, null, false);
   }
 
   public void rotateAxisAngle(V3 rotAxis, float radians) {
@@ -359,7 +323,7 @@ public class TransformManager {
   }
 
   private synchronized void rotateAxisAngle2(A4 axisAngle, BS bsAtoms) {
-    applyRotation(matrixTemp3.setAA(axisAngle), false, bsAtoms, null, false, null);
+    applyRotation(matrixTemp3.setAA(axisAngle), false, bsAtoms, null, false, null, false);
   }
 
   /*
@@ -396,7 +360,7 @@ public class TransformManager {
     rotationAxis.setT(rotAxis);
     rotationRate = degreesPerSecond;
     if (isSpin) {
-      fixedRotationAxis.setVA(rotAxis, degreesPerSecond * JC.radiansPerDegree);
+      fixedRotationAxis.setVA(rotAxis, (float) (degreesPerSecond * JC.radiansPerDegree));
       isSpinInternal = false;
       isSpinFixed = true;
       isSpinSelected = (bsAtoms != null);
@@ -404,7 +368,7 @@ public class TransformManager {
       // fixed spin -- we will wait
       return (endDegrees != Float.MAX_VALUE);
     }
-    float radians = endDegrees * JC.radiansPerDegree;
+    float radians = (float) (endDegrees * JC.radiansPerDegree);
     fixedRotationAxis.setVA(rotAxis, endDegrees);
     rotateAxisAngleRadiansFixed(radians, bsAtoms);
     return true;
@@ -445,7 +409,7 @@ public class TransformManager {
                                     float endDegrees, boolean isClockwise,
                                     boolean isSpin, BS bsAtoms,
                                     boolean isGesture, V3 translation,
-                                    Lst<P3> finalPoints, float[] dihedralList, M4 m4) {
+                                    Lst<P3> finalPoints, float[] dihedralList, M4 m4, boolean useModelKit) {
 
     // *THE* Viewer INTERNAL frame rotation entry point
 
@@ -483,8 +447,8 @@ public class TransformManager {
           if (translation != null)
             internalTranslation.scale(1f / nFrames);
         }
-        internalRotationAxis.setVA(axis, (Float.isNaN(rotationRate) ? 0
-            : rotationRate) * JC.radiansPerDegree);
+        internalRotationAxis.setVA(axis, (float) ((Float.isNaN(rotationRate) ? 0
+            : rotationRate) * JC.radiansPerDegree));
         isSpinInternal = true;
         isSpinFixed = false;
         isSpinSelected = isSelected;
@@ -495,14 +459,14 @@ public class TransformManager {
           isGesture);
       return !Float.isNaN(endDegrees);
     }
-    float radians = endDegrees * JC.radiansPerDegree;
+    float radians = (float) (endDegrees * JC.radiansPerDegree);
     internalRotationAxis.setVA(axis, radians);
-    rotateAxisAngleRadiansInternal(radians, bsAtoms, m4);
+    rotateAxisAngleRadiansInternal(radians, bsAtoms, m4, useModelKit);
     return false;
   }
 
   public synchronized void rotateAxisAngleRadiansInternal(float radians,
-                                                          BS bsAtoms, M4 m4) {
+                                                          BS bsAtoms, M4 m4, boolean useModelKit) {
 
     // final matrix rotation when spinning or just rotating
 
@@ -517,7 +481,7 @@ public class TransformManager {
     // NOW apply that rotation  
 
     applyRotation(matrixTemp3.setAA(axisangleT), true, bsAtoms,
-        internalTranslation, radians > 1e6f, m4);
+        internalTranslation, radians > 1e6f, m4, useModelKit);
     if (bsAtoms == null)
       getNewFixedRotationCenter();
   }
@@ -2234,7 +2198,7 @@ public class TransformManager {
   }
 
   public void setVibrationT(float t) {
-    vibrationT.x = t;
+    vibrationT.x = vibrationT.y = vibrationT.z = t;
     if (vibrationScale == 0)
       vibrationScale = vwr.g.vibrationScale;
   }
@@ -2293,7 +2257,7 @@ public class TransformManager {
 
   void setStereoDegrees(float stereoDegrees) {
     this.stereoDegrees = stereoDegrees;
-    stereoRadians = stereoDegrees * JC.radiansPerDegree;
+    stereoRadians = (float) (stereoDegrees * JC.radiansPerDegree);
   }
 
   boolean stereoFrame;
@@ -2431,7 +2395,7 @@ public class TransformManager {
         break;
       }
       vwr.moveAtoms(null, null, matrixRotate, v, internalRotationCenter,
-          false, bsAtoms, true);
+          false, bsAtoms, true, false);
       return;
     }
     this.bsSelectedAtoms = bsAtoms;
