@@ -91,6 +91,7 @@ import org.jmol.i18n.GT;
 import org.jmol.script.T;
 import org.jmol.util.Logger;
 import org.jmol.util.Parser;
+import org.jmol.viewer.JC;
 import org.jmol.viewer.Viewer;
 import org.openscience.jmol.app.HistoryFile;
 import org.openscience.jmol.app.Jmol;
@@ -152,6 +153,7 @@ public class JmolPanel extends JPanel
   protected StatusListener myStatusListener;
   protected SurfaceTool surfaceTool;
   protected MeasurementTable measurementTable;
+  protected JmolJME jmolJME;
 
   protected Map<String, Action> commands;
   protected Map<String, JMenuItem> menuItems;
@@ -159,6 +161,7 @@ public class JmolPanel extends JPanel
 
   // --- action implementations -----------------------------------
 
+  protected TwoDEditorAction twoDEditorAction = new TwoDEditorAction(this);
   protected ExportAction exportAction = new ExportAction();
   protected PovrayAction povrayAction = new PovrayAction();
   protected ToWebAction toWebAction = new ToWebAction();
@@ -203,6 +206,7 @@ public class JmolPanel extends JPanel
   protected static final String openpdbAction = "openpdb";
   protected static final String openmolAction = "openmol";
   protected static final String newAction = "new";
+  protected static final String twoDEditorActionProperty = "twoDEditor";
   protected static final String exportActionProperty = "export";
   protected static final String closeAction = "close";
   protected static final String exitAction = "exit";
@@ -297,7 +301,7 @@ public class JmolPanel extends JPanel
         (addAtomChooser ? new AtomSetChooserAction() : null),
         viewMeasurementTableAction,
         (allowGaussian ? new GaussianAction() : null), /*new NBOAction(),*/
-        new ResizeAction(), surfaceToolAction };
+        new ResizeAction(), surfaceToolAction, twoDEditorAction };
 
     List<Action> actions = new ArrayList<Action>();
     actions.addAll(Arrays.asList(defaultActions));
@@ -635,6 +639,8 @@ public class JmolPanel extends JPanel
 
   void dispose(JFrame f, boolean saveSize) {
     // Save window positions and status in the history
+    if (jmolJME != null)
+      jmolJME.dispose();
     if (webExport != null)
       WebExport.cleanUp();
     if (saveSize)
@@ -1409,10 +1415,7 @@ public class JmolPanel extends JPanel
 
     public OpenPdbAction() {
       super(openpdbAction);
-      script = "var x__id__ = _modelTitle; if (x__id__.length != 4) { x__id__ = '1crn'};x__id__ = prompt('"
-          + GT.$(
-              "Enter a four-digit PDB model ID or \"=\" and a three-digit ligand ID")
-          + "',x__id__);if (!x__id__) { quit }; load @{'=' + x__id__}";
+      script = JC.getMenuScript("openPDB");
     }
   }
 
@@ -1420,10 +1423,7 @@ public class JmolPanel extends JPanel
 
     public OpenMolAction() {
       super(openmolAction);
-      script = "var x__id__ = _smilesString; if (!x__id__) { x__id__ = 'tylenol'};x__id__ = prompt('"
-          + GT.$(
-              "Enter the name or identifier (SMILES, InChI, CAS) of a compound. Preface with \":\" to load from PubChem; otherwise Jmol will use the NCI/NIH Resolver.")
-          + "',x__id__);if (!x__id__) { quit }; load @{(x__id__[1]==':' ? x__id__ : '$' + x__id__)}";
+      script = JC.getMenuScript("openMOL");
     }
 
   }
@@ -1462,6 +1462,30 @@ public class JmolPanel extends JPanel
     }
   }
 
+  public class TwoDEditorAction extends AbstractAction {
+
+    private JmolPanel jmolPanel;
+
+    public TwoDEditorAction(JmolPanel jmolPanel) {
+      super(twoDEditorActionProperty);
+      this.jmolPanel = jmolPanel;
+    }
+
+    @Override
+    public void actionPerformed(ActionEvent e) {
+      if (jmolJME == null) {
+        jmolJME = (JmolJME) Interface.getInterface(
+            "org.openscience.jmol.app.jmolpanel.JmolJME", vwr,
+            "2dEditorAction");
+        jmolJME.setViewer(null, vwr, jmolPanel.getTopLevelAncestor());
+      }
+      jmolJME.setFrameVisible(true);
+      jmolJME.from3D();
+    }
+
+  }
+
+
   final static String[] imageChoices = { "JPEG", "PNG", "GIF", "PPM", "PDF" };
   final static String[] imageExtensions = { "jpg", "png", "gif", "ppm", "pdf" };
 
@@ -1731,6 +1755,8 @@ public class JmolPanel extends JPanel
         fullPathName = fullPathName.substring(0, pt);
       if (recentFiles != null)
         recentFiles.notifyFileOpen(fullPathName);
+      if (title.indexOf(JC.ADD_HYDROGEN_TITLE) >= 0)
+        title = "<modelkit>";
       frame.setTitle(title);
     }
     if (atomSetChooser == null && addAtomChooser) {
