@@ -212,17 +212,20 @@ public class AppConsole extends JmolConsole
 
   @Override
   public void sendConsoleMessage(String strStatus) {
-    if (strStatus == null) {
-      console.clearContent(null);
-      console.outputStatus("");
-    } else if (strStatus.indexOf("ERROR:") >= 0) {
-      console.outputError(strStatus);
-      setError(true);
-    } else {
-      console.outputStatus(strStatus);
-      isError = false;
+    try {
+      if (strStatus == null) {
+        console.clearContent(null);
+        console.outputStatus("");
+      } else if (strStatus.indexOf("ERROR:") >= 0) {
+        console.outputError(strStatus);
+        setError(true);
+      } else {
+        console.outputStatus(strStatus);
+        isError = false;
+      }
+    } catch (Throwable t) {
+      System.out.println("AppConsole error " + t);
     }
-    //system.out.println("AppConsole " + console.getText().length());
   }
 
   @Override
@@ -1191,6 +1194,11 @@ public class AppConsole extends JmolConsole
     }
   }
 
+  @Override
+  public void regainFocus() {
+    //console.requestFocus();
+  }
+
 }
 
 interface EnterListener {
